package com.qmth.distributed.print.business.entity;

import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.TableField;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.distributed.print.business.bean.params.GenericExamCardModelFourParams;
import com.qmth.distributed.print.business.enums.ExamCardStatusEnum;
import com.qmth.teachcloud.common.base.BaseEntity;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import java.io.Serializable;
import java.util.List;

/**
 * <p>
 * 模式4题卡
 * </p>
 *
 * @author wangliang
 * @since 2025-03-06
 */
@ApiModel(value = "ExamCardModelFour对象", description = "模式4题卡")
public class ExamCardModelFour extends BaseEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "学校id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long schoolId;

    @ApiModelProperty(value = "机构id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long orgId;

    @ApiModelProperty(value = "考试id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long examId;

    @ApiModelProperty(value = "课程ID（basic_course表ID）")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long courseId;

    @ApiModelProperty(value = "标题")
    private String title;

    @ApiModelProperty(value = "STAGE-暂存，SUBMIT-提交")
    private ExamCardStatusEnum status;

    @ApiModelProperty(value = "题卡工具制作题卡内容")
    private String content;

    @ApiModelProperty(value = "暂存内容（提交后置空）")
    @TableField(value = "stage_content", updateStrategy = FieldStrategy.IGNORED)
    private String stageContent;

    @ApiModelProperty(value = "html格式内容")
    private String htmlContent;

    @ApiModelProperty(value = "1正常，0禁用")
    private Boolean enable;

    @ApiModelProperty(value = "题卡规则Id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long cardRuleId;

    @ApiModelProperty(value = "题卡转换成的jpg文件信息")
    private String jpgAttachment;

    @ApiModelProperty(value = "题卡纸张大小（A3，8K）")
    private String pageSize;

    @ApiModelProperty(value = "备注")
    private String remark;

    @ApiModelProperty(value = "图片urls")
    @TableField(exist = false)
    private List<String> imageUrls;

    @ApiModelProperty(value = "课程代码")
    @TableField(exist = false)
    private String courseCode;

    @ApiModelProperty(value = "课程名称")
    @TableField(exist = false)
    private String courseName;

    @ApiModelProperty(value = "学期id")
    @JsonSerialize(using = ToStringSerializer.class)
    @TableField(exist = false)
    private Long semesterId;

    public ExamCardModelFour() {

    }

    public ExamCardModelFour(Long schoolId, Long orgId, Long userId) {
        this.insertInfo(userId);
        this.schoolId = schoolId;
        this.orgId = orgId;
        this.enable = true;
    }

    public void updateInformation(GenericExamCardModelFourParams params) {
        this.title = params.getTitle();
        this.cardRuleId = params.getCardRuleId();
        this.status = params.getStatus();
        this.remark = params.getRemark();
        this.pageSize = params.getPageSize();
        this.examId = params.getExamId();
        this.courseId = params.getCourseId();
    }

    public Long getSemesterId() {
        return semesterId;
    }

    public void setSemesterId(Long semesterId) {
        this.semesterId = semesterId;
    }

    public String getCourseCode() {
        return courseCode;
    }

    public void setCourseCode(String courseCode) {
        this.courseCode = courseCode;
    }

    public String getCourseName() {
        return courseName;
    }

    public void setCourseName(String courseName) {
        this.courseName = courseName;
    }

    public Long getOrgId() {
        return orgId;
    }

    public void setOrgId(Long orgId) {
        this.orgId = orgId;
    }

    public Long getSchoolId() {
        return schoolId;
    }

    public void setSchoolId(Long schoolId) {
        this.schoolId = schoolId;
    }

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public Long getCourseId() {
        return courseId;
    }

    public void setCourseId(Long courseId) {
        this.courseId = courseId;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public ExamCardStatusEnum getStatus() {
        return status;
    }

    public void setStatus(ExamCardStatusEnum status) {
        this.status = status;
    }

    public String getContent() {
        return content;
    }

    public void setContent(String content) {
        this.content = content;
    }

    public String getStageContent() {
        return stageContent;
    }

    public void setStageContent(String stageContent) {
        this.stageContent = stageContent;
    }

    public String getHtmlContent() {
        return htmlContent;
    }

    public void setHtmlContent(String htmlContent) {
        this.htmlContent = htmlContent;
    }

    public Boolean getEnable() {
        return enable;
    }

    public void setEnable(Boolean enable) {
        this.enable = enable;
    }

    public Long getCardRuleId() {
        return cardRuleId;
    }

    public void setCardRuleId(Long cardRuleId) {
        this.cardRuleId = cardRuleId;
    }

    public String getJpgAttachment() {
        return jpgAttachment;
    }

    public void setJpgAttachment(String jpgAttachment) {
        this.jpgAttachment = jpgAttachment;
    }

    public String getPageSize() {
        return pageSize;
    }

    public void setPageSize(String pageSize) {
        this.pageSize = pageSize;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public List<String> getImageUrls() {
        return imageUrls;
    }

    public void setImageUrls(List<String> imageUrls) {
        this.imageUrls = imageUrls;
    }
}
