package com.qmth.distributed.print.business.entity;

import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.teachcloud.common.base.BaseEntity;
import com.qmth.teachcloud.common.enums.CreatePdfTypeEnum;
import com.qmth.teachcloud.common.enums.TaskResultEnum;
import com.qmth.teachcloud.common.enums.TaskStatusEnum;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import java.io.Serializable;

/**
 * <p>
 * pdf生成日志表
 * </p>
 *
 * @author xf
 * @since 2024-02-06
 */
@TableName("t_b_task_pdf")
@ApiModel(value = "TBTaskPdf对象", description = "pdf生成日志表")
public class TBTaskPdf extends BaseEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "学校ID")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long schoolId;

    @ApiModelProperty(value = "学期ID")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long semesterId;

    @ApiModelProperty(value = "考试ID")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long examId;

    @ApiModelProperty(value = "印刷计划ID")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long printPlanId;

    @ApiModelProperty(value = "课程名称")
    private String courseNameCode;

    @ApiModelProperty(value = "试卷编号")
    private String paperNumber;
    @ApiModelProperty(value = "生成pdf类型")
    private CreatePdfTypeEnum createType;

    @ApiModelProperty(value = "任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成")
    private TaskStatusEnum status;

    @ApiModelProperty(value = "实时摘要信息")
    private String summary;

    @ApiModelProperty(value = "数据结果，SUCCESS：成功，ERROR：失败")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private TaskResultEnum result;

    @ApiModelProperty(value = "备注")
    private String remark;

    @ApiModelProperty(value = "时间差")
    @TableField(exist = false)
    private Integer diffTime;

    public TBTaskPdf() {
    }

    public TBTaskPdf(Long id, Long schoolId, Long semesterId, Long examId, Long printPlanId, String courseNameCode, String paperNumber, Long createId) {
        setId(id);
        this.schoolId = schoolId;
        this.semesterId = semesterId;
        this.examId = examId;
        this.printPlanId = printPlanId;
        this.courseNameCode = courseNameCode;
        this.paperNumber = paperNumber;
        setCreateId(createId);
        setCreateTime(System.currentTimeMillis());
        this.status = TaskStatusEnum.INIT;
        this.createType = CreatePdfTypeEnum.ALL;
    }

    public Integer getDiffTime() {
        return diffTime;
    }

    public void setDiffTime(Integer diffTime) {
        this.diffTime = diffTime;
    }

    public Long getSchoolId() {
        return schoolId;
    }

    public void setSchoolId(Long schoolId) {
        this.schoolId = schoolId;
    }

    public Long getSemesterId() {
        return semesterId;
    }

    public void setSemesterId(Long semesterId) {
        this.semesterId = semesterId;
    }

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public Long getPrintPlanId() {
        return printPlanId;
    }

    public void setPrintPlanId(Long printPlanId) {
        this.printPlanId = printPlanId;
    }

    public String getCourseNameCode() {
        return courseNameCode;
    }

    public void setCourseNameCode(String courseNameCode) {
        this.courseNameCode = courseNameCode;
    }

    public String getPaperNumber() {
        return paperNumber;
    }

    public void setPaperNumber(String paperNumber) {
        this.paperNumber = paperNumber;
    }

    public CreatePdfTypeEnum getCreateType() {
        return createType;
    }

    public void setCreateType(CreatePdfTypeEnum createType) {
        this.createType = createType;
    }

    public TaskStatusEnum getStatus() {
        return status;
    }

    public void setStatus(TaskStatusEnum status) {
        this.status = status;
    }

    public String getSummary() {
        return summary;
    }

    public void setSummary(String summary) {
        this.summary = summary;
    }

    public TaskResultEnum getResult() {
        return result;
    }

    public void setResult(TaskResultEnum result) {
        this.result = result;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    @Override
    public String toString() {
        return "TBTaskPdf{" +
                ", schoolId=" + schoolId +
                ", semesterId=" + semesterId +
                ", examId=" + examId +
                ", printPlanId=" + printPlanId +
                ", courseNameCode=" + courseNameCode +
                ", paperNumber=" + paperNumber +
                ", status=" + status +
                ", summary=" + summary +
                ", result=" + result +
                ", remark=" + remark +
                "}";
    }
}
