package com.qmth.distributed.print.business.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.distributed.print.business.bean.query.BasicExamStudentPageQuery;
import com.qmth.distributed.print.business.bean.query.BasicExamStudentQuery;
import com.qmth.distributed.print.business.bean.dto.DeleteBasicExamStudentStatusDto;
import com.qmth.distributed.print.business.bean.dto.ExamStudentDto;
import com.qmth.distributed.print.business.bean.params.BasicExamStudentParam;
import com.qmth.distributed.print.business.bean.result.BasicExamStudentResult;
import com.qmth.distributed.print.business.entity.BasicCardRule;
import com.qmth.distributed.print.business.entity.ExamStudent;
import com.qmth.teachcloud.common.bean.sync.TSyncDataStudent;
import com.qmth.teachcloud.common.entity.BasicExam;
import com.qmth.teachcloud.common.entity.BasicExamStudent;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.BasicExamStudentStatusEnum;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 考生字典表 服务类
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-02-22
 */
public interface BasicExamStudentService extends IService<BasicExamStudent> {

    IPage<BasicExamStudent> queryPage(BasicExamStudentPageQuery query);

    /**
     * 列表查询
     */
    List<BasicExamStudent> exportList(BasicExamStudentPageQuery query, SysUser sysUser, Long privilegeId);

    /**
     * 新增编辑
     *
     * @param basicExamStudentParam 参数
     * @param requestUser           用户
     * @return id
     */
    Long save(BasicExamStudentParam basicExamStudentParam, SysUser requestUser);

    /**
     * 导出
     */
    void exportLogic(BasicExamStudentPageQuery query) throws Exception;

    /**
     * 批量删除
     *
     * @param idList 要删除的考生字典id集合
     */
    void deleteBatch(List<Long> idList);

    /**
     * 构建考生字典对象
     *
     * @param basicExamStudentParam 考生字典参数
     * @param requestUser           请求人
     * @return 考生字典
     */
    BasicExamStudent editEntityHelp(BasicExamStudentParam basicExamStudentParam, BasicExam basicExam, SysUser requestUser);

    List<BasicExamStudent> listByExamIdAndCourseIdAndPaperNumber(Long examId, Long courseId, String paperNumber);

    List<BasicExamStudentResult> listByParams(BasicExamStudentQuery basicExamStudentQuery);

    boolean updateStatus(Long id, BasicExamStudentStatusEnum status);

    void updateTeacherIdById(List<BasicExamStudent> basicExamStudentList);

    void updatePaperNumberAndPaperTypeById(List<ExamStudent> examStudentList, BasicCardRule basicCardRule, Boolean openAb);

    void updateMarkData(Long examId, List<BasicExamStudent> basicExamStudentList, Long createId);

    List<DeleteBasicExamStudentStatusDto> getStudentMarkStatus(List<Long> idList);

    /**
     * 查询班级和老师信息
     *
     * @param examId
     * @param courseId
     * @param paperNumber
     * @return
     */
    ExamStudentDto queryExamStudent(Long examId, Long courseId, String paperNumber);

    List<BasicExamStudent> listByExamIdAndCourseIdAndTeachClassName(Long examId, Long courseId, String teachClassName);

    void clearPaperNumberAndPaperTypeById(List<Long> ids);

    void clearPaperTypeByExamIdAndPaperNumber(Long examId, String paperNumber);
    void clearPaperNumberAndPaperTypeByExamIdAndPaperNumber(Long examId, String paperNumber);

    Map<String, Object> saveBasicExamStudentFormSync(Long schoolId, Long examId, List<TSyncDataStudent> examStudentDataVo);

    String listByTeacherByExamIdAndPaperNumber(Long examId, String paperNumber);

    BasicExamStudent findByExamIdAndCourseIdAndStudentCode(Long examId, Long courseId, String studentCode);
}