package com.qmth.distributed.print.business.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.distributed.print.business.entity.BasicMessage;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.teachcloud.common.bean.params.ApproveUserResult;
import com.qmth.teachcloud.common.enums.EnumResult;
import com.qmth.teachcloud.common.enums.MessageEnum;

import java.util.List;

/**
 * @Description: 短信消息发送
 * @Author: CaoZixuan
 * @Date: 2021-04-28
 */
public interface BasicMessageService extends IService<BasicMessage> {

    void saveMessageSendLog(Long schoolId, Long userId, String userName, String mobileNumber, String variableParams, MessageEnum messageType);

    /**
     * 发送给命题老师-命题待办提醒
     *
     * @param examTasks
     */
    void sendNoticeTaskCreate(ExamTask... examTasks);

    /**
     * 发送给考务老师-审核待办提醒、修改审核待办提醒
     *
     * @param examTask
     * @param messageType
     */
    void sendNoticeTaskAuditCreateOrReview(ExamTask examTask, MessageEnum messageType);

    /**
     * 发送到期预警、逾期提醒
     *
     * @param messageType
     * @param userId
     * @param ids
     */
    void sendNoticeExpireOrOverdue(Long schoolId, MessageEnum messageType, Long userId, List<String> ids);

    /**
     * 重发失败短信
     */
    void resendSmsTask();

    /**
     * 新流程，发送审核提醒短信
     *
     * @param examTask
     * @param messageType
     */
    void sendNoticeTaskAuditFlow(ExamTask examTask, List<ApproveUserResult> users, MessageEnum messageType);

    /**
     * 发送审核抄送短信
     *
     * @param examTask
     * @param users
     * @param auditUserNames
     * @param messageType
     */
    void sendNoticeAuditCopyUser(ExamTask examTask, List<ApproveUserResult> users, String auditUserNames, MessageEnum messageType);

    /**
     * 查询
     *
     * @param messageType
     * @param sendStatus
     * @param mobileNumber
     * @param pageNumber
     * @param pageSize
     * @return
     */
    IPage<BasicMessage> listByMessageType(MessageEnum messageType, Boolean sendStatus, String mobileNumber, Integer pageNumber, Integer pageSize);

    /**
     * 查询短信类型
     *
     * @return
     */
    List<EnumResult> listMessageTypes();

    /**
     * 短信失败重发
     *
     * @param id
     */
    void resendSmsTaskById(Long id);

    /**
     * 保存message和备注
     *
     * @param schoolId
     * @param userId
     * @param userName
     * @param mobileNumber
     * @param paperNumber
     * @param courseCode
     * @param variableParams
     * @param messageType
     * @param createId
     * @param remark
     */
    void saveMessageSendLogRemark(Long schoolId, Long userId, String userName, String mobileNumber, String paperNumber, String courseCode, String variableParams, MessageEnum messageType, Long createId, String remark);

    /**
     * 发送阅卷待办短信
     *
     * @param examId
     * @param courseId
     * @param paperNumber
     */
    void smsMarkTask(Long examId, Long courseId, String paperNumber);
}