package com.qmth.distributed.print.business.service;

import com.qmth.distributed.print.business.bean.dto.PdfDto;
import com.qmth.distributed.print.business.bean.dto.PdfPackageDto;
import com.qmth.distributed.print.business.bean.dto.PdfSignDto;
import com.qmth.distributed.print.business.bean.excel.ExcelField;
import com.qmth.distributed.print.business.entity.BasicTemplate;
import com.qmth.distributed.print.business.entity.ExamDetail;
import com.qmth.teachcloud.common.bean.vo.PrintPathVo;
import com.qmth.teachcloud.common.entity.BasicAttachment;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.PageSizeEnum;
import com.qmth.teachcloud.common.enums.TaskTypeEnum;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @Description: 公共服务service
 * @Author: wangliang
 * @Date: 2021/3/26
 */
public interface PrintCommonService {

    void saveAttachmentSignPdf(PdfSignDto pdfFillDto, ExamDetail examDetail, Integer printCount, BasicTemplate basicTemplate);

    BasicAttachment saveAttachmentPdfFromHtml(String fileName, String htmlContent, Long userId, List<PdfDto> localFileList, List<File> fileTempList, String pageSize) throws IOException;

    BasicAttachment saveAttachmentPdfFromHtml(String fileName, File file, Long userId, List<PdfDto> localFileList, List<File> fileTempList, PageSizeEnum pageSizeEnum) throws IOException;

    /**
     * 保存html附件和该html转成pdf的附件
     *
     * @param fileName    文件名称
     * @param htmlContent html内容
     * @param userId      创建人
     * @return 附件对象
     */
    BasicAttachment saveAttachmentHtmlAndPdf(String fileName, String htmlContent, Long userId) throws IOException;

    /**
     * 保存附件公用
     *
     * @param file
     * @param md5
     * @param type
     * @param objId
     * @return
     */
    BasicAttachment saveAttachmentCommon(MultipartFile file, String md5, UploadFileEnum type, Long objId);

    /**
     * 保存附件
     *
     * @param file
     * @param md5
     * @param type
     * @return
     */
    BasicAttachment saveAttachment(MultipartFile file, String md5, UploadFileEnum type);

    /**
     * 科目查询
     *
     * @param param
     * @param printPlanIdList
     * @param teachingRoomId
     * @return
     */
    List<BasicCourse> list(String param, List<Long> printPlanIdList, Long teachingRoomId);

    /**
     * 保存任务
     *
     * @param file
     * @param taskTypeEnum
     * @return
     */
    Map<String, Object> saveTask(MultipartFile file, TaskTypeEnum taskTypeEnum);

    /**
     * 保存任务
     *
     * @param file
     * @param taskTypeEnum
     * @return
     */
    Map<String, Object> saveTask(MultipartFile file, Long examId, TaskTypeEnum taskTypeEnum);

    /**
     * 保存任务
     *
     * @param printPlanId
     * @param file
     * @param taskTypeEnum
     * @return
     */
    Map<String, Object> saveTask(Long printPlanId, MultipartFile file, TaskTypeEnum taskTypeEnum);

    /**
     * 保存任务(导出)
     *
     * @param taskTypeEnum
     * @return
     */
    Map<String, Object> saveTask(TaskTypeEnum taskTypeEnum);

    /**
     * 保存任务
     *
     * @param taskTypeEnum
     * @return
     */
    Map<String, Object> saveTask(Long printPlanId, TaskTypeEnum taskTypeEnum);

    /**
     * 校验课程关联考场是否提交打印
     *
     * @param schoolId
     * @param printPlanId
     * @param examId
     * @param courseId
     * @param paperNumber
     * @return
     */
    Boolean checkExamDetailStatus(Long schoolId, Long printPlanId, Long examId, Long courseId, String paperNumber);

    /**
     * 生成唯一的试卷编号
     */
    String autoCreatePaperNumber(Long examId);

    /**
     * 校验是否可以提交
     */
    void checkExamDataAndCreatePdfTask(Long schoolId, Long examId, Long courseId, String paperNumber, SysUser user) throws IOException;

    void checkDataMakeup(Long schoolId, ExamDetail examDetail, Long courseId, String paperNumber, SysUser user);

    /**
     * 保存任务附件(导出)
     *
     * @param fos
     * @param oss
     * @return
     */
    String saveTaskAttachment(ByteArrayOutputStream fos, boolean oss) throws IOException;

    /**
     * 提交初始化数据后更新批次状态
     *
     * @param schoolId    学校ID
     * @param paperNumber 试卷编号
     * @param examId      考试id
     * @param paperType   试卷类型
     */
    public void updateGradeBatchStatus(Long schoolId, Long examId, String paperNumber, String paperType);

    void saveAttachmentPackagePdf(PdfPackageDto pdfPackageDto, ExamDetail examDetail, Integer printCount, BasicTemplate basicTemplate);

    String parseAttachmentPath(String attachmentPath, PrintPathVo printPathVo);

    /**
     * 成绩管理导入excel校验
     */
    public MarkPaper scoreImportExcelValid(MultipartFile file, Long cultureProgramId, String paperNumber, Long courseId) throws IOException;

    /**
     * 获取阅卷数据
     *
     * @param examId
     * @param paperNumber
     * @param courseId
     * @return
     */
    public MarkPaper getMarkPaper(Long examId, String paperNumber, Long courseId);

    /**
     * 成绩管理动态生成excel
     *
     * @param describe
     * @param sheetName
     * @param fileName
     * @throws IOException
     */
    public void scoreDownLoadExaminationTemplate(String describe, String sheetName, String fileName, List<Set<ExcelField>> excelFieldRowList) throws IOException;
}
