package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.distributed.print.business.bean.dto.ExamTaskDetailDto;
import com.qmth.distributed.print.business.entity.ExamCard;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.entity.ExamTaskDetail;
import com.qmth.distributed.print.business.mapper.ExamCardMapper;
import com.qmth.distributed.print.business.mapper.ExamTaskDetailMapper;
import com.qmth.distributed.print.business.mapper.ExamTaskMapper;
import com.qmth.distributed.print.business.service.DownloadService;
import com.qmth.distributed.print.business.service.ExamTaskDetailService;
import com.qmth.distributed.print.business.service.ExamTaskService;
import com.qmth.distributed.print.business.util.CreatePdfUtil;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.vo.PaperInfoVo;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicAttachment;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.PageSizeEnum;
import com.qmth.teachcloud.common.mapper.BasicAttachmentMapper;
import com.qmth.teachcloud.common.service.AttachmentCommonService;
import com.qmth.teachcloud.common.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.service.SysUserService;
import com.qmth.teachcloud.common.util.Base64Util;
import com.qmth.teachcloud.common.util.ConvertUtil;
import com.qmth.distributed.print.business.util.ExamTaskUtil;
import com.qmth.teachcloud.common.util.HtmlToPdfUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.FileCopyUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class DownloadServiceImpl implements DownloadService {
    private final static Logger log = LoggerFactory.getLogger(DownloadServiceImpl.class);

    @Resource
    ExamTaskMapper examTaskMapper;
    @Resource
    SysUserService sysUserService;

    @Resource
    BasicAttachmentMapper basicAttachmentMapper;
    @Resource
    ExamTaskService examTaskService;
    @Resource
    private ExamTaskDetailService examTaskDetailService;

    @Resource
    ExamCardMapper examCardMapper;
    @Resource
    AttachmentCommonService attachmentCommonService;

    @Resource
    CreatePdfUtil createPdfUtil;
    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;

    @Override
    public IPage<ExamTaskDetailDto> page(Long examId, Long courseId, String paperNumber, Long orgId, String userParam, Integer pageNumber, Integer pageSize) {
        Page<ExamTaskDetailDto> page = new Page<>(pageNumber, pageSize);
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(requestUser.getSchoolId(), requestUser.getId(), ServletUtil.getRequest().getServletPath());
        IPage<ExamTaskDetailDto> examTaskDetailDtoIPage = examTaskMapper.listExamTaskDetailDownload(page, examId, courseId, paperNumber, orgId, userParam, dpr);
        for (ExamTaskDetailDto record : examTaskDetailDtoIPage.getRecords()) {
            List<ExamTaskDetail> examTaskDetailList = examTaskDetailService.listByExamTaskId(record.getId());
            record.setPaperType(examTaskDetailList.stream().map(m -> m.getSerialNumber().toString()).collect(Collectors.joining(",")));
            record.setExposedPaperType(examTaskDetailList.stream().filter(ExamTaskDetail::getExposed).map(m -> m.getSerialNumber().toString()).collect(Collectors.joining(",")));
            record.setUnexposedPaperType(examTaskDetailList.stream().filter(m -> !m.getExposed()).map(m -> m.getSerialNumber().toString()).collect(Collectors.joining(",")));
        }
        return examTaskDetailDtoIPage;
    }

    @Override
    public void paperDownload(HttpServletResponse response, Long id) {
        ExamTask examTask = examTaskService.getByIdIncludeCourse(id);
        List<ExamTaskDetail> examTaskDetailList = examTaskDetailService.listByExamTaskId(id);
        if (CollectionUtils.isEmpty(examTaskDetailList)) {
            throw ExceptionResultEnum.ERROR.exception("未找到试卷");
        }
        List<ExamTaskDetail> examTaskDetails = examTaskDetailList.stream().filter(m -> m.getExposed() == null || !m.getExposed()).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(examTaskDetails)) {
            throw ExceptionResultEnum.ERROR.exception("没有未曝光试卷");
        }

        // 收集处理试卷和题卡
        // 本地存储目录
        File fileTemp = null;
        String rootPath = null;
        try {
            fileTemp = SystemConstant.getFileTempVar(SystemConstant.ZIP_PREFIX);
            rootPath = fileTemp.getParent() + File.separator + System.currentTimeMillis();
            List<File> fileList = new ArrayList<>();
            for (ExamTaskDetail examTaskDetail : examTaskDetails) {
                List<PaperInfoVo> paperInfoDtoList = ExamTaskUtil.parsePaperAttachmentPath(examTaskDetail.getPaperAttachmentIds());
                for (PaperInfoVo paperInfo : paperInfoDtoList) {
                    String paperType = paperInfo.getName();
                    // 试卷附件id
                    String attachmentId = paperInfo.getAttachmentId();
                    if (StringUtils.isNotBlank(attachmentId)) {
                        // 试卷
                        BasicAttachment attachment = basicAttachmentMapper.selectById(attachmentId);
                        Optional.ofNullable(attachment).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未查询到试卷"));

                        String paperPath = rootPath + File.separator + examTask.getCourseName().replaceAll(" ", "") + SystemConstant.HYPHEN + examTask.getCourseCode() + File.separator + examTask.getPaperNumber();
                        String fileName = "卷" + examTaskDetail.getSerialNumber() + SystemConstant.HYPHEN + "试卷" + SystemConstant.HYPHEN + attachment.getName() + SystemConstant.HYPHEN + paperType + attachment.getType();
                        fileList.add(attachmentCommonService.downloadFile(paperPath, fileName, attachment));
                    }

                    // 题卡id
                    String cardId = paperInfo.getCardId();
                    // 题卡
                    if (StringUtils.isNotBlank(cardId)) {
                        ExamCard examCard = examCardMapper.selectById(cardId);
                        Optional.ofNullable(examCard).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未查询到题卡"));

                        String cardPath = rootPath + File.separator + examTask.getCourseName().replaceAll(" ", "") + SystemConstant.HYPHEN + examTask.getCourseCode() + File.separator + examTask.getPaperNumber();
                        String cardHtmlPath = cardPath + File.separator + "卷" + examTaskDetail.getSerialNumber() + SystemConstant.HYPHEN + "题卡" + SystemConstant.HYPHEN + examTask.getCourseName() + SystemConstant.HYPHEN + paperType + SystemConstant.HTML_PREFIX;
                        String cardPdfPath = cardPath + File.separator + "卷" + examTaskDetail.getSerialNumber() + SystemConstant.HYPHEN + "题卡" + SystemConstant.HYPHEN + examTask.getCourseName() + SystemConstant.HYPHEN + paperType + SystemConstant.PDF_PREFIX;

                        // html
                        File htmlFile = new File(cardHtmlPath);
                        if (!htmlFile.exists()) {
                            htmlFile.getParentFile().mkdirs();
                            htmlFile.createNewFile();
                        }
                        String htmlContent = createPdfUtil.replaceBlankHtmlContent(examCard.getHtmlContent(), examCard.getCourseId());
                        // 生成html文件
                        FileCopyUtils.copy(htmlContent.getBytes(StandardCharsets.UTF_8), htmlFile);
                        fileList.add(htmlFile);
                        // 转pdf文件
                        File pdfFile = new File(cardPdfPath);
                        if (!pdfFile.exists()) {
                            pdfFile.getParentFile().mkdirs();
                            pdfFile.createNewFile();
                        }
                        HtmlToPdfUtil.convert(cardHtmlPath, cardPdfPath, "8K".equals(examCard.getPageSize()) ? PageSizeEnum.K8 : PageSizeEnum.valueOf(examCard.getPageSize()));
                        fileList.add(pdfFile);
                    }
                }
            }


            if (fileList.size() == 0) {
                throw ExceptionResultEnum.ERROR.exception("没有可导出文件");
            }
            String zipName = examTask.getCourseName() + "未曝光试卷" + SystemConstant.ZIP_PREFIX;
            SysUser sysUser = sysUserService.getById(ServletUtil.getRequestUserId());
            attachmentCommonService.zipFiles(response, rootPath, zipName, fileList, new String(Base64Util.decode(sysUser.getPassword())));
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
        } finally {
            if (Objects.nonNull(fileTemp)) {
                fileTemp.delete();
            }
            ConvertUtil.delFolder(rootPath);
        }
    }

    @Override
    public List<ExamTaskDetailDto> listExamQuery(Long semesterId, Long examId, Long orgId, String courseName, Set<Long> idSet, DataPermissionRule dpr) {
        return examTaskMapper.listExamTaskDetailDownloadByQuery(semesterId, examId, courseName, orgId, idSet, dpr);
    }

}
