package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.distributed.print.business.bean.dto.initMarkData.ExamDetailCourseInitMarkDto;
import com.qmth.distributed.print.business.bean.params.GenericExamCardModelFourParams;
import com.qmth.distributed.print.business.entity.BasicCardRule;
import com.qmth.distributed.print.business.entity.ExamCardModelFour;
import com.qmth.distributed.print.business.enums.CardTypeEnum;
import com.qmth.distributed.print.business.enums.ExamCardStatusEnum;
import com.qmth.distributed.print.business.mapper.ExamCardModelFourMapper;
import com.qmth.distributed.print.business.service.BasicCardRuleService;
import com.qmth.distributed.print.business.service.ExamCardModelFourService;
import com.qmth.distributed.print.business.service.ExamCardService;
import com.qmth.distributed.print.business.service.PrintFinishService;
import com.qmth.distributed.print.business.util.CreatePdfUtil;
import com.qmth.distributed.print.business.util.HtmlToJpgUtil;
import com.qmth.distributed.print.business.util.PdfUtil;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.marking.ConvertJpgStorage;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicAttachment;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExamModelEnum;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FieldUniqueEnum;
import com.qmth.teachcloud.common.enums.PageSizeEnum;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.util.*;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.entity.ScanAnswerCard;
import com.qmth.teachcloud.mark.enums.CardSource;
import com.qmth.teachcloud.mark.enums.LockType;
import com.qmth.teachcloud.mark.service.*;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.util.*;

/**
 * <p>
 * 模式4题卡 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2025-03-06
 */
@Service
public class ExamCardModelFourServiceImpl extends ServiceImpl<ExamCardModelFourMapper, ExamCardModelFour> implements ExamCardModelFourService {

    @Resource
    ConcurrentService concurrentService;

    @Resource
    BasicCardRuleService basicCardRuleService;

    @Resource
    BasicCourseService basicCourseService;

    @Resource
    CreatePdfUtil createPdfUtil;

    @Resource
    HtmlToJpgUtil htmlToJpgUtil;

    @Resource
    BasicRoleDataPermissionService basicRoleDataPermissionService;

    @Resource
    TeachcloudCommonService teachcloudCommonService;

    @Resource
    BasicAttachmentService basicAttachmentService;

    @Resource
    FileUploadService fileUploadService;

    @Resource
    SysUserService sysUserService;

    @Resource
    PrintFinishService printFinishService;

    @Resource
    MarkPaperService markPaperService;

    @Resource
    ScanAnswerCardService scanAnswerCardService;
    @Resource
    ScanAnswerCardQuestionService scanAnswerCardQuestionService;

    @Resource
    MarkQuestionService markQuestionService;
    @Resource
    MarkQuestionAnswerService markQuestionAnswerService;

    /**
     * 新增/修改模式4题卡
     *
     * @param params
     * @return
     */
    @Override
    @Transactional
    public Map<String, String> saveGeneric(GenericExamCardModelFourParams params) {
        Objects.requireNonNull(params, "题卡信息不能为空");
        Objects.requireNonNull(params.getExamId(), "考试id不能为空");
        Objects.requireNonNull(params.getCourseId(), "课程id不能为空");
        Objects.requireNonNull(params.getTitle(), "题卡标题不能为空");
        Objects.requireNonNull(params.getContentFile(), "题卡内容附件不能为空");
        Objects.requireNonNull(params.getContentMd5(), "题卡内容附件md5不能为空");
        Objects.requireNonNull(params.getHtmlContentFile(), "题卡html附件不能为空");
        Objects.requireNonNull(params.getHtmlContentMd5(), "题卡html附件md5不能为空");
        Objects.requireNonNull(params.getCardRuleId(), "题卡规则id不能为空");
        Objects.requireNonNull(params.getStatus(), "提交状态不能为空");
        Objects.requireNonNull(params.getPageSize(), "题卡纸张大小不能为空");

        Map<String, String> map = null;
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        ExamCardModelFour examCardModelFour = null;
        concurrentService.getReadWriteLock(LockType.CUSTOM_MODEL_FOUR_CARD_SAVE + "-" + params.getExamId() + CardTypeEnum.CUSTOM + params.getTitle()).writeLock().tryLock();
        try {
            QueryWrapper<ExamCardModelFour> checkTitleQueryWrapper = new QueryWrapper<>();
            checkTitleQueryWrapper.lambda().eq(ExamCardModelFour::getExamId, params.getExamId())
                    .eq(ExamCardModelFour::getTitle, params.getTitle().trim());
            ExamCardModelFour checkTitleExamCardModelFourDb = this.getOne(checkTitleQueryWrapper);

            // 新增
            if (Objects.isNull(params.getId())) {
                if (Objects.nonNull(checkTitleExamCardModelFourDb)) {
                    throw ExceptionResultEnum.ERROR.exception("题卡名称已使用");
                }
                examCardModelFour = new ExamCardModelFour(schoolId, sysUser.getOrgId(), sysUser.getId());
            }
            // 修改
            else {
                examCardModelFour = this.getById(params.getId());
                if (Objects.nonNull(checkTitleExamCardModelFourDb) && params.getId().longValue() != examCardModelFour.getId().longValue()) {
                    throw ExceptionResultEnum.ERROR.exception("题卡名称重复");
                }
                int count = this.baseMapper.countByCardNumber(examCardModelFour.getId());
                if (count > 0) {
                    throw ExceptionResultEnum.ERROR.exception("题卡已扫描不能修改");
                }
                examCardModelFour.updateInfo(sysUser.getId());
            }

            examCardModelFour.updateInformation(params);
            try {
                // content文件
                String contentMd5 = DigestUtils.md5Hex(params.getContentFile().getBytes());
                if (!Objects.equals(contentMd5, params.getContentMd5())) {
                    throw ExceptionResultEnum.MD5_EQUALS_FALSE.exception();
                }
                String content = SystemConstant.readContent(params.getContentFile().getInputStream());

                if (ExamCardStatusEnum.SUBMIT.equals(params.getStatus())) {
                    examCardModelFour.setContent(content);
                    examCardModelFour.setStageContent(null);

                    // content文件
                    String htmlMd5 = DigestUtils.md5Hex(params.getHtmlContentFile().getBytes());
                    if (!Objects.equals(htmlMd5, params.getHtmlContentMd5())) {
                        throw ExceptionResultEnum.MD5_EQUALS_FALSE.exception();
                    }
                    String htmlContent = SystemConstant.readContent(params.getHtmlContentFile().getInputStream());
                    examCardModelFour.setHtmlContent(htmlContent);
                } else {
                    examCardModelFour.setStageContent(content);
                }
            } catch (IOException e) {
                throw ExceptionResultEnum.ERROR.exception("题卡内容文件解析失败");
            }

            if (ExamCardStatusEnum.SUBMIT.equals(params.getStatus())) {
                List<MarkPaper> markPaperList = markPaperService.listByExamIdAndCourseIdForModel4(examCardModelFour.getExamId(), examCardModelFour.getCourseId());
                if (CollectionUtils.isNotEmpty(markPaperList) && markPaperList.size() == 1) {
                    saveQuestionAndAnswerCard(examCardModelFour, markPaperList.get(0), sysUser.getId());
                }
            }
            this.saveOrUpdate(examCardModelFour);
            map = new HashMap<>();
            map.put("id", String.valueOf(examCardModelFour.getId()));
            map.put("title", examCardModelFour.getTitle());
            return map;
        } catch (Exception e) {
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            concurrentService.getReadWriteLock(LockType.CUSTOM_MODEL_FOUR_CARD_SAVE + "-" + params.getExamId() + CardTypeEnum.CUSTOM + params.getTitle()).writeLock().unlock();
        }
        return map;
    }

    /**
     * 创建jpg图片
     *
     * @param examCardModelFour
     * @return
     */
    @Override
    public ExamCardModelFour createJpgImage(ExamCardModelFour examCardModelFour) {
        BasicCardRule basicCardRule = basicCardRuleService.getById(examCardModelFour.getCardRuleId());
        BasicCourse basicCourse = examCardModelFour.getCourseId() != null ? basicCourseService.getById(examCardModelFour.getCourseId()) : null;
        String replaceBackupCardHtmlContent = createPdfUtil.replaceBackupCardHtmlParam(basicCourse != null ? basicCourse.getCode() : null, examCardModelFour, null, basicCardRule, null);
        File pdfFile = PdfUtil.htmlToPdf(examCardModelFour.getPageSize(), replaceBackupCardHtmlContent);
        try {
            List<ConvertJpgStorage> convertJpgStorageList = htmlToJpgUtil.convertPdfToJpg(examCardModelFour.getId(), pdfFile, examCardModelFour.getCreateId());
            if (CollectionUtils.isNotEmpty(convertJpgStorageList) && convertJpgStorageList.size() > 0) {
                examCardModelFour.setJpgAttachment(JSON.toJSONString(convertJpgStorageList));
            } else {
                examCardModelFour.setJpgAttachment(null);
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            if (pdfFile.exists()) {
                FileUtil.deleteFile(pdfFile);
            }
        }
        return examCardModelFour;
    }

    /**
     * 模式4答题卡列表（分页）
     *
     * @param semesterId
     * @param examId
     * @param courseId
     * @param paperNumber
     * @param openCollegeId
     * @param title
     * @return
     */
    @Override
    public IPage<ExamCardModelFour> listPage(IPage<Map> iPage, Long semesterId, Long examId, Long courseId, String paperNumber, Long openCollegeId, String title) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(), ServletUtil.getRequest().getServletPath());
        IPage<ExamCardModelFour> cardPageDtoIPage = this.baseMapper.listPage(iPage, schoolId, semesterId, examId, courseId, openCollegeId, title, paperNumber, dpr);
        for (ExamCardModelFour record : cardPageDtoIPage.getRecords()) {
            if (StringUtils.isNotBlank(record.getJpgAttachment())) {
                List<JSONObject> mapList = JSON.parseArray(record.getJpgAttachment(), JSONObject.class);
                List<String> imageUrls = new ArrayList<>();
                // 组装图片访问url
                for (JSONObject object : mapList) {
                    imageUrls.add(teachcloudCommonService.filePreview(object.getLong("attachmentId")));
                }
                record.setImageUrls(imageUrls);
            }
        }
        return cardPageDtoIPage;
    }

    /**
     * 根据id获取模式4题卡
     *
     * @param id
     * @return
     */
    @Override
    public ExamCardModelFour getOne(Long id) {
        ExamCardModelFour examCardModelFour = this.getById(id);
        if (Objects.nonNull(examCardModelFour) && Objects.nonNull(examCardModelFour.getCourseId())) {
            BasicCourse basicCourse = basicCourseService.getById(examCardModelFour.getCourseId());
            examCardModelFour.setCourseCode(Objects.nonNull(basicCourse) ? basicCourse.getCode() : null);
            examCardModelFour.setCourseName(Objects.nonNull(basicCourse) ? basicCourse.getName() : null);
        }
        return examCardModelFour;
    }

    /**
     * 根据题卡id删除
     *
     * @param id
     * @return
     */
    @Override
    @Transactional
    public Boolean deleteGeneric(Long id) {
        int count = this.baseMapper.countByCardNumber(id);
        if (count > 0) {
            throw ExceptionResultEnum.ERROR.exception("已扫描的题卡不允许删除");
        }
        ExamCardModelFour examCardModelFour = this.getById(id);
        List<MarkPaper> markPaperList = markPaperService.listByExamIdAndCourseIdForModel4(examCardModelFour.getExamId(), examCardModelFour.getCourseId());
        if (CollectionUtils.isNotEmpty(markPaperList)) {
            MarkPaper markPaper = markPaperList.get(0);
            markQuestionService.deleteByExamIdAndPaperNumber(markPaper.getExamId(), markPaper.getPaperNumber());
            markQuestionAnswerService.deleteByExamIdAndPaperNumber(markPaper.getExamId(), markPaper.getPaperNumber());
            List<ScanAnswerCard> scanAnswerCardList = scanAnswerCardService.findByExamAndCardIdAndCardSource(markPaper.getExamId(), id, CardSource.WEB);
            if (CollectionUtils.isNotEmpty(scanAnswerCardList)) {
                for (ScanAnswerCard scanAnswerCard : scanAnswerCardList) {
                    scanAnswerCardService.removeById(scanAnswerCard.getId());
                    scanAnswerCardQuestionService.deleteByExamIdAndPaperNumberAndCardNumber(markPaper.getExamId(), markPaper.getPaperNumber(), scanAnswerCard.getNumber());
                }
            }
        }
        return this.removeById(id);
    }

    /**
     * 根据题卡id生成图片
     *
     * @param id
     */
    @Override
    @Transactional
    public void convertImage(Long id) {
        ExamCardModelFour examCardModelFour = this.getById(id);
        if (!ExamCardStatusEnum.SUBMIT.equals(examCardModelFour.getStatus())) {
            throw ExceptionResultEnum.ERROR.exception("提交状态题卡才能生成图片");
        }
        createJpgImage(examCardModelFour);
        if (StringUtils.isNotBlank(examCardModelFour.getJpgAttachment())) {
            UpdateWrapper<ExamCardModelFour> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(ExamCardModelFour::getJpgAttachment, examCardModelFour.getJpgAttachment())
                    .eq(ExamCardModelFour::getId, id);
            this.update(updateWrapper);
        }
    }

    @Resource
    ExamCardService examCardService;

    /**
     * 导出题卡文件
     *
     * @param id
     */
    @Override
    public void downloadCard(Long id) {
        Map<String, Object> map = this.createZipFile(id, true);
        String rootPath = null;
        try {
            if (!org.springframework.util.CollectionUtils.isEmpty(map)) {
                rootPath = (String) map.get("rootPath");
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (Objects.nonNull(rootPath)) {
                FileUtil.deleteDirectory(rootPath);
            }
        }
    }

    /**
     * 批量导出题卡文件
     *
     * @param ids
     */
    @Override
    public void batchDownloadCard(List<Long> ids) {
        List<String> rootPathList = new ArrayList<>(ids.size());
        List<File> zipFileList = new ArrayList<>(ids.size());
        try {
            for (Long l : ids) {
                Map<String, Object> map = this.createZipFile(l, false);
                if (!org.springframework.util.CollectionUtils.isEmpty(map)) {
                    rootPathList.add((String) map.get("rootPath"));
                    zipFileList.add(new File((String) map.get("zipFilePath")));
                }
            }
            if (CollectionUtils.isNotEmpty(zipFileList)) {
                String zipFileRootPath = FileUtil.getFileTempDirPath(true);
                String zipFileName = zipFileRootPath + File.separator + "批量下载" + SystemConstant.ZIP_PREFIX;
                Zip4jUtil.zipFile(zipFileName, zipFileList);
                File zipFile = new File(zipFileName);
                rootPathList.add(zipFileRootPath);
                FileUtil.outputFile(ServletUtil.getResponse(), zipFile, zipFile.getName());
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (CollectionUtils.isNotEmpty(rootPathList)) {
                for (String s : rootPathList) {
                    FileUtil.deleteDirectory(s);
                }
            }
        }
    }

    @Override
    public ExamCardModelFour getByExamIdAndCourseId(Long examId, Long courseId) {
        QueryWrapper<ExamCardModelFour> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamCardModelFour::getExamId, examId)
                .eq(ExamCardModelFour::getCourseId, courseId);
        return this.getOne(queryWrapper);
    }

    @Override
    public void validCourse(GenericExamCardModelFourParams params) {
        ExamCardModelFour examCardModelFour = this.getByExamIdAndCourseId(params.getExamId(), params.getCourseId());
        if (params.getId() == null) {
            if (examCardModelFour != null) {
                throw ExceptionResultEnum.ERROR.exception("每个课程只能创建一个题卡");
            }
        } else {
            if (examCardModelFour != null && !examCardModelFour.getId().equals(params.getId())) {
                throw ExceptionResultEnum.ERROR.exception("每个课程只能创建一个题卡");
            }
        }
    }

    @Override
    public void saveQuestionAndAnswerCard(ExamCardModelFour examCardModelFour, MarkPaper markPaper, Long userId) {
        if (examCardModelFour == null) {
            examCardModelFour = this.getByExamIdAndCourseId(markPaper.getExamId(), markPaper.getCourseId());
        }
        if (examCardModelFour == null) {
            return;
        }
        ExamDetailCourseInitMarkDto dto = new ExamDetailCourseInitMarkDto(examCardModelFour, markPaper, userId);
        // 解析卡格式中试卷结构并保存
        printFinishService.insertMarkQuestion(dto.getExamId(), dto.getCourseId(), dto.getPaperNumber(), examCardModelFour.getContent(), null, ExamModelEnum.MODEL4);
        // 生成并保存卡格式文件（后缀为.json）
        ScanAnswerCard scanAnswerCard = printFinishService.insertScanAnswerCard(dto, examCardModelFour.getId().toString(), examCardModelFour.getContent());
        if (Objects.nonNull(scanAnswerCard)) {
            printFinishService.insertScanAnswerCardQuestion(dto.getExamId(), dto.getPaperNumber(), examCardModelFour.getContent(), scanAnswerCard.getNumber());
        }
        // 自动保存题卡遮盖区
        printFinishService.updateMarkSheetConfig(dto, examCardModelFour.getContent());
    }

    @Async
    @Override
    public void createJpgImageSync(Long id) {
        ExamCardModelFour examCardModelFour = this.getById(id);
        if (examCardModelFour != null) {
            ExamCardModelFour cardModelFour = createJpgImage(examCardModelFour);
            this.updateById(cardModelFour);
        }
    }

    /**
     * 创建zip
     *
     * @param id
     * @param output
     * @return
     */
    protected Map<String, Object> createZipFile(Long id, boolean output) {
        Map<String, Object> map = new HashMap<>();
        try {
            ExamCardModelFour examCardModelFour = this.getById(id);
            // 去掉题卡名称中的空格，否则创建目录报错
            String title = StringUtils.deleteWhitespace(examCardModelFour.getTitle());
            // 本地保存目录
            String rootPath = FileUtil.getFileTempDirPath(true);
            // 下载文件保存路径（以题卡名称命名）
            String downloadFilePath = rootPath + File.separator + title;
            File downloadPathFile = new File(downloadFilePath);
            if (!downloadPathFile.exists()) {
                downloadPathFile.mkdirs();
            }

            // HTML文件
            String htmlContent = createPdfUtil.replaceBlankHtmlContent(examCardModelFour.getHtmlContent(), examCardModelFour.getCourseId());
            File htmlFile = new File(downloadFilePath, title + SystemConstant.HTML_PREFIX);
            if (!htmlFile.exists()) {
                htmlFile.createNewFile();
            }
            FileUtil.writeContent(htmlFile, htmlContent);

            // PDF文件
            File pdfFile = new File(downloadFilePath, title + SystemConstant.PDF_PREFIX);
            if (!pdfFile.exists()) {
                pdfFile.createNewFile();
            }
            HtmlToPdfUtil.convert(htmlFile.getPath(), pdfFile.getPath(), "8K".equals(examCardModelFour.getPageSize()) ? PageSizeEnum.K8 : PageSizeEnum.valueOf(examCardModelFour.getPageSize()));

            // JPG文件
            String jpgAttachmentIds = examCardModelFour.getJpgAttachment();
            if (StringUtils.isNotBlank(jpgAttachmentIds)) {
                List<ConvertJpgStorage> jsonObjectList = JSONObject.parseArray(jpgAttachmentIds, ConvertJpgStorage.class);
                for (ConvertJpgStorage convertJpgStorage : jsonObjectList) {
                    String jpgAttachmentId = convertJpgStorage.getAttachmentId();
                    Integer index = convertJpgStorage.getIndex();
                    if (jpgAttachmentId != null) {
                        BasicAttachment attachment = basicAttachmentService.getById(jpgAttachmentId);
                        if (attachment != null) {
                            String fileName = downloadFilePath + File.separator + title + SystemConstant.HYPHEN + index + attachment.getType();
                            try {
                                fileUploadService.downloadFile(attachment.getPath(), fileName);
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                    }
                }
            }

            // json文件
            String content = examCardModelFour.getContent();
            if (StringUtils.isNotBlank(content)) {
                String jsonPath = downloadFilePath + File.separator + title + SystemConstant.JSON_PREFIX;
                FileUtil.writeContent(new File(jsonPath), content);
            }

            String zipFileName = title + SystemConstant.ZIP_PREFIX;
            SysUser sysUser = sysUserService.getById(ServletUtil.getRequestUserId());
            if (output) {
                FileUtil.downloadEncryptZip(ServletUtil.getResponse(), downloadPathFile, zipFileName, new String(Base64Util.decode(sysUser.getPassword())));
            } else {
                FileUtil.downloadEncryptZip(downloadPathFile, zipFileName, new String(Base64Util.decode(sysUser.getPassword())));
            }
            map.put("rootPath", rootPath);
            map.put("zipFilePath", rootPath + File.separator + zipFileName);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return map;
    }
}
