package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.*;
import com.qmth.distributed.print.business.bean.dto.initMarkData.ExamDetailCourseInitMarkDto;
import com.qmth.distributed.print.business.bean.dto.initMarkData.InitExamDetailDto;
import com.qmth.distributed.print.business.bean.result.ExaminationDetailResult;
import com.qmth.distributed.print.business.bean.result.ExaminationResult;
import com.qmth.distributed.print.business.bean.result.SummarizedDataResult;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.*;
import com.qmth.distributed.print.business.mapper.ExamDetailMapper;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.execute.AsyncExamStudentImportService;
import com.qmth.distributed.print.business.util.ExamTaskUtil;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.examRule.CodeNameEnableValue;
import com.qmth.teachcloud.common.bean.examRule.FieldsDto;
import com.qmth.teachcloud.common.bean.vo.PaperInfoVo;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.*;
import com.qmth.teachcloud.common.enums.*;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.util.ConvertUtil;
import com.qmth.teachcloud.common.util.RedisCounterUtil;
import com.qmth.teachcloud.common.util.RedisUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.entity.MarkStudent;
import com.qmth.teachcloud.mark.service.*;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.FillPatternType;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 考务数据 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamDetailServiceImpl extends ServiceImpl<ExamDetailMapper, ExamDetail> implements ExamDetailService {
    private final static Logger log = LoggerFactory.getLogger(ExamDetailServiceImpl.class);

    @Resource
    private ExamDetailCourseService examDetailCourseService;
    @Resource
    private BasicExamStudentService basicExamStudentService;
    @Resource
    private BasicExamRuleService basicExamRuleService;

    @Resource
    private ExamDetailMapper examDetailMapper;

    @Resource
    private CommonCacheService commonCacheService;

    @Resource
    private ExamStudentService examStudentService;

    @Resource
    private ExamPrintPlanService examPrintPlanService;

    @Resource
    private BasicAttachmentService basicAttachmentService;

    @Resource
    RedisCounterUtil redisCounterUtil;

    @Resource
    ClientStatusService clientStatusService;

    @Resource
    TeachcloudCommonService teachcloudCommonService;

    @Resource
    ExamTaskService examTaskService;

    @Resource
    ExamTaskDetailService examTaskDetailService;

    @Resource
    RedisUtil redisUtil;
    @Resource
    BasicRoleDataPermissionService basicRoleDataPermissionService;

    @Resource
    private ScanPaperService scanPaperService;

    @Resource
    @Lazy
    private PrintCommonService printCommonService;

    @Resource
    private AsyncExamStudentImportService asyncExamStudentImportService;
    @Resource
    private MarkStudentService markStudentService;
    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkPaperPackageService markPaperPackageService;
    @Resource
    private ScanAnswerCardService scanAnswerCardService;
    @Resource
    private MarkUserQuestionService markUserQuestionService;
    @Resource
    private MarkUserClassService markUserClassService;
    @Resource
    private TBTaskPdfService tbTaskPdfService;

    @Resource
    BasicSemesterService basicSemesterService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public double calculateTotalSubjects(Long printPlanId) {
        List<ExamDetail> examDetailList = this.list(new QueryWrapper<ExamDetail>().lambda().eq(ExamDetail::getPrintPlanId, printPlanId));
        List<Long> examDetailIds = examDetailList.stream().map(m -> m.getId()).collect(Collectors.toList());

        if (examDetailIds.isEmpty()) {
            return 0;
        }
        List<ExamDetailCourse> examDetailCourseList = examDetailCourseService.list(new QueryWrapper<ExamDetailCourse>().lambda().in(ExamDetailCourse::getExamDetailId, examDetailIds));
        return examDetailCourseList.isEmpty() ? 0 : examDetailCourseList.stream().map(m -> m.getCourseCode()).distinct().count();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public double calculateTotalPackages(Long printPlanId) {
        List<ExamDetail> examDetailList = this.list(new QueryWrapper<ExamDetail>().lambda().eq(ExamDetail::getPrintPlanId, printPlanId));
        return examDetailList.size();
    }

    @Override
    public IPage<PrintTaskDto> listPrintTask(Long semesterId, Long examId, Long printPlanId, String status, Long courseId, String paperNumber, String examPlace, String examRoom, Long examStartTime, Long examEndTime, Long printStartTime, Long printEndTime, Long printHouseId, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(), ServletUtil.getRequest().getServletPath());
        Page<PrintTaskDto> page = new Page<>(pageNumber, pageSize);
        return this.baseMapper.listPrintTask(page, schoolId, semesterId, examId, printPlanId, status, courseId, paperNumber, examPlace, examRoom, examStartTime, examEndTime, printStartTime, printEndTime, printHouseId, dpr);
    }

    @Override
    public PrintTaskTotalDto taskTotalData(Long semesterId, Long examId, Long printPlanId, String status, Long courseId, String paperNumber, String examPlace, String examRoom, Long examStartTime, Long examEndTime, Long printStartTime, Long printEndTime, Long printHouseId) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(), ServletUtil.getRequest().getServletPath());
        PrintTaskTotalDto printTaskTotalDto = this.baseMapper.taskTotalData(schoolId, semesterId, examId, printPlanId, status, courseId, paperNumber, examPlace, examRoom, examStartTime, examEndTime, printStartTime, printEndTime, printHouseId, dpr);

        if (printTaskTotalDto != null) {
            // 试卷总计
            Set<String> paperNumberTypeSet = new HashSet<>();
            List<ExamDetailCourse> examDetailCourseList = this.baseMapper.selectPaperCount(schoolId, semesterId, examId, printPlanId, status, courseId, paperNumber, examPlace, examRoom, examStartTime, examEndTime, printStartTime, printEndTime, printHouseId, dpr);
            for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                for (String s : examDetailCourse.getPaperType().split(",")) {
                    paperNumberTypeSet.add(examDetailCourse.getPaperNumber().concat(s));
                }
            }
            printTaskTotalDto.setPaperCount(paperNumberTypeSet.size());
        }
        return printTaskTotalDto;
    }

    @Override
    public List<JSONObject> taskViewPDF(Long examDetailId) {
        ExamDetail examDetail = this.getById(examDetailId);
        List<JSONObject> list = new ArrayList<>();
        if (StringUtils.isNotBlank(examDetail.getMergePdfPath())) {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put(SystemConstant.TYPE, PdfTypeEnum.MERGE_PDF);
            jsonObject.put("url", teachcloudCommonService.filePreview(examDetail.getMergePdfPath()));
            list.add(jsonObject);
        }
        if (examDetail.getAttachmentId() != null) {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put(SystemConstant.TYPE, PdfTypeEnum.PAPER);
            BasicAttachment attachment = basicAttachmentService.getById(examDetail.getAttachmentId());
            jsonObject.put("url", attachment == null ? null : teachcloudCommonService.filePreview(attachment.getPath()));
            list.add(jsonObject);
        }
        if (examDetail.getCardAttachmentId() != null) {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put(SystemConstant.TYPE, PdfTypeEnum.CARD_A3);
            BasicAttachment attachment = basicAttachmentService.getById(examDetail.getCardAttachmentId());
            jsonObject.put("url", attachment == null ? null : teachcloudCommonService.filePreview(attachment.getPath()));
            list.add(jsonObject);
        }
        if (Objects.nonNull(examDetail.getAttachmentPath())) {
            JSONObject js = JSONObject.parseObject(examDetail.getAttachmentPath());
            JSONArray jsonArray = js.getJSONArray(SystemConstant.PATH);
            for (int i = 0; i < jsonArray.size(); i++) {
                JSONObject object = jsonArray.getJSONObject(i);
                JSONObject jsonObject = new JSONObject();
                jsonObject.put(SystemConstant.TYPE, Objects.nonNull(object.get("printType")) ? PdfTypeEnum.valueOf((String) object.get("printType")) : null);
                String uploadType = null;
                if (object.get(SystemConstant.UPLOAD_TYPE) instanceof JSONArray) {
                    JSONArray uploadTypeJsonArray = object.getJSONArray(SystemConstant.UPLOAD_TYPE);
                    uploadTypeFor:
                    for (int y = 0; y < uploadTypeJsonArray.size(); y++) {
                        uploadType = uploadTypeJsonArray.getString(y);
                        if (uploadType.equalsIgnoreCase("PDF")) {
                            break uploadTypeFor;
                        }
                    }
                } else {
                    uploadType = (String) object.get(SystemConstant.UPLOAD_TYPE);
                }
                jsonObject.put("url", Objects.nonNull(object.get(SystemConstant.PDF_PATH)) ? teachcloudCommonService.filePreviewByPathAndType((String) object.get(SystemConstant.PDF_PATH), uploadType, (String) object.get("type"), false) : null);
                list.add(jsonObject);
            }
        }
        return list;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<FieldsDto> findExaminationFields(Long schoolId) {
        BasicExamRule basicExamRule = basicExamRuleService.getBySchoolId(schoolId);
        if (basicExamRule == null) {
            throw ExceptionResultEnum.ERROR.exception("通用规则未设置");
        }
        String requiredFields = basicExamRule.getRequiredFields();
        String extendFields = basicExamRule.getExtendFields();
        // 基础字段
        List<FieldsDto> requiredFieldsList = JSONObject.parseArray(requiredFields, FieldsDto.class);
        requiredFieldsList = requiredFieldsList.stream().filter(m -> m.getEnable()).collect(Collectors.toList());
        for (FieldsDto fieldsDto : requiredFieldsList) {
            fieldsDto.setLevel("primary");
        }

        // 扩展字段
        List<FieldsDto> extendFieldsList = JSONObject.parseArray(extendFields, FieldsDto.class);
        List<String> extendCodeList = extendFieldsList.stream().map(FieldsDto::getCode).collect(Collectors.toList());
        List<FieldsDto> finalRequiredFieldsList = requiredFieldsList;
        if (extendCodeList.stream().anyMatch(e -> finalRequiredFieldsList.stream().map(FieldsDto::getCode).collect(Collectors.toList()).contains(e))) {
            throw ExceptionResultEnum.ERROR.exception("扩展字段中存在和基础字段相同的字段变量名");
        }
        // 有效的扩展字段
        List<FieldsDto> validExtendList = extendFieldsList.stream().filter(FieldsDto::getEnable).collect(Collectors.toList());
        for (FieldsDto fieldsDto : validExtendList) {
            fieldsDto.setLevel("secondary");
        }

        // 有效的考务字段
        List<FieldsDto> validExaminationFieldList = new ArrayList<>();
        validExaminationFieldList.addAll(requiredFieldsList);
        validExaminationFieldList.addAll(validExtendList);

        return validExaminationFieldList;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void downLoadExaminationTemplate(Long schoolId, String type, HttpServletResponse response) throws IOException {
        final String DEFALUT_EXT = ".xlsx";
        final String DEFALUT_CONTENT_TYPE = "application/vnd.ms-excel";
        List<FieldsDto> fieldsNameList = this.findExaminationFields(schoolId);

        log.debug("导出Excel开始...");
        XSSFWorkbook wb = new XSSFWorkbook();
        String fileName = "考务数据模板";
        response.setHeader("Content-Disposition", "inline;filename=" + URLEncoder.encode(fileName, SystemConstant.CHARSET_NAME) + DEFALUT_EXT);
        response.setContentType(DEFALUT_CONTENT_TYPE);
        XSSFSheet sheet = wb.createSheet(commonCacheService.schoolCache(schoolId).getName() + "考务数据模板");

        Font defaultFont = wb.createFont();
        defaultFont.setFontHeightInPoints((short) 12);
        defaultFont.setFontName("宋体");


        int cellCount = fieldsNameList.size();
        // 说明
        XSSFCellStyle describeStyle = wb.createCellStyle();
        describeStyle.setAlignment(HorizontalAlignment.LEFT);
        Font describeFont = wb.createFont();
        describeFont.setFontHeightInPoints((short) 12);
        describeFont.setFontName("宋体");
        describeFont.setColor(IndexedColors.RED.getIndex());
        describeStyle.setFont(describeFont);
//        String describe = SystemConstant.EXAMINATION_DESCRIBE;
        String describe = SystemConstant.getDescString(type, basicExamRuleService.getBySchoolId(schoolId).getRequiredFieldList());
        describe = describe.trim();
        XSSFRow rowDescribe = sheet.createRow(0);
        rowDescribe.setHeightInPoints(195); //行高设置成195px
        for (int i = 0; i < cellCount; i++) {
            XSSFCell cell = rowDescribe.createCell(i);
            if (i == 0) {
                cell.setCellValue(describe);
                cell.setCellStyle(describeStyle);
            }
        }
        CellRangeAddress region = new CellRangeAddress(0, 0, 0, cellCount - 1);
        sheet.addMergedRegion(region);

        // 表头行
        XSSFCellStyle headerStyle = wb.createCellStyle();
        headerStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        headerStyle.setAlignment(HorizontalAlignment.CENTER);
        headerStyle.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
        headerStyle.setFont(defaultFont);
        XSSFRow rowHead = sheet.createRow(1);
        for (int i = 0; i < fieldsNameList.size(); i++) {
            XSSFCell cell = rowHead.createCell(i);
            cell.setCellValue(fieldsNameList.get(i).getName());
            cell.setCellStyle(headerStyle);
        }

        // 样例行
        XSSFDataFormat format = wb.createDataFormat();
        XSSFCellStyle exampleStyle = wb.createCellStyle();
        exampleStyle.setDataFormat(format.getFormat("@"));
        exampleStyle.setAlignment(HorizontalAlignment.LEFT);
        exampleStyle.setFont(defaultFont);
        XSSFRow rowExample = sheet.createRow(2);
        for (int i = 0; i < fieldsNameList.size(); i++) {
            XSSFCell cell = rowExample.createCell(i);
            FieldsDto fieldsDto = fieldsNameList.get(i);
            if ("examDate".equals(fieldsDto.getCode())) {
                XSSFDataFormat formatDf = wb.createDataFormat();
                XSSFCellStyle exampleStyleDf = wb.createCellStyle();
                exampleStyleDf.setDataFormat(formatDf.getFormat("yyyy-mm-dd"));
                exampleStyleDf.setAlignment(HorizontalAlignment.LEFT);
                exampleStyleDf.setFont(defaultFont);
                cell.setCellValue("2023-01-01");
                cell.setCellStyle(exampleStyleDf);
            } else if ("examTime".equals(fieldsDto.getCode())) {
                cell.setCellValue("10:00-12:00");
                cell.setCellStyle(exampleStyle);
            } else if ("studentCode".equals(fieldsDto.getCode())) {
                cell.setCellValue("20240101");
                cell.setCellStyle(exampleStyle);
            } else {
                cell.setCellValue("测试" + fieldsNameList.get(i).getName());
                cell.setCellStyle(exampleStyle);
            }
        }

        for (int i = 0; i < cellCount; i++) {
            sheet.autoSizeColumn(i);
            sheet.setColumnWidth(i, sheet.getColumnWidth(i) * 17 / 10);
        }
        ServletOutputStream outputStream = response.getOutputStream();
        wb.write(outputStream);
        outputStream.flush();
        outputStream.close();
        log.debug("导出Excel结束...");
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public IPage<ExaminationResult> findExaminationBriefPage(Long schoolId, Long semesterId, Long examId, List<Long> printPlanIdList, Long courseId, String paperNumber, String examPlace, String examRoom, String packageCode, Long startDate, Long endDate, int pageNumber, int pageSize, DataPermissionRule dpr) {
        IPage<ExaminationResult> page = examDetailMapper.findBriefPage(new Page<>(pageNumber, pageSize), schoolId, semesterId, examId, printPlanIdList, courseId, paperNumber, examPlace, examRoom, packageCode, startDate, endDate, dpr);
        List<ExaminationResult> list = page.getRecords();
        for (ExaminationResult examinationResult : list) {
            // 处理时间
            Long startTime = examinationResult.getExamStartTime();
            Long endTime = examinationResult.getExamEndTime();
            Map<String, String> dateMap = ConvertUtil.analyzeDateAndTime(startTime, endTime);
            String date = dateMap.get("date");
            String time = dateMap.get("time");
            examinationResult.setExamDate(date);
            examinationResult.setExamTime(time);

            Set<String> stringSet = new HashSet<>();
            for (String id : examinationResult.getExamDetailCourseIds().split(",")) {
                ExamDetailCourse examDetailCourse = examDetailCourseService.getById(id);
                if (StringUtils.isNotBlank(examDetailCourse.getPaperType())) {
                    ExamTaskDetail examTaskDetail = examTaskDetailService.getByExamIdAndPaperNumberAndSerialNumber(examinationResult.getExamId(), examDetailCourse.getPaperNumber(), examDetailCourse.getSerialNumber());
                    String paperType = examDetailCourse.getPaperType();
                    List<PaperInfoVo> paperInfoVoList = ExamTaskUtil.parsePaperAttachmentPath(examTaskDetail.getPaperAttachmentIds(), paperType);
                    for (PaperInfoVo paperInfoVo : paperInfoVoList) {
                        String cardTypeString = paperInfoVo.getCardType();
                        String createMethod = paperInfoVo.getCreateMethod();
                        if (cardTypeString.equals(CardTypeEnum.GENERIC.name()) && StringUtils.isNotBlank(createMethod) && createMethod.equals(CardCreateMethodEnum.UPLOAD.name())) {
                            stringSet.add(SyncCardTypeEnum.GENERIC.getDesc());
                        } else {
                            stringSet.add(SyncCardTypeEnum.CUSTOM.getDesc());
                        }
                    }
                }
            }
            if (stringSet.size() > 0) {
                examinationResult.setCardType(String.join(",", stringSet));
            } else {
                examinationResult.setCardType("-");
            }
        }
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public SummarizedDataResult findSummarizedData(Long schoolId, Long semesterId, Long examId, List<Long> printPlanIdList, Long courseId, String paperNumber, String examPlace, String examRoom, String packageCode, Long startDate, Long endDate, DataPermissionRule dpr) {
        SummarizedDataResult summarizedDataResult = this.baseMapper.findSummarizedData(schoolId, semesterId, examId, printPlanIdList, courseId, paperNumber, examPlace, examRoom, packageCode, startDate, endDate, dpr);
        if (summarizedDataResult == null) {
            summarizedDataResult = new SummarizedDataResult();
            summarizedDataResult.setSchoolId(schoolId);
            summarizedDataResult.setTotalSubjects(0);
            summarizedDataResult.setPackageCount(0);
        }

        return summarizedDataResult;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public IPage<ExaminationDetailResult> findExaminationDetail(Long schoolId, Long semesterId, Long examId, List<Long> printPlanIdList, Long courseId, String paperNumber, String examPlace, String examRoom, String studentParams, Long startDate, Long endDate, int pageNumber, int pageSize) {
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission();
        return examDetailMapper.findDetailPage(new Page<>(pageNumber, pageSize), schoolId, semesterId, examId, printPlanIdList, courseId, paperNumber, examPlace, examRoom, SystemConstant.translateSpecificSign(studentParams), startDate, endDate, dpr);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public IPage<ExaminationDetailResult> findExaminationDetail(Long examDetailId, String param, int pageNumber, int pageSize) {
        return examDetailMapper.findDetailPageById(new Page<>(pageNumber, pageSize), examDetailId, param);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<String> findExamPlaceDatasource(String param, Long examId, List<Long> printPlanIdList) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        QueryWrapper<ExamDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamDetail::getSchoolId, schoolId);
        if (examId != null) {
            queryWrapper.lambda().eq(ExamDetail::getExamId, examId);
        }
        if (printPlanIdList.size() > 0) {
            queryWrapper.lambda().in(ExamDetail::getPrintPlanId, printPlanIdList);
        }
        if (StringUtils.isNotBlank(param)) {
            queryWrapper.lambda().like(ExamDetail::getExamPlace, SystemConstant.translateSpecificSign(param));
        }
        List<ExamDetail> examDetail = this.list(queryWrapper);
        return examDetail.stream().filter(m -> StringUtils.isNotBlank(m.getExamPlace())).map(ExamDetail::getExamPlace).distinct().collect(Collectors.toList());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<String> findExamRoomDatasource(String param, Long examId, List<Long> printPlanIdList) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        QueryWrapper<ExamDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamDetail::getSchoolId, schoolId);
        if (examId != null) {
            queryWrapper.lambda().eq(ExamDetail::getExamId, examId);
        }
        if (printPlanIdList.size() > 0) {
            queryWrapper.lambda().in(ExamDetail::getPrintPlanId, printPlanIdList);
        }
        if (StringUtils.isNotBlank(param)) {
            queryWrapper.lambda().like(ExamDetail::getExamRoom, SystemConstant.translateSpecificSign(param));
        }
        List<ExamDetail> examDetail = this.list(queryWrapper);
        return examDetail.stream().filter(m -> StringUtils.isNotBlank(m.getExamRoom())).map(ExamDetail::getExamRoom).distinct().collect(Collectors.toList());
    }

    @Transactional
    @Override
    public boolean taskSubmit(ExamDetail examDetail) {
        // 检查前置操作是否完成(是否生成pdf)
        ExamDetail detail = this.getById(examDetail.getId());
        if (detail == null) {
            throw ExceptionResultEnum.ERROR.exception("印刷任务数据不存在");
        }

        // 只有就绪状态并且附件生成才可提交
        if (!ExamDetailStatusEnum.READY.equals(detail.getStatus())) {
            throw ExceptionResultEnum.ERROR.exception("印刷任务就绪状态才可提交印刷");
        } else if (detail.getAttachmentId() == null && detail.getCardAttachmentId() == null) {
            throw ExceptionResultEnum.ERROR.exception("印刷任务PDF文件未生成");
        }

        // 修改印刷计划状态为印刷中
        UpdateWrapper<ExamPrintPlan> examPrintPlanUpdateWrapper = new UpdateWrapper<>();
        examPrintPlanUpdateWrapper.lambda().set(ExamPrintPlan::getStatus, PrintPlanStatusEnum.PRINTING).eq(ExamPrintPlan::getId, detail.getPrintPlanId());
        examPrintPlanService.update(examPrintPlanUpdateWrapper);

        // 修改考场状态为待印刷
        UpdateWrapper<ExamDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamDetail::getStatus, ExamDetailStatusEnum.WAITING).eq(ExamDetail::getId, examDetail.getId());

        return this.update(updateWrapper);
    }

    @Transactional
    @Override
    public boolean taskFinish(ExamDetail examDetail) {
        // 检查前置操作是否完成(是否生成pdf)
        ExamDetail detail = this.getById(examDetail.getId());
        if (detail == null) {
            throw ExceptionResultEnum.ERROR.exception("印刷任务数据不存在");
        }

        // 只有待印刷状态并且附件生成才可提交
        if (!ExamDetailStatusEnum.WAITING.equals(detail.getStatus())) {
            throw ExceptionResultEnum.ERROR.exception("印刷任务[待印刷]状态才可完成");
        }

        // 修改考场状态为已完成
        UpdateWrapper<ExamDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamDetail::getStatus, ExamDetailStatusEnum.FINISH)
                .set(ExamDetail::getPrintStartTime, System.currentTimeMillis())
                .set(ExamDetail::getPrintEndTime, System.currentTimeMillis())
                .eq(ExamDetail::getId, examDetail.getId());
        this.update(updateWrapper);

        // 所有考场打印完成，更新印刷计划状态
        examPrintPlanService.updateStatusById(examDetail.getPrintPlanId(), PrintPlanStatusEnum.FINISH);

        // 更新曝光卷型
        examTaskDetailService.updateExposePaperType(examDetail.getExamId(), examDetailCourseService.listByExamDetailId(examDetail.getId()));
        return true;
    }

    @Transactional
    @Override
    public boolean taskCancel(Long id) {
        ExamDetail detail = this.getById(id);
        if (!detail.getStatus().equals(ExamDetailStatusEnum.WAITING)) {
            throw ExceptionResultEnum.ERROR.exception("印刷任务状态为[" + ExamDetailStatusEnum.WAITING.getName() + "]的任务可撤回提交");
        }
        UpdateWrapper<ExamDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamDetail::getStatus, ExamDetailStatusEnum.READY).eq(ExamDetail::getId, id);
        this.update(updateWrapper);

        //所有考场都撤回，印刷任务状态改为就绪
        examPrintPlanService.updateStatusById(detail.getPrintPlanId(), PrintPlanStatusEnum.READY);

        // 当关联课程所有考场都撤回，清除提当前计划下打样记录
        QueryWrapper<ExamDetailCourse> examDetailCourseQueryWrapper = new QueryWrapper<>();
        examDetailCourseQueryWrapper.lambda().eq(ExamDetailCourse::getExamDetailId, detail.getId());
        List<ExamDetailCourse> examDetailCourses = examDetailCourseService.list(examDetailCourseQueryWrapper);
        for (ExamDetailCourse examDetailCours : examDetailCourses) {
            // 清空审核表附件ID
            UpdateWrapper<ExamTask> examTaskUpdateWrapper = new UpdateWrapper<>();
            examTaskUpdateWrapper.lambda().set(ExamTask::getApproveFormAttachmentId, null)
                    .eq(ExamTask::getSchoolId, examDetailCours.getSchoolId())
                    .eq(ExamTask::getCourseId, examDetailCours.getCourseId())
                    .eq(ExamTask::getPaperNumber, examDetailCours.getPaperNumber());
            examTaskService.update(examTaskUpdateWrapper);

            List<ExamDetail> examDetailList = this.listByPrintPlanIdAndCourseCodeAndPaperNumber(detail.getSchoolId(), detail.getPrintPlanId(), examDetailCours.getCourseId(), examDetailCours.getPaperNumber());
            long count = examDetailList.stream().filter(m -> !m.getStatus().name().equals(ExamDetailStatusEnum.NEW.name()) && !m.getStatus().name().equals(ExamDetailStatusEnum.READY.name())).count();
            if (count == 0) {
                UpdateWrapper<ClientStatus> clientStatusUpdateWrapper = new UpdateWrapper<>();
                clientStatusUpdateWrapper.lambda().eq(ClientStatus::getSchoolId, detail.getSchoolId())
                        .eq(ClientStatus::getPrintPlanId, detail.getPrintPlanId())
                        .eq(ClientStatus::getCourseId, examDetailCours.getCourseId())
                        .eq(ClientStatus::getPaperNumber, examDetailCours.getPaperNumber());
                clientStatusService.remove(clientStatusUpdateWrapper);
            }
        }
        return true;
    }

    @Transactional
    @Override
    public boolean taskBatchCancel(Map data) {
        String type = String.valueOf(data.get("type"));
        String ids = String.valueOf(data.get("ids"));
        for (String id : ids.split(",")) {
            ExamDetail examDetail = this.getById(id);
            if ("submit".equals(type)) {
                taskSubmit(examDetail);
            } else if ("cancel".equals(type)) {
                taskCancel(Long.valueOf(id));
            } else if ("finish".equals(type)) {
                taskFinish(examDetail);
            }
        }
        return true;
    }

    @Override
    public List<ExamDetail> listByPrintPlanIdAndCourseCodeAndPaperNumber(Long schoolId, Long printPlanId, Long courseId, String paperNumber) {
        List<ExamDetailCourse> examDetailCourses = examDetailCourseService.listByPrintPlanIdAndCourseIdAndPaperNumber(schoolId, printPlanId, courseId, paperNumber);
        if (examDetailCourses != null && examDetailCourses.size() > 0) {
            Set<Long> examDetailIds = examDetailCourses.stream().map(ExamDetailCourse::getExamDetailId).collect(Collectors.toSet());
            return this.listByIds(examDetailIds);
        }
        return null;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<ExaminationExportDto> findExaminationExportDtoDatasource(Map<String, Object> map) {
        Long semesterId = null;
        if (map.get("semesterId") != null) {
            semesterId = Long.valueOf(String.valueOf(map.get("semesterId")));
        }
        Long examId = null;
        if (map.get("examId") != null) {
            examId = Long.valueOf(String.valueOf(map.get("examId")));
        }
        List<Long> printPlanIdList = null;
        if (map.get("printPlanIdList") != null) {
            printPlanIdList = JSON.parseArray(map.get("printPlanIdList").toString(), Long.class);
        }
        Long courseId = null;
        if (map.get("courseId") != null) {
            courseId = Long.valueOf(String.valueOf(map.get("courseId")));
        }
        String paperNumber = null;
        if (map.get("paperNumber") != null) {
            paperNumber = String.valueOf(map.get("paperNumber"));
        }
        String examPlace = null;
        if (map.get("examPlace") != null) {
            examPlace = String.valueOf(map.get("examPlace"));
        }
        String examRoom = null;
        if (map.get("examRoom") != null) {
            examRoom = String.valueOf(map.get("examRoom"));
        }
        String packageCode = null;
        if (map.get("packageCode") != null) {
            packageCode = String.valueOf(map.get("packageCode"));
        }
        Long startDate = null;
        if (map.get("startDate") != null) {
            startDate = Long.valueOf(String.valueOf(map.get("startDate")));
        }
        Long endDate = null;
        if (map.get("endDate") != null) {
            endDate = Long.valueOf(String.valueOf(map.get("endDate")));
        }
        DataPermissionRule dpr = new DataPermissionRule();
        if (Objects.nonNull(map.get("dpr"))) {
            dpr = (DataPermissionRule) map.get("dpr");
        }
        TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
        Long schoolId = tbTask.getSchoolId();

        List<ExaminationResult> examinationResultList = this.findExaminationBriefPage(schoolId, semesterId, examId,
                printPlanIdList, courseId, paperNumber, examPlace, examRoom, packageCode, startDate, endDate,
                SystemConstant.PAGE_NUMBER, SystemConstant.PAGE_SIZE, dpr).getRecords();
        int index = 0;
        List<ExaminationExportDto> examinationExportDtoList = new ArrayList<>();
        for (ExaminationResult examinationResult : examinationResultList) {
            index++;
            ExaminationExportDto examinationExportDto = new ExaminationExportDto();
            examinationExportDto.setIndex((long) index);
            examinationExportDto.setExamName(examinationResult.getExamName());
            examinationExportDto.setSemesterName(examinationResult.getSemesterName());
            examinationExportDto.setPrintPlanName(examinationResult.getPrintPlanName());
            examinationExportDto.setExamDate(examinationResult.getExamDate());
            examinationExportDto.setExamTime(examinationResult.getExamTime());
            examinationExportDto.setExamPlace(examinationResult.getExamPlace());
            examinationExportDto.setExamRoom(examinationResult.getExamRoom());
            examinationExportDto.setPackageCode(examinationResult.getPackageCode());
            examinationExportDto.setCourseNameCode(examinationResult.getCourseNameCode());
            examinationExportDto.setPaperNumber(examinationResult.getPaperNumber());
            examinationExportDto.setTotalSubjects(examinationResult.getTotalSubjects());
            examinationExportDto.setCardType(examinationResult.getCardType());
            examinationExportDtoList.add(examinationExportDto);
        }
        return examinationExportDtoList;
    }

    @Override
    public List<Map> listStudentByExamDetailCourseId(Long schoolId, Long examDetailCourseId, String studentCode, String type) {
        return this.baseMapper.listStudentByExamDetailCourseId(schoolId, examDetailCourseId, studentCode, type);
    }

    @Override
    public boolean taskNormal(ExamDetail examDetail) {
        ExamDetail examDetailExist = this.getById(examDetail.getId());
        // 已扫描过，不能作废
        if (!examDetail.getNormal()) {
            List<ExamDetailCourse> examDetailCourseList = examDetailCourseService.listByExamDetailId(examDetail.getId());
            for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                boolean isScanPaper = scanPaperService.isScanByExamIdAndPaperNumber(examDetailExist.getExamId(), examDetailCourse.getPaperNumber());
                if (isScanPaper) {
                    throw ExceptionResultEnum.ERROR.exception("有课程已开始扫描，无法作废");
                }
            }
        }
        ExamDetail detail = this.getById(examDetail.getId());
        if (ExamDetailStatusEnum.CANCEL.equals(detail.getStatus()) && examDetail.getNormal()) {
            throw ExceptionResultEnum.ERROR.exception("印刷任务关联的命题任务已作废，无法恢复");
        }
        UpdateWrapper<ExamDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamDetail::getNormal, examDetail.getNormal()).eq(ExamDetail::getId, examDetail.getId());
        return this.update(updateWrapper);
    }

    @Override
    public String findClazzNamesByClazzIds(List<Long> clazzIdList, String separator) {
        if (!SystemConstant.strNotNull(separator)) {
            separator = ",";
        }
        Set<Long> clazzIdSet = new HashSet<>(clazzIdList);
        List<String> clazzNameList = this.findClazzNamesByClazzIdSet(clazzIdSet);
        return String.join(separator, clazzNameList);
    }

    /**
     * 根据id下载examDetail
     *
     * @param ids
     * @return
     */
    @Override
    public List<ExamDetailPdfDownloadDto> findPdfDownload(List<Long> ids) {
        return examDetailMapper.findPdfDownload(ids);
    }

    @Transactional
    @Override
    public void dataDelete(Long id) {
        ExamDetail examDetail = this.getById(id);
        if (examDetail == null) {
            throw ExceptionResultEnum.ERROR.exception("考场数据不存在");
        }
        if (ExamDetailStatusEnum.WAITING.equals(examDetail.getStatus())) {
            throw ExceptionResultEnum.ERROR.exception("考场数据已提交印刷，请先撤回后再删除");
        } else if (ExamDetailStatusEnum.PRINTING.equals(examDetail.getStatus())) {
            throw ExceptionResultEnum.ERROR.exception("考场数据已提交印刷，正在打印中，不允许删除");
        } else if (ExamDetailStatusEnum.FINISH.equals(examDetail.getStatus()) && examDetail.getNormal()) {
            throw ExceptionResultEnum.ERROR.exception("考场数据已打印完成，不允许删除");
        }
        //删除exam_detail
        this.removeById(id);
        // 删除pdf生成任务
        tbTaskPdfService.removeById(id);
        // 删除exam_detail_course
        QueryWrapper<ExamDetailCourse> examDetailCourseQueryWrapper = new QueryWrapper<>();
        examDetailCourseQueryWrapper.lambda().eq(ExamDetailCourse::getExamDetailId, id);
        List<ExamDetailCourse> examDetailCourses = examDetailCourseService.list(examDetailCourseQueryWrapper);
        if (!examDetailCourses.isEmpty()) {
            for (ExamDetailCourse examDetailCourse : examDetailCourses) {
                examDetailCourseService.removeById(examDetailCourse.getId());

                // 删除 exam_student
                List<ExamStudent> examStudents = examStudentService.listByExamDetailCourseId(examDetailCourse.getId());
                if (!examStudents.isEmpty()) {
                    List<Long> examStudentIds = examStudents.stream().map(ExamStudent::getId).collect(Collectors.toList());
                    // 已扫描
                    QueryWrapper<MarkStudent> queryWrapper = new QueryWrapper<>();
                    queryWrapper.lambda().in(MarkStudent::getId, examStudentIds).eq(MarkStudent::getUpload, true);
                    if (markStudentService.count(queryWrapper) > 0) {
                        throw ExceptionResultEnum.ERROR.exception("有考生已扫描，不允许删除");
                    }
                    examStudentService.removeByIds(examStudentIds);
                    // 删除扫描数据
                    markStudentService.removeByIds(examStudentIds);
                    // 当考务数据考生表中学生数量为0时，扫描表中学生数量大于0，说明从考生管理新增的考生未删除，则通过试卷编号全部删掉
                    if (examStudentService.countByExamIdAndPaperNumber(examDetail.getExamId(), examDetailCourse.getPaperNumber()) == 0) {
                        if (markStudentService.countByExamIdAndPaperNumber(examDetail.getExamId(), examDetailCourse.getPaperNumber()) > 0) {
                            markStudentService.deleteByExamIdAndPaperNumber(examDetail.getExamId(), examDetailCourse.getPaperNumber());
                        }
                        // 考生数据全部删除时，还原已曝光卷型
                        examTaskDetailService.updateUnexposePaperType(examDetail.getExamId(), examDetailCourse.getPaperNumber(), examDetailCourse.getSerialNumber());
                    }
                    if (markStudentService.countByExamIdAndPaperNumber(examDetail.getExamId(), examDetailCourse.getPaperNumber()) == 0) {
                        markPaperService.deleteByExamIdAndPaperNumber(examDetail.getExamId(), examDetailCourse.getPaperNumber());
                        markPaperPackageService.deleteByExamIdAndPaperNumber(examDetail.getExamId(), examDetailCourse.getPaperNumber());
                        markQuestionService.deleteByExamIdAndPaperNumber(examDetail.getExamId(), examDetailCourse.getPaperNumber());
                        scanAnswerCardService.deleteByExamIdAndPaperNumber(examDetail.getExamId(), examDetailCourse.getPaperNumber());
                        markUserQuestionService.deleteByExamIdAndPaperNumberAndQuestionId(examDetail.getExamId(), examDetailCourse.getPaperNumber(), null);
                        markUserClassService.deleteByExamIdAndPaperNumber(examDetail.getExamId(), examDetailCourse.getPaperNumber());
                    } else {
                        markPaperPackageService.deleteByExamIdAndPaperNumberAndPackageCode(examDetail.getExamId(), examDetailCourse.getPaperNumber(), examDetail.getPackageCode());
                        markPaperService.updateStudentCountByExamIdAndPaperNumber(examDetail.getExamId(), examDetailCourse.getPaperNumber());
                    }
                    if (markPaperService.getByExamIdAndPaperNumber(examDetail.getExamId(), examDetailCourse.getPaperNumber()) != null) {
                        // 清空考生管理中卷型
                        basicExamStudentService.clearPaperNumberAndPaperTypeById(examStudents.stream().filter(m -> m.getBasicStudentId() != null).map(ExamStudent::getBasicStudentId).collect(Collectors.toList()));
                    } else {
                        basicExamStudentService.clearPaperTypeByExamIdAndPaperNumber(examDetail.getExamId(), examDetailCourse.getPaperNumber());
                    }

                }
            }
        }

        // 修改exam_print_plan状态
        QueryWrapper<ExamDetail> examDetailQueryWrapper = new QueryWrapper<>();
        examDetailQueryWrapper.lambda().eq(ExamDetail::getPrintPlanId, examDetail.getPrintPlanId());
        List<ExamDetail> examDetailList = this.list(examDetailQueryWrapper);
        PrintPlanStatusEnum printPlanStatusEnum = PrintPlanStatusEnum.READY;
        if (examDetailList.isEmpty()) {
            printPlanStatusEnum = PrintPlanStatusEnum.NEW;
        } else {
            // 打印中
            long printingCount = examDetailList.stream().filter(m -> m.getStatus().equals(ExamDetailStatusEnum.PRINTING)).count();
            // 已完成
            long finishCount = examDetailList.stream().filter(m -> !m.getStatus().equals(ExamDetailStatusEnum.FINISH)).count();
            if (printingCount > 0) {
                printPlanStatusEnum = PrintPlanStatusEnum.PRINTING;
            } else if (finishCount == 0) {
                printPlanStatusEnum = PrintPlanStatusEnum.FINISH;
            }

        }
        // 由于考场数据变动，要同步更新最新的计划状态
        UpdateWrapper<ExamPrintPlan> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamPrintPlan::getStatus, printPlanStatusEnum).eq(ExamPrintPlan::getId, examDetail.getPrintPlanId());
        examPrintPlanService.update(updateWrapper);
    }

    @Override
    public List<ExamStudent> listStudentByExamDetailId(String examDetailId) {
        QueryWrapper<ExamDetailCourse> examDetailCourseQueryWrapper = new QueryWrapper<>();
        examDetailCourseQueryWrapper.lambda().eq(ExamDetailCourse::getExamDetailId, examDetailId);
        List<ExamDetailCourse> examDetailCourses = examDetailCourseService.list(examDetailCourseQueryWrapper);

        if (CollectionUtils.isNotEmpty(examDetailCourses)) {
            List<Long> examDetailCourseIds = examDetailCourses.stream().map(ExamDetailCourse::getId).collect(Collectors.toList());

            QueryWrapper<ExamStudent> examStudentQueryWrapper = new QueryWrapper<>();
            examStudentQueryWrapper.lambda().in(ExamStudent::getExamDetailCourseId, examDetailCourseIds);
            return examStudentService.list(examStudentQueryWrapper);
        }
        return null;
    }

    @Override
    public List<ExamDetailCourseDto> listByExamIdAndCourseId(Long schoolId, Long examId, Long courseId) {
        return this.baseMapper.listByExamIdAndCourseId(schoolId, examId, courseId);
    }

    @Override
    public void deleteByExamIdAndPaperNumber(Long schoolId, Long examId, String paperNumber) {
        List<ExamDetailCourse> examDetailCourseList = examDetailCourseService.listByExamIdAndPaperNumber(examId, paperNumber);
        if (CollectionUtils.isNotEmpty(examDetailCourseList)) {
            for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                // 删除考场
                this.removeById(examDetailCourse.getExamDetailId());
                // 删除考场科目
                examDetailCourseService.removeById(examDetailCourse.getId());
                // 删除考生
                examStudentService.removeByExamDetailCourseId(examDetailCourse.getId());
                // 删除pdf生成任务
                tbTaskPdfService.removeById(examDetailCourse.getExamDetailId());
            }
        }
    }

    @Override
    public TBTask importData(String printPlanId, MultipartFile file) {
        if (this.countByPrintPlanId(printPlanId) > 0) {
            throw ExceptionResultEnum.ERROR.exception("计划下已导入考务数据，不允许多次导入");
        }
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        try {
            Map<String, Object> map = printCommonService.saveTask(SystemConstant.convertIdToLong(printPlanId), file, TaskTypeEnum.EXAMINATION_IMPORT);
            boolean lock = redisUtil.lock(SystemConstant.REDIS_LOCK_IMPORT_EXAM_DATA + schoolId, SystemConstant.REDIS_LOCK_IMPORT_EXAM_DATA_TIME_OUT);
            if (lock) {
                asyncExamStudentImportService.importTask(map);
            }
            TBTask tbTask = Objects.nonNull(map.get(SystemConstant.TASK)) ? (TBTask) map.get(SystemConstant.TASK) : null;
            return tbTask;
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            redisUtil.releaseLock(SystemConstant.REDIS_LOCK_IMPORT_EXAM_DATA + schoolId);
        }
    }

    @Transactional
    @Override
    public void deleteExaminationDataByExamDetailIds(List<Long> examDetailIdList) {
        if (CollectionUtils.isEmpty(examDetailIdList)) {
            return;
        }
        // 删除exam_detail 表数据
        this.remove(new QueryWrapper<ExamDetail>().lambda().in(ExamDetail::getId, examDetailIdList));

        // 准备删除的考务-科目表id
        List<Long> examDetailCourseIds = examDetailCourseService.list(new QueryWrapper<ExamDetailCourse>().lambda().in(ExamDetailCourse::getExamDetailId, examDetailIdList))
                .stream().map(ExamDetailCourse::getId).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(examDetailCourseIds)) {
            return;
        }
        // 删除exam_detail_course 表数据
        examDetailCourseService.remove(new QueryWrapper<ExamDetailCourse>().lambda().in(ExamDetailCourse::getId, examDetailCourseIds));

        // 准备删除的考务-考生表id
        List<Long> examStudentIds = examStudentService.list(new QueryWrapper<ExamStudent>().lambda().in(ExamStudent::getExamDetailCourseId, examDetailCourseIds))
                .stream().map(ExamStudent::getId).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(examStudentIds)) {
            return;
        }
        // 删除exam_student 表数据
        examStudentService.remove(new QueryWrapper<ExamStudent>().lambda().in(ExamStudent::getId, examStudentIds));
    }

    @Transactional
    @Override
    public void resetExamDetail(Long entityId, CreatePdfTypeEnum type, ExamDetailStatusEnum status, boolean clearPaperType) {
        // 清掉客户端产生的数据，包含缓存状态、校验状态
        ExamDetail examDetail = this.getById(entityId);
        if (CreatePdfTypeEnum.ALL.equals(type) || CreatePdfTypeEnum.PAPER.equals(type) || CreatePdfTypeEnum.CARD_A3.equals(type)) {
            QueryWrapper<ClientStatus> clientStatusQueryWrapper = new QueryWrapper<>();
            clientStatusQueryWrapper.lambda().eq(ClientStatus::getSchoolId, examDetail.getSchoolId())
                    .eq(ClientStatus::getExamDetailId, entityId);
            List<ClientStatus> clientStatusList = clientStatusService.list(clientStatusQueryWrapper);
            if (!CollectionUtils.isEmpty(clientStatusList)) {
                for (ClientStatus clientStatus : clientStatusList) {
                    clientStatus.setPass(false);
                    clientStatus.setDownload(false);
                }
                clientStatusService.updateBatchById(clientStatusList);
            }

            // 清掉客户端产生的数据和考场文件，包含状态、校验状态，pdf文件路径，打印信息
            UpdateWrapper<ExamDetail> examDetailUpdateWrapper = new UpdateWrapper<>();
            LambdaUpdateWrapper<ExamDetail> lambdaUpdateWrapper = examDetailUpdateWrapper.lambda();
            lambdaUpdateWrapper.set(ExamDetail::getStatus, status)
                    .set(ExamDetail::getValidate, false).set(ExamDetail::getPrintProgress, null)
                    .set(ExamDetail::getPrintStartTime, null)
                    .set(ExamDetail::getPrintEndTime, null);
            if (CreatePdfTypeEnum.ALL.equals(type) || CreatePdfTypeEnum.PAPER.equals(type)) {
                lambdaUpdateWrapper.set(ExamDetail::getAttachmentId, null);
            }
            if (CreatePdfTypeEnum.ALL.equals(type) || CreatePdfTypeEnum.CARD_A3.equals(type)) {
                lambdaUpdateWrapper.set(ExamDetail::getCardAttachmentId, null);
            }
            if (CreatePdfTypeEnum.ALL.equals(type)) {
                lambdaUpdateWrapper.set(ExamDetail::getAttachmentPath, null);
            }
            lambdaUpdateWrapper.set(ExamDetail::getMergePdfPath, null);
            lambdaUpdateWrapper.eq(ExamDetail::getId, entityId);
            this.update(examDetailUpdateWrapper);
        }

        if (clearPaperType) {
            // 清掉考场-课程表中关联的试卷类型
            List<ExamDetailCourse> examDetailCourseList = examDetailCourseService.listByExamDetailId(entityId);
            if (!CollectionUtils.isEmpty(examDetailCourseList)) {
                for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                    examDetailCourse.setSerialNumber(null);
                    examDetailCourse.setPaperType(null);
                }
                examDetailCourseService.updateBatchById(examDetailCourseList);
            }
        }
    }

    @Override
    public List<ExamDetailCourseInitMarkDto> listPrintFinishExamDetailCourse(String printStatus, long startTime) {
        List<ExamDetailCourseInitMarkDto> examDetailCourseInitMarkDtoList = this.baseMapper.listPrintFinishExamDetailCourse(printStatus, startTime);
        return examDetailCourseInitMarkDtoList.stream().filter(t -> this.allExamDetailIsFinish(t.getExamId(), t.getCourseId(), t.getPaperNumber())).collect(Collectors.toList());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void deleteExaminationData(Long printPlanId, ExamDataSourceEnum source) {
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(printPlanId);
        if (examPrintPlan == null) {
            throw ExceptionResultEnum.ERROR.exception("未找到印刷计划信息");
        }
        PrintPlanStatusEnum status = examPrintPlan.getStatus();
        if (PrintPlanStatusEnum.NEW != status && PrintPlanStatusEnum.READY != status) {
            throw ExceptionResultEnum.ERROR.exception(status.getName() + "状态下的印刷计划不能删除考务数据");
        }
        // 准备删除的考务-场次表id
        LambdaQueryWrapper<ExamDetail> examDetailQueryWrapper = new QueryWrapper<ExamDetail>().lambda().eq(ExamDetail::getPrintPlanId, printPlanId);
        if (Objects.nonNull(source)) {
            examDetailQueryWrapper.eq(ExamDetail::getExamDataSource, source);
        }
        List<Long> examDetailIds = this.list(examDetailQueryWrapper).stream().map(ExamDetail::getId).collect(Collectors.toList());
        if (examDetailIds.size() == 0) {
            return;
        }
        // 删除exam_detail 表数据
        this.remove(new QueryWrapper<ExamDetail>().lambda().in(ExamDetail::getId, examDetailIds));

        // 准备删除的考务-科目表id
        List<Long> examDetailCourseIds = examDetailCourseService.list(new QueryWrapper<ExamDetailCourse>().lambda().in(ExamDetailCourse::getExamDetailId, examDetailIds))
                .stream().map(ExamDetailCourse::getId).collect(Collectors.toList());
        if (examDetailCourseIds.size() == 0) {
            return;
        }
        // 删除exam_detail_course 表数据
        examDetailCourseService.remove(new QueryWrapper<ExamDetailCourse>().lambda().in(ExamDetailCourse::getId, examDetailCourseIds));

        // 准备删除的考务-考生表id
        List<Long> examStudentIds = examStudentService.list(new QueryWrapper<ExamStudent>().lambda().in(ExamStudent::getExamDetailCourseId, examDetailCourseIds))
                .stream().map(ExamStudent::getId).collect(Collectors.toList());
        if (examStudentIds.size() == 0) {
            return;
        }
        // 删除exam_student 表数据
        examStudentService.remove(new QueryWrapper<ExamStudent>().lambda().in(ExamStudent::getId, examStudentIds));
    }

    private List<CodeNameEnableValue> getExtendFieldsByFields(List<FieldsDto> fieldsDtoList) {
        List<CodeNameEnableValue> extendFieldsDtoList = new ArrayList<>();
        for (FieldsDto fieldsDto : fieldsDtoList) {
            CodeNameEnableValue codeNameEnableValue = new CodeNameEnableValue();
            BeanUtils.copyProperties(fieldsDto, codeNameEnableValue);
            extendFieldsDtoList.add(codeNameEnableValue);
        }
        return extendFieldsDtoList;
    }

    /**
     * 根据clazzIdSet集合查询班级名称集合
     *
     * @param clazzIdSet 班级idset
     * @return 班级名称集合
     */
    private List<String> findClazzNamesByClazzIdSet(Set<Long> clazzIdSet) {
        Set<String> clazzNameSet = new HashSet<>();
//        if (clazzIdSet.size() > 0) {
//            Set<String> basicClazzSet = basicClazzService.list(new QueryWrapper<BasicClazz>().lambda().in(BasicClazz::getId, clazzIdSet)).stream().map(BasicClazz::getClazzName).collect(Collectors.toSet());
//            Set<String> teachClazzSet = teachClazzService.list(new QueryWrapper<TeachClazz>().lambda().in(TeachClazz::getId, clazzIdSet)).stream().map(TeachClazz::getClazzName).collect(Collectors.toSet());
//            clazzNameSet.addAll(basicClazzSet);
//            clazzNameSet.addAll(teachClazzSet);
//        }
        return new ArrayList<>(clazzNameSet);
    }

    @Override
    public int countMakeupUsedByExamIdAndPaperNumber(Long examId, String paperNumber) {
        return this.baseMapper.countMakeupUsedByExamIdAndPaperNumber(examId, paperNumber);
    }

    @Override
    public void updateStatusById(Long id, ExamDetailStatusEnum examDetailStatusEnum) {
        UpdateWrapper<ExamDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamDetail::getStatus, examDetailStatusEnum)
                .eq(ExamDetail::getId, id);
        this.update(updateWrapper);
    }

    @Override
    public List<ExamDetailCourseInitMarkDto> listPrintFinishPackageCode(Long examId, String paperNumber, Integer serialNumber) {
        return this.baseMapper.listPrintFinishPackageCode(examId, paperNumber, serialNumber);
    }

    @Override
    public List<InitExamDetailDto> listPrintFinishExamDetailCourseIds(Long examId, String paperNumber, Integer serialNumber) {
        return this.baseMapper.listPrintFinishExamDetailCourseIds(examId, paperNumber, serialNumber);
    }

    @Override
    public boolean allExamDetailIsExist(Long examId, Long courseId, String paperNumber) {
        List<ExamDetailStatusEnum> examDetailStatusList = Arrays.asList(ExamDetailStatusEnum.READY, ExamDetailStatusEnum.WAITING, ExamDetailStatusEnum.PRINTING, ExamDetailStatusEnum.FINISH);
        List<ExamDetail> examDetailList = this.listExamDetailByExamIdAndCourseIdAndPaperNumber(examId, courseId, paperNumber);
        return examDetailList.stream().filter(m -> !examDetailStatusList.contains(m.getStatus())).count() == 0;
    }

    @Override
    public boolean allExamDetailIsFinish(Long examId, Long courseId, String paperNumber) {
        List<ExamDetail> examDetailList = this.listExamDetailByExamIdAndCourseIdAndPaperNumber(examId, courseId, paperNumber);
        return examDetailList.stream().filter(m -> !ExamDetailStatusEnum.FINISH.equals(m.getStatus())).count() == 0;
    }

    @Override
    public List<ExamDetail> listExamDetailByExamIdAndCourseIdAndPaperNumber(Long examId, Long courseId, String paperNumber) {
        return this.baseMapper.listExamDetailByExamIdAndCourseIdAndPaperNumber(examId, courseId, paperNumber);
    }

    @Transactional
    @Override
    public List<Long> saveImportExamStudent(ExamPrintPlan examPrintPlan, List<BasicExamStudent> basicExamStudentList, Long createId) {
        // 校区（考点）为非必填项
        BasicExamRule basicExamRule = basicExamRuleService.getBySchoolId(examPrintPlan.getSchoolId());
        Optional<EnumResult> optional = basicExamRule.getRequiredFieldList().stream().filter(m -> RequiredFieldsEnum.EXAM_PLACE.getCode().equals(m.getCode()) && m.getEnable()).findFirst();
        Map<String, List<BasicExamStudent>> examDetailMap;
        if (optional.isPresent()) {
            examDetailMap = basicExamStudentList.stream().collect(Collectors.groupingBy(m -> m.getExamPlace().concat(m.getExamRoom()).concat(m.getExamStartTime().toString()).concat(m.getExamEndTime().toString())));
        } else {
            examDetailMap = basicExamStudentList.stream().collect(Collectors.groupingBy(m -> m.getExamRoom().concat(m.getExamStartTime().toString()).concat(m.getExamEndTime().toString())));
        }
        List<ExamDetail> examDetailList = new ArrayList<>();
        List<ExamDetailCourse> examDetailCourseList = new ArrayList<>();
        List<ExamStudent> examStudentList = new ArrayList<>();
        // 考点
        examDetailMap.forEach((examDetailKey, examDetailValues) -> {
            BasicExamStudent examDetailInfo = examDetailValues.get(0);
            ExamDetail examDetail = new ExamDetail();
            examDetail.setId(SystemConstant.getDbUuid());
            examDetail.setPackageCode("1" + redisCounterUtil.getCounter(examPrintPlan.getSchoolId(), 6, "packageCode"));
            examDetail.setSchoolId(examPrintPlan.getSchoolId());
            examDetail.setPrintPlanId(examPrintPlan.getId());
            examDetail.setPrintPlanName(examPrintPlan.getName());
            examDetail.setExamId(examPrintPlan.getExamId());
            examDetail.setExamPlace(examDetailInfo.getExamPlace());
            examDetail.setExamRoom(examDetailInfo.getExamRoom());
            examDetail.setStatus(ExamDetailStatusEnum.NEW);
            examDetail.setTotalSubjects(examDetailValues.size());
            examDetail.setExamStartTime(examDetailInfo.getExamStartTime());
            examDetail.setExamEndTime(examDetailInfo.getExamEndTime());
            examDetail.setExamDataSource(ExamDataSourceEnum.FILE_IMPORT);
            examDetail.setNormal(true);
            examDetail.setCreateId(createId);
            examDetail.setBackupCount(examPrintPlan.getBackupCount());
            examDetailList.add(examDetail);
            // 考点课程
            Map<String, List<BasicExamStudent>> examDetailCourseMap = examDetailValues.stream().collect(Collectors.groupingBy(m -> m.getCourseId().toString().concat(m.getPaperNumber())));
            examDetailCourseMap.forEach((examDetailCourseKey, examDetailCourseValues) -> {
                BasicExamStudent examDetailCourseInfo = examDetailCourseValues.get(0);
                ExamDetailCourse examDetailCourse = new ExamDetailCourse();
                examDetailCourse.setId(SystemConstant.getDbUuid());
                examDetailCourse.setSchoolId(examPrintPlan.getSchoolId());
                examDetailCourse.setExamDetailId(examDetail.getId());
                examDetailCourse.setCourseId(examDetailCourseInfo.getCourseId());
                examDetailCourse.setPaperNumber(examDetailCourseInfo.getPaperNumber());
                examDetailCourse.setCoursePaperId(examDetailCourseInfo.getCoursePaperId());
                examDetailCourse.setTotalSubjects(examDetailCourseValues.size());
                examDetailCourse.setCreateId(createId);
                String clazzName = examDetailCourseValues.stream().filter(m -> StringUtils.isNotBlank(m.getTeachClassName())).map(BasicExamStudent::getTeachClassName).distinct().collect(Collectors.joining(","));
                if (StringUtils.isNotBlank(clazzName)) {
                    examDetailCourse.setClazzName(clazzName);
                }
                examDetailCourseList.add(examDetailCourse);

                // 考生
                examDetailCourseValues.forEach(s -> {
                    ExamStudent examStudent = new ExamStudent();
                    examStudent.setId(SystemConstant.getDbUuid());
                    examStudent.setSchoolId(examPrintPlan.getSchoolId());
                    examStudent.setExamId(examPrintPlan.getExamId());
                    examStudent.setExamDetailCourseId(examDetailCourse.getId());
                    examStudent.setPaperNumber(s.getPaperNumber());
                    examStudent.setCoursePaperId(s.getCoursePaperId());
                    examStudent.setBasicStudentId(s.getId());
                    examStudent.setTeachClazzName(s.getTeachClassName());
                    examStudent.setStudentCode(s.getStudentCode());
                    examStudent.setCreateId(createId);
                    examStudent.setCreateTime(System.currentTimeMillis());
                    examStudentList.add(examStudent);
                });
            });
        });

        // 保存考场、考场课程、考生数据
        this.saveBatch(examDetailList);
        examDetailCourseService.saveBatch(examDetailCourseList);
        examStudentService.saveBatch(examStudentList);
        return examDetailList.stream().map(ExamDetail::getId).collect(Collectors.toList());
    }

    @Override
    public int countByPrintPlanId(String printPlanId) {
        QueryWrapper<ExamDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamDetail::getPrintPlanId, printPlanId);
        return this.count(queryWrapper);
    }

    @Override
    public void updateTotalSubjects(Long id, int totalSubjects) {
        UpdateWrapper<ExamDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamDetail::getTotalSubjects, totalSubjects)
                .eq(ExamDetail::getId, id);
        this.update(updateWrapper);
    }

    @Override
    public void updateMergePdfPath(Long id, String mergePdfPath) {
        UpdateWrapper<ExamDetail> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamDetail::getMergePdfPath, mergePdfPath)
                .eq(ExamDetail::getId, id);
        this.update(updateWrapper);
    }

    /**
     * 根据考试id和条码查找
     *
     * @param examId
     * @param packageCode
     * @return
     */
    @Override
    public ExamDetail findByExamIdAndPackageCode(Long examId, String packageCode) {
        QueryWrapper<ExamDetail> examDetailQueryWrapper = new QueryWrapper<>();
        examDetailQueryWrapper.lambda().eq(ExamDetail::getExamId, examId).eq(ExamDetail::getPackageCode, packageCode);
        return this.getOne(examDetailQueryWrapper);
    }

    /**
     * 条码校验
     *
     * @param semesterId
     * @param examId
     * @param packageCodeStart
     * @param packageCodeEnd
     * @return
     */
    @Override
    public Boolean packageCodeVerify(Long semesterId, Long examId, String packageCodeStart, String packageCodeEnd) {
        BasicSemester basicSemester = basicSemesterService.getById(semesterId);
        Objects.requireNonNull(basicSemester, "未找到学期信息");

        ExamDetail examDetail = this.findByExamIdAndPackageCode(examId, packageCodeStart);
        Objects.requireNonNull(examDetail, "起始条码有误，请重新扫描");

        String tempCode = StringUtils.substring(packageCodeEnd, 0, packageCodeEnd.length() - 2);
        if (!Objects.equals(tempCode, examDetail.getPackageCode())) {
            throw ExceptionResultEnum.ERROR.exception("结束条码有误，请重新扫描");
        }
        return true;
    }
}
