package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.entity.TBVersion;
import com.qmth.distributed.print.business.mapper.TBVersionMapper;
import com.qmth.distributed.print.business.service.ClientUpgradeService;
import com.qmth.distributed.print.business.service.TBVersionService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysConfig;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * <p>
 * 版本管理表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2025-02-27
 */
@Service
public class TBVersionServiceImpl extends ServiceImpl<TBVersionMapper, TBVersion> implements TBVersionService {

    @Override
    public TBVersion getByClientVersion(String version) {
        QueryWrapper<TBVersion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TBVersion::getClientVersion, version);
        return this.getOne(queryWrapper);
    }

    /**
     * 版本管理分页列表
     *
     * @param iPage
     * @return
     */
    @Override
    public IPage<TBVersion> versionList(IPage<Map> iPage) {
        return this.baseMapper.versionList(iPage);
    }

    @Override
    public void validVersion(String version) {
        TBVersion tbVersion = this.getByClientVersion(version);
        if (tbVersion == null || !StringUtils.equalsAny(SystemConstant.VERSION_VALUE, tbVersion.listSoftVersion().toArray(new String[0]))) {
            throw ExceptionResultEnum.CLIENT_VERSION_ERROR.exception("客户端版本号[" + version + "]，无法使用当前服务端，请联系管理员处理");
        }
    }
}
