package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.distributed.print.business.bean.dto.excel.TeacherImportDto;
import com.qmth.distributed.print.business.bean.params.TeachCourseSelectParam;
import com.qmth.distributed.print.business.bean.result.CourseResult;
import com.qmth.distributed.print.business.bean.result.TeachCourseResult;
import com.qmth.distributed.print.business.entity.TeachClazz;
import com.qmth.distributed.print.business.entity.TeachCourse;
import com.qmth.distributed.print.business.entity.TeachStudent;
import com.qmth.distributed.print.business.mapper.TeachCourseMapper;
import com.qmth.distributed.print.business.service.TeachClazzService;
import com.qmth.distributed.print.business.service.TeachCourseService;
import com.qmth.distributed.print.business.service.TeachStudentService;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.result.DictionaryResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.BasicExamStudent;
import com.qmth.teachcloud.common.entity.SysOrg;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FieldUniqueEnum;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description: 教学课程服务实现类
 * @Author: CaoZixuan
 * @Date: 2022-02-07
 */
@Service
public class TeachCourseServiceImpl extends ServiceImpl<TeachCourseMapper, TeachCourse> implements TeachCourseService {

    @Resource
    private BasicCourseService basicCourseService;

    @Resource
    private SysOrgService sysOrgService;

    @Resource
    private SysUserService sysUserService;

    @Resource
    private TeachClazzService teachClazzService;

    @Resource
    private TeachStudentService teachStudentService;

    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;

    @Resource
    private TeachcloudCommonService teachcloudCommonService;

    @Override
    public IPage<TeachCourseResult> teachCoursePage(Long examId, String courseName, int pageNumber, int pageSize) {
        courseName = SystemConstant.translateSpecificSign(courseName);
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(requestUser.getSchoolId(),
                requestUser.getId(), ServletUtil.getRequest().getServletPath());
        return this.baseMapper.findTeachCoursePage(new Page<>(pageNumber, pageSize), examId, courseName, requestUser.getSchoolId(), dpr);
    }

    @Transactional
    @Override
    public void selectCreate(TeachCourseSelectParam teachCourseSelectParam) {
        Long examId = teachCourseSelectParam.getExamId();
        List<Long> courseIds = teachCourseSelectParam.getCourseIds();

        List<Long> userCourseIds = this.list(new QueryWrapper<TeachCourse>().lambda().eq(TeachCourse::getExamId, examId)
                        .eq(TeachCourse::getUserId, teachCourseSelectParam.getUserId())).stream().map(TeachCourse::getCourseId)
                .collect(Collectors.toList());

        List<TeachCourse> teachCourseList = new ArrayList<>();
        for (Long courseId : courseIds) {
            if (CollectionUtils.isNotEmpty(courseIds) && userCourseIds.contains(courseId)) {
                continue;
            }
            TeachCourse teachCourse = new TeachCourse();
            teachCourse.setSchoolId(teachCourseSelectParam.getSchoolId());
            teachCourse.setExamId(examId);
            teachCourse.setUserId(teachCourseSelectParam.getUserId());
            teachCourse.setEnable(true);
            teachCourse.setWeightSetting(false);
            teachCourse.setCourseId(courseId);
            teachCourse.insertInfo(teachCourseSelectParam.getUserId());
            teachCourseList.add(teachCourse);
        }

        if (CollectionUtils.isNotEmpty(teachCourseList)) {
            this.saveBatch(teachCourseList);
        }
    }

    @Override
    public List<DictionaryResult> findBasicCourseByUser(SysUser requestUser) {
        Long userId = requestUser.getId();
        Set<Long> orgIdSet = sysOrgService.findDeepOrgIdListByUserId(userId);
        QueryWrapper<BasicCourse> basicCourseQueryWrapper = new QueryWrapper<>();
        basicCourseQueryWrapper.lambda().eq(BasicCourse::getEnable, true);
        if (orgIdSet.size() > 0) {
            basicCourseQueryWrapper.lambda().in(BasicCourse::getTeachingRoomId, orgIdSet);
        }
        List<BasicCourse> basicCourseList = basicCourseService.list(basicCourseQueryWrapper);

        List<DictionaryResult> resultList = new ArrayList<>();
        for (BasicCourse basicCourse : basicCourseList) {
            DictionaryResult dictionaryResult = new DictionaryResult();
            BeanUtils.copyProperties(basicCourse, dictionaryResult);
            resultList.add(dictionaryResult);
        }
        return resultList;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void deleteTeachCourseBatch(List<Long> idList) {
        for (Long teachCourseId : idList) {
            // 删除教学学生
            teachStudentService.remove(
                    new QueryWrapper<TeachStudent>().lambda().eq(TeachStudent::getTeachCourseId, teachCourseId));
            // 删除教学班
            teachClazzService.remove(
                    new QueryWrapper<TeachClazz>().lambda().eq(TeachClazz::getTeachCourseId, teachCourseId).or()
                            .eq(TeachClazz::getBelongTeachCourseId, teachCourseId));

            // 删除教学课程
            this.removeById(teachCourseId);
        }
    }

    @Override
    public TeachCourse findByExamIdCourseIdAndUserId(Long examId, Long courseId, Long userId) {
        TeachCourse teachCourse = this.getOne(
                new QueryWrapper<TeachCourse>().lambda().eq(TeachCourse::getExamId, examId).eq(TeachCourse::getCourseId, courseId).eq(TeachCourse::getUserId, userId)
                        .last(SystemConstant.LIMIT1));
        if (Objects.isNull(teachCourse)) {
            throw ExceptionResultEnum.ERROR.exception("教学课程不存在");
        }
        return teachCourse;
    }

    @Override
    public List<DictionaryResult> findTeacherCourseInfo(Long userId) {
        SysUser sysUser = sysUserService.getById(userId);
        if (Objects.isNull(sysUser)) {
            throw ExceptionResultEnum.ERROR.exception("用户不存在");
        }
        List<Long> courseCodeList = this.list(
                new QueryWrapper<TeachCourse>().lambda().eq(TeachCourse::getUserId, userId)).stream().map(TeachCourse::getCourseId).distinct().collect(Collectors.toList());
        return courseCodeList.stream().flatMap(e -> {
            BasicCourse basicCourse = basicCourseService.getById(e);
            DictionaryResult dictionaryResult = new DictionaryResult();
            dictionaryResult.setCode(basicCourse.getCode());
            dictionaryResult.setName(basicCourse.getName());
            return Stream.of(dictionaryResult);
        }).collect(Collectors.toList());
    }

    @Override
    public CourseResult listUserCourse(String param, Long examId, Long teachingRoomId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        CourseResult courseResult = new CourseResult();
        // 教学课程
        List<BasicCourse> teachCourseList = this.listByUserId(examId, sysUser.getId());
        courseResult.setTeachCourseList(teachCourseList);

        Set<Long> orgIds = teachcloudCommonService.listSubOrgIds(teachingRoomId);
        QueryWrapper<BasicCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().in(BasicCourse::getTeachingRoomId, orgIds).eq(BasicCourse::getEnable, true);
        if (StringUtils.isNotBlank(param)) {
            queryWrapper.lambda()
                    .and(i -> i.like(BasicCourse::getCode, SystemConstant.translateSpecificSign(param)).or().like(BasicCourse::getName, SystemConstant.translateSpecificSign(param)));
        }
        List<BasicCourse> basicCourseList = basicCourseService.list(queryWrapper);
        courseResult.setBasicCourseList(basicCourseList);

        return courseResult;
    }

    @Override
    public List<BasicCourse> listByUserId(Long examId, Long userId) {
        QueryWrapper<TeachCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TeachCourse::getExamId, examId);
        queryWrapper.lambda().eq(TeachCourse::getUserId, userId);
        List<TeachCourse> list = this.list(queryWrapper);
        return list.stream().map(m -> basicCourseService.getById(m.getCourseId())).collect(Collectors.toList());
    }

    @Override
    public void batchCreateTeachCourse(Long schoolId, Long examId, List<BasicExamStudent> basicExamStudentList) {
        Map<Long, List<BasicExamStudent>> listMap = basicExamStudentList.stream().filter(e -> Objects.nonNull(e.getTeacherId()))
                .collect(Collectors.groupingBy(BasicExamStudent::getTeacherId));
        for (Map.Entry<Long, List<BasicExamStudent>> entry : listMap.entrySet()) {
            TeachCourseSelectParam teachCourseSelectParam = new TeachCourseSelectParam();
            teachCourseSelectParam.setSchoolId(schoolId);
            teachCourseSelectParam.setExamId(examId);
            teachCourseSelectParam.setUserId(entry.getKey());
            teachCourseSelectParam.setCourseIds(entry.getValue().stream().map(BasicExamStudent::getCourseId).distinct().collect(Collectors.toList()));
            selectCreate(teachCourseSelectParam);
        }
    }

    /**
     * 课程任课老师导入
     *
     * @param file
     * @param examId
     * @return
     */
    @Override
    @Transactional
    public Map<String, String> teachCourseImport(MultipartFile file, Long examId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Map<String, String> messageMap = new LinkedHashMap<>();
        try {
            StringJoiner errorData = new StringJoiner("");
            StringJoiner successData = new StringJoiner("");

            ExcelReader excelReader = ExcelReader.create(ExcelType.XLSX, file.getInputStream(), 1);
            List<TeacherImportDto> teacherImportDtoList = excelReader.getObjectList(TeacherImportDto.class);
            if (CollectionUtils.isNotEmpty(teacherImportDtoList)) {
                String[] columnNames = excelReader.getColumnNames();
                for (int i = 0; i < columnNames.length; i++) {
                    if (i == 0 && Objects.nonNull(columnNames[i]) && !columnNames[i].trim().contains("课程代码")) {
                        throw ExceptionResultEnum.ERROR.exception("excel表头第一行为课程代码");
                    } else if (i == 1 && Objects.nonNull(columnNames[i]) && !columnNames[i].trim().contains("课程名称")) {
                        throw ExceptionResultEnum.ERROR.exception("excel表头第二行为课程名称");
                    } else if (i == 2 && Objects.nonNull(columnNames[i]) && !columnNames[i].trim().contains("开课学院")) {
                        throw ExceptionResultEnum.ERROR.exception("excel表头第三行为开课学院");
                    } else if (i == 3 && Objects.nonNull(columnNames[i]) && !columnNames[i].trim().contains("任课老师工号")) {
                        throw ExceptionResultEnum.ERROR.exception("excel表头第四行为任课老师工号");
                    }
                }

                List<TeachCourse> teachCourseList = new ArrayList<>(teacherImportDtoList.size());
                Set<TeacherImportDto> teacherImportDtoSet = new HashSet<>();
                for (int i = 0; i < teacherImportDtoList.size(); i++) {
                    TeacherImportDto teacherImportDto = teacherImportDtoList.get(i);
                    if (!teacherImportDtoSet.add(teacherImportDto)) {
                        errorData.add("excel第").add((i + 1) + "").add("行数据重复").add("\r\n");
                    }
                    if (Objects.isNull(teacherImportDto.getCourseCode()) || Objects.equals(teacherImportDto.getCourseCode().trim(), "")) {
                        errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[0] + "]为空;").add("\r\n");
                    }
                    if (Objects.isNull(teacherImportDto.getCourseName()) || Objects.equals(teacherImportDto.getCourseName().trim(), "")) {
                        errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[1] + "]为空;").add("\r\n");
                    }
                    if (Objects.isNull(teacherImportDto.getTeachingRoomName()) || Objects.equals(teacherImportDto.getTeachingRoomName().trim(), "")) {
                        errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[2] + "]为空;").add("\r\n");
                    }
                    if (Objects.isNull(teacherImportDto.getTeacherCode()) || Objects.equals(teacherImportDto.getTeacherCode().trim(), "")) {
                        errorData.add("excel第").add((i + 1) + "").add("行[").add(columnNames[3] + "]为空;").add("\r\n");
                    }
                    basicCourseService.findByCourseCode(teacherImportDto.getCourseCode(), sysUser.getSchoolId());
                    SysOrg sysOrg = sysOrgService.findOrg(sysUser.getSchoolId(), teacherImportDto.getTeachingRoomName());
                    SysUser teacherUser = sysUserService.findByCode(sysUser.getSchoolId(), teacherImportDto.getTeacherCode());
                    BasicCourse basicCourse = basicCourseService.getByTeachRoomIdAndCode(sysOrg.getId(), teacherImportDto.getCourseCode());
                    Objects.requireNonNull(basicCourse, "课程代码[" + teacherImportDto.getCourseCode() + "]的开课学院[" + teacherImportDto.getTeachingRoomName() + "]不匹配");
                    teachCourseList.add(new TeachCourse(sysUser.getSchoolId(), examId, basicCourse.getId(), teacherUser.getId(), sysUser.getId()));
                }
                successData.add("共导入").add(teachCourseList.size() + "").add("条数据");
                this.saveOrUpdateBatch(teachCourseList);
            }
            messageMap.put(SystemConstant.SUCCESS, successData.length() > 0 ? successData.toString() : "无");
            messageMap.put(SystemConstant.EXCEL_ERROR, errorData.length() > 0 ? errorData.toString() : "无");
        } catch (Exception e) {
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return messageMap;
    }
}