package com.qmth.distributed.print.business.templete.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.boot.tools.excel.model.DataMap;
import com.qmth.distributed.print.business.bean.dto.ObjectiveStructDto;
import com.qmth.distributed.print.business.bean.dto.ObjectiveStructImportDto;
import com.qmth.distributed.print.business.bean.dto.SubjectiveStructDto;
import com.qmth.distributed.print.business.bean.dto.SubjectiveStructImportDto;
import com.qmth.distributed.print.business.bean.dto.importFile.BasicExamStudentImport;
import com.qmth.distributed.print.business.bean.dto.importFile.BasicExamStudentParseDto;
import com.qmth.distributed.print.business.entity.BasicExamRule;
import com.qmth.distributed.print.business.entity.ExamDetail;
import com.qmth.distributed.print.business.entity.ExamPrintPlan;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.enums.PrintPlanStatusEnum;
import com.qmth.distributed.print.business.enums.RequiredFieldsEnum;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.service.ImportLogicService;
import com.qmth.teachcloud.common.bean.examRule.CodeNameEnableDisabledValue;
import com.qmth.teachcloud.common.bean.examRule.CodeNameEnableValue;
import com.qmth.teachcloud.common.bean.result.SysUserResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.*;
import com.qmth.teachcloud.common.enums.*;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.util.ConvertUtil;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.entity.MarkQuestionAnswer;
import com.qmth.teachcloud.mark.entity.MarkUserQuestion;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import com.qmth.teachcloud.mark.service.MarkQuestionAnswerService;
import com.qmth.teachcloud.mark.service.MarkQuestionService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description: 任务处理逻辑impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
@Service
public class ImportLogicServiceImpl implements ImportLogicService {
    private final static Logger log = LoggerFactory.getLogger(ImportLogicServiceImpl.class);

    @Resource
    private BasicExamRuleService basicExamRuleService;
    @Resource
    private BasicCourseService basicCourseService;
    @Resource
    private BasicExamService basicExamService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private SysOrgService sysOrgService;
    @Resource
    private BasicExamStudentService basicExamStudentService;
    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private ExamDetailService examDetailService;
    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private SysRoleService sysRoleService;
    @Resource
    private CommonCacheService commonCacheService;
    @Resource
    private TBTaskPdfService tbTaskPdfService;
    @Resource
    private ExamPrintPlanService examPrintPlanService;
    @Resource
    private ExamTaskService examTaskService;
    @Resource
    private ExamStudentService examStudentService;
    @Resource
    private MarkQuestionAnswerService markQuestionAnswerService;

    @Transactional
    @Override
    public Map<String, Object> executeImportBasicExamStudentLogic(Map<String, Object> map, StringJoiner stringJoinerSummary) {
        SysUser requestUser = (SysUser) map.get(SystemConstant.USER);
        InputStream inputStream = (InputStream) map.get("inputStream");
        Long examId = SystemConstant.convertIdToLong(String.valueOf(map.get("examId")));
        Long schoolId = requestUser.getSchoolId();
        BasicExam basicExam = basicExamService.getById(examId);
        if (Objects.isNull(basicExam)) {
            throw ExceptionResultEnum.ERROR.exception("考试不存在");
        }
        Long semesterId = SystemConstant.convertIdToLong(basicExam.getSemesterId());

        SystemConstant.addSummary(stringJoinerSummary, "开始解析导入文件内容");
        BasicExamStudentParseDto basicExamStudentParseDto;
        try {
            // 解析excel文件内容
            basicExamStudentParseDto = this.parseImportExamStudentData(schoolId, inputStream);
        } catch (Exception e) {
            SystemConstant.addSummary(stringJoinerSummary, "解析导入文件内容结束，解析失败。" + e.getMessage());
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        }
        SystemConstant.addSummary(stringJoinerSummary, "解析导入文件内容结束，开始进行数据校验");

        List<BasicExamStudentImport> basicExamStudentImportList = basicExamStudentParseDto.getBasicExamStudentImportList();
        if (CollectionUtils.isEmpty(basicExamStudentImportList)) {
            throw ExceptionResultEnum.ERROR.exception("未读取到有效数据");
        }

        List<BasicExamStudent> basicExamStudentList = basicExamStudentService.list(new QueryWrapper<BasicExamStudent>().lambda()
                .select(BasicExamStudent::getId, BasicExamStudent::getCourseId, BasicExamStudent::getStudentCode, BasicExamStudent::getPaperNumber, BasicExamStudent::getPaperType, BasicExamStudent::getExamStartTime, BasicExamStudent::getExamEndTime)
                .eq(BasicExamStudent::getSchoolId, schoolId).eq(BasicExamStudent::getExamId, examId));
        Map<String, BasicExamStudent> courseIdStudentCodeMap = basicExamStudentList.stream().collect(Collectors.toMap(k -> k.getCourseId() + SystemConstant.HYPHEN + k.getStudentCode(), Function.identity(), (v1, v2) -> v1));

        BasicSchool basicSchool = commonCacheService.schoolCache(schoolId);
        // 学校设置强制校验卷型或者模式4
        boolean needPaperNumber = (basicSchool.getHasPaperNumber() != null && basicSchool.getHasPaperNumber());
        // 试卷编号->课程
        Map<String, Long> paperNumberCourseIdMap = new HashMap<>();
        // 试卷编号->考试时间
        Map<String, String> paperNumberExamTimeMap = basicExamStudentList.stream().filter(m -> StringUtils.isNotBlank(m.getPaperNumber())).collect(Collectors.toMap(BasicExamStudent::getPaperNumber, m -> m.getExamStartTime() + "-" + m.getExamEndTime(), (v1, v2) -> v1));
        Map<String, Long> basicCourseIdMap = new HashMap<>();
        // 课程id+教学班名称->任课老师Id
        Map<String, Long> courseTeachClassTeacherMap = new HashMap<>();
        // 课程学号唯一
        List<String> courseCodeStudentCodeList = new ArrayList<>();
        // 课程+考生在数据库中已存在的试卷编号和导入文件中不致的数据
        List<String> oldPaperNumberList = new ArrayList<>();
        // 是否有错误提示
        boolean hasError = false;

        List<BasicExamStudent> basicExamStudents = new ArrayList<>();
        for (BasicExamStudentImport basicExamStudentImport : basicExamStudentImportList) {
            BasicExamStudent basicExamStudent = new BasicExamStudent();
            BeanUtils.copyProperties(basicExamStudentImport, basicExamStudent);
            basicExamStudent.setSchoolId(schoolId);
            basicExamStudent.setSemesterId(semesterId);
            basicExamStudent.setExamId(examId);

            StringJoiner stringJoiner = new StringJoiner(";");
            // 校验excel内容
            basicExamStudentImport.getRequiredFieldList().forEach(m -> {
                // 模式4必填字段校验
                if (ExamModelEnum.MODEL4.equals(basicExam.getExamModel())) {
                    if (StringUtils.isBlank(basicExamStudentImport.getPaperNumber())) {
                        stringJoiner.add(RequiredFieldsEnum.PAPER_NUMBER.getName() + "必填");
                    }
                    if (StringUtils.isBlank(basicExamStudentImport.getStudentCode())) {
                        stringJoiner.add(RequiredFieldsEnum.STUDENT_CODE.getName() + "必填");
                    }
                    if (StringUtils.isBlank(basicExamStudentImport.getStudentName())) {
                        stringJoiner.add(RequiredFieldsEnum.STUDENT_NAME.getName() + "必填");
                    }
                    if (StringUtils.isBlank(basicExamStudentImport.getCourseCode())) {
                        stringJoiner.add(RequiredFieldsEnum.COURSE_CODE.getName() + "必填");
                    }
                    if (StringUtils.isBlank(basicExamStudentImport.getCourseName())) {
                        stringJoiner.add(RequiredFieldsEnum.COURSE_NAME.getName() + "必填");
                    }
                    if (StringUtils.isBlank(basicExamStudentImport.getTeachingRoomName())) {
                        stringJoiner.add(RequiredFieldsEnum.TEACHING_ROOM_NAME.getName() + "必填");
                    }
                    if (StringUtils.isBlank(basicExamStudent.getTeachClassName())) {
                        stringJoiner.add(RequiredFieldsEnum.TEACH_CLASS_NAME.getName() + "必填");
                    }
                } else {
                    // 通用规则中基础字段默认不可取消的，必填
                    if (m.getDisabled() && StringUtils.isBlank(m.getValue())) {
                        stringJoiner.add(m.getName() + "必填");
                    }
                    if (m.getCode().equals(RequiredFieldsEnum.PAPER_NUMBER.getCode()) && needPaperNumber) {
                        if (StringUtils.isBlank(m.getValue())) {
                            stringJoiner.add(m.getName() + "必填");
                        }
                    }
                }
//                // 试卷编号不为空时，校区（考点）必填
//                if (StringUtils.isNotBlank(basicExamStudent.getPaperNumber()) && m.getCode().equals(RequiredFieldsEnum.EXAM_PLACE.getCode()) && m.getEnable() && StringUtils.isBlank(basicExamStudent.getExamPlace())) {
//                    stringJoiner.add(RequiredFieldsEnum.EXAM_PLACE.getName() + "必填");
//                }
            });

            // 学号校验
            if (StringUtils.isNotBlank(basicExamStudent.getStudentCode()) && !basicExamStudent.getStudentCode().matches(SystemConstant.REGULAR_EXPRESSION_OF_STUDENT_CODE)) {
                stringJoiner.add("学号只能由大写字母、数字组成");
            }

            if (ExamModelEnum.MODEL4.equals(basicExam.getExamModel())) {
                if (basicExamStudent.getPaperType() == null) {
                    basicExamStudent.setPaperType(SystemConstant.DEFAULT_PAPER_TYPE_A);
                }
            }
            // 任课老师和任课老师工号都有值或都没值
            if (StringUtils.isNotBlank(basicExamStudentImport.getTeacherCode()) && StringUtils.isBlank(basicExamStudentImport.getTeacherName())) {
                stringJoiner.add(RequiredFieldsEnum.TEACHER_NAME.getName() + "必填");
            }
            if (StringUtils.isNotBlank(basicExamStudentImport.getTeacherName()) && StringUtils.isBlank(basicExamStudentImport.getTeacherCode())) {
                stringJoiner.add(RequiredFieldsEnum.TEACHER_CODE.getName() + "必填");
            }

            // 解析考试日期和考试时间
            if (StringUtils.isNoneBlank(basicExamStudentImport.getExamDate(), basicExamStudentImport.getExamTime())) {
                try {
                    Map<String, Object> timeMap = ConvertUtil.analyzeExamTime(basicExamStudentImport.getExamDate(), basicExamStudentImport.getExamTime());
                    basicExamStudent.setExamStartTime(Long.valueOf(String.valueOf(timeMap.get("startTime"))));
                    basicExamStudent.setExamEndTime(Long.valueOf(String.valueOf(timeMap.get("endTime"))));

                    // 反向更新examDate为标准日期格式，
                    for (CodeNameEnableDisabledValue codeNameEnableDisabledValue : basicExamStudentImport.getRequiredFieldList()) {
                        if (codeNameEnableDisabledValue.getCode().equals(RequiredFieldsEnum.EXAM_DATE.getCode())) {
                            codeNameEnableDisabledValue.setValue(String.valueOf(timeMap.get("dateStr")));
                        }
                    }
                } catch (Exception e) {
                    stringJoiner.add(e.getMessage());
                }
            }

            basicExamStudent.setRequiredFields(CollectionUtils.isNotEmpty(basicExamStudentImport.getRequiredFieldList()) ? JSON.toJSONString(basicExamStudentImport.getRequiredFieldList()) : null);
            basicExamStudent.setExtendFields(CollectionUtils.isNotEmpty(basicExamStudentImport.getExtendFieldList()) ? JSON.toJSONString(basicExamStudentImport.getExtendFieldList()) : null);

            // 校验课程开课学院和课程代码
            if (StringUtils.isNoneBlank(basicExamStudentImport.getCourseCode(), basicExamStudentImport.getTeachingRoomName())) {
                String key = basicExamStudentImport.getCourseCode() + SystemConstant.HYPHEN + basicExamStudentImport.getTeachingRoomName();
                if (!basicCourseIdMap.containsKey(key)) {
                    List<SysOrg> sysOrgList = sysOrgService.getSecondOrg(schoolId, basicExamStudentImport.getTeachingRoomName());
                    if (CollectionUtils.isEmpty(sysOrgList)) {
                        stringJoiner.add("开课学院[" + basicExamStudentImport.getTeachingRoomName() + "]在学院层级不存在");
                    } else if (sysOrgList.size() == 1) {
                        SysOrg sysOrg = sysOrgList.get(0);
                        // 校验课程代码和开课学院
                        BasicCourse basicCourse = basicCourseService.getByTeachRoomIdAndCode(sysOrg.getId(), basicExamStudentImport.getCourseCode());
                        if (Objects.isNull(basicCourse)) {
                            stringJoiner.add("课程代码[" + basicExamStudentImport.getCourseCode() + "]在开课学院[" + basicExamStudentImport.getTeachingRoomName() + "]不存在");
                        } else {
                            basicExamStudent.setCourseId(basicCourse.getId());
                            basicCourseIdMap.put(key, basicCourse.getId());
                        }
                    } else {
                        stringJoiner.add("开课学院[" + basicExamStudentImport.getTeachingRoomName() + "]存在多个相同值");
                    }
                } else {
                    basicExamStudent.setCourseId(basicCourseIdMap.get(key));
                }
            }

            // 校验任课老师工号和姓名
            if (StringUtils.isNoneBlank(basicExamStudentImport.getTeacherCode(), basicExamStudentImport.getTeacherName())) {
                SysUser sysUser = sysUserService.getByLoginName(schoolId, basicExamStudentImport.getTeacherCode());
                if (sysUser == null) {
                    stringJoiner.add("任课老师工号[" + basicExamStudentImport.getTeacherCode() + "]用户不存在");
                } else {
                    if (!sysUser.getEnable()) {
                        stringJoiner.add("任课老师工号[" + basicExamStudentImport.getTeacherCode() + "]用户已禁用");
                    } else if (!sysUser.getRealName().equals(basicExamStudentImport.getTeacherName())) {
                        stringJoiner.add("任课老师[" + basicExamStudentImport.getTeacherName() + "]与用户管理中姓名[" + sysUser.getRealName() + "]不一致");
                    } else {
                        basicExamStudent.setTeacherId(sysUser.getId());
                    }
                }

                // 校验同课程下，同一教学班只能有一个任课老师
                String courseTeachClassTeacherKey = basicExamStudent.getCourseId() + basicExamStudent.getTeachClassName();
                if (courseTeachClassTeacherMap.containsKey(courseTeachClassTeacherKey)) {
                    Long teacherId = courseTeachClassTeacherMap.get(courseTeachClassTeacherKey);
                    if ((teacherId == null && basicExamStudent.getTeacherId() != null) || (teacherId != null && basicExamStudent.getTeacherId() == null) || (teacherId != null && !teacherId.equals(basicExamStudent.getTeacherId()))) {
                        stringJoiner.add("同课程同教学班，只允许设置一个任课老师");
                    }
                } else {
                    courseTeachClassTeacherMap.put(courseTeachClassTeacherKey, basicExamStudent.getTeacherId());
                }
            }

            if (basicExamStudent.getCourseId() != null) {
                if (courseIdStudentCodeMap.containsKey(basicExamStudent.getCourseId() + SystemConstant.HYPHEN + basicExamStudent.getStudentCode())) {
                    BasicExamStudent student = courseIdStudentCodeMap.get(basicExamStudent.getCourseId() + SystemConstant.HYPHEN + basicExamStudent.getStudentCode());
                    if (StringUtils.isNotBlank(student.getPaperNumber()) && !student.getPaperNumber().equals(basicExamStudent.getPaperNumber())) {
                        throw ExceptionResultEnum.ERROR.exception("不允许修改试卷编号");
                    } else {
                        basicExamStudent.setId(student.getId());
                        basicExamStudent.setPaperType(student.getPaperType());
                    }
                }

                if (StringUtils.isNotBlank(basicExamStudent.getPaperNumber())) {
                    if (paperNumberCourseIdMap.containsKey(basicExamStudent.getPaperNumber())) {
                        if (!paperNumberCourseIdMap.get(basicExamStudent.getPaperNumber()).equals(basicExamStudent.getCourseId())) {
                            stringJoiner.add("试卷编号[" + basicExamStudent.getPaperNumber() + "]不能绑定多个课程");
                        }
                    } else {
                        long count = basicExamStudentList.stream().filter(m -> m.getPaperNumber() != null && m.getPaperNumber().equals(basicExamStudent.getPaperNumber()) && !Objects.equals(m.getCourseId(), basicExamStudent.getCourseId())).count();
                        if (count > 0) {
                            stringJoiner.add("试卷编号[" + basicExamStudent.getPaperNumber() + "]已绑定其它课程");
                        } else {
                            paperNumberCourseIdMap.put(basicExamStudent.getPaperNumber(), basicExamStudent.getCourseId());
                        }
                    }

                    // 同一试卷编号只能对应同一考试时间
                    if (basicExamStudent.getExamStartTime() != null && basicExamStudent.getExamEndTime() != null) {
                        String examTimeValue = basicExamStudent.getExamStartTime() + "-" + basicExamStudent.getExamEndTime();
                        if (paperNumberExamTimeMap.containsKey(basicExamStudent.getPaperNumber())) {
                            if (!paperNumberExamTimeMap.get(basicExamStudent.getPaperNumber()).equals(examTimeValue)) {
                                stringJoiner.add("试卷编号[" + basicExamStudent.getPaperNumber() + "]只能有相同考试时间");
                            }
                        } else {
                            paperNumberExamTimeMap.put(basicExamStudent.getPaperNumber(), examTimeValue);
                        }
                    }
                }

                // 校验课程学号唯一
                String courseCodeStudentCodeKey = basicExamStudent.getCourseId() + basicExamStudentImport.getStudentCode();
                if (courseCodeStudentCodeList.contains(courseCodeStudentCodeKey)) {
                    stringJoiner.add(String.format("课程代码[%s]，开课学院[%s]，学号[%s]存在重复考生", basicExamStudentImport.getCourseCode(), basicExamStudentImport.getTeachingRoomName(), basicExamStudentImport.getStudentCode()));
                } else {
                    courseCodeStudentCodeList.add(courseCodeStudentCodeKey);
                }
            }

            if (stringJoiner.toString().length() > 0) {
                basicExamStudentImport.setErrorMsg(stringJoiner.toString());
            }

            if (!hasError && stringJoiner.toString().length() > 0) {
                hasError = true;
            }

            if (stringJoiner.toString().length() == 0) {
                basicExamStudents.add(basicExamStudent);
            }
        }

        map.put(SystemConstant.ERROR_DATA_LIST, basicExamStudentImportList);
        map.put(SystemConstant.DATASOURCE, basicExamStudents);
        map.put(SystemConstant.DATA_COUNT, basicExamStudentImportList.size());
        map.put(SystemConstant.SUCCESS_DATA_COUNT, basicExamStudents.size());
        map.put(SystemConstant.ERROR_DATA_COUNT, basicExamStudentImportList.size() - basicExamStudents.size());
        map.put(SystemConstant.HAS_ERROR_DATA, hasError);
        map.put(SystemConstant.COLUMN_NAMES, basicExamStudentParseDto.getColumnNames());
        map.put("oldPaperNumberList", oldPaperNumberList);

        return map;
    }

    @Transactional
    @Override
    public Map<String, Object> executeImportBasicExamStudentTeacherLogic(Map<String, Object> map, StringJoiner stringJoinerSummary) {
        SysUser requestUser = (SysUser) map.get(SystemConstant.USER);
        InputStream inputStream = (InputStream) map.get("inputStream");
        Long examId = SystemConstant.convertIdToLong(String.valueOf(map.get("examId")));
        Long schoolId = requestUser.getSchoolId();
        BasicExam basicExam = basicExamService.getById(examId);
        if (Objects.isNull(basicExam)) {
            throw ExceptionResultEnum.ERROR.exception("考试不存在");
        }

        SystemConstant.addSummary(stringJoinerSummary, "开始解析导入文件内容");
        BasicExamStudentParseDto basicExamStudentParseDto;
        try {
            basicExamStudentParseDto = this.parseImportExamStudentData(schoolId, inputStream);
        } catch (Exception e) {
            SystemConstant.addSummary(stringJoinerSummary, "解析导入文件内容结束，解析失败。" + e.getMessage());
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        }
        SystemConstant.addSummary(stringJoinerSummary, "解析导入文件内容结束，开始进行数据校验");

        List<BasicExamStudentImport> basicExamStudentImportList = basicExamStudentParseDto.getBasicExamStudentImportList();
        if (CollectionUtils.isEmpty(basicExamStudentImportList)) {
            throw ExceptionResultEnum.ERROR.exception("未读取到有效数据");
        }

        List<BasicExamStudent> basicExamStudentList = basicExamStudentService.list(new QueryWrapper<BasicExamStudent>().lambda()
                .select(BasicExamStudent::getId, BasicExamStudent::getCourseId, BasicExamStudent::getStudentCode, BasicExamStudent::getPaperNumber, BasicExamStudent::getTeachClassName, BasicExamStudent::getTeacherId, BasicExamStudent::getRequiredFields)
                .eq(BasicExamStudent::getSchoolId, schoolId).eq(BasicExamStudent::getExamId, examId));
        Map<String, BasicExamStudent> courseIdStudentCodeMap = basicExamStudentList.stream().collect(Collectors.toMap(k -> k.getCourseId() + SystemConstant.HYPHEN + k.getStudentCode(), Function.identity(), (v1, v2) -> v1));

        Map<String, Long> basicCourseIdMap = new HashMap<>();
        // 课程学号唯一
        List<String> courseCodeStudentCodeList = new ArrayList<>();
        // 课程id+教学班名称->任课老师Id
        Map<String, Long> courseTeachClassTeacherMap = new HashMap<>();
        // 是否有错误提示
        boolean hasError = false;

        // 必填字段为课程代码、开课学院、学号、教学班、任课老师工号、任课老师
        List<String> requiredCodes = Arrays.asList(RequiredFieldsEnum.STUDENT_CODE.getCode(), RequiredFieldsEnum.COURSE_CODE.getCode(), RequiredFieldsEnum.TEACHING_ROOM_NAME.getCode(), RequiredFieldsEnum.TEACH_CLASS_NAME.getCode(), RequiredFieldsEnum.TEACHER_NAME.getCode(), RequiredFieldsEnum.TEACHER_CODE.getCode());

        List<BasicExamStudent> basicExamStudents = new ArrayList<>();
        for (BasicExamStudentImport basicExamStudentImport : basicExamStudentImportList) {
            BasicExamStudent basicExamStudent = new BasicExamStudent();
            StringJoiner stringJoiner = new StringJoiner(";");

            // 校验excel内容
            basicExamStudentImport.getRequiredFieldList().forEach(m -> {
                if (requiredCodes.contains(m.getCode()) && StringUtils.isBlank(m.getValue())) {
                    stringJoiner.add(m.getName() + "必填");
                }
            });

            // 校验课程
            if (StringUtils.isNoneBlank(basicExamStudentImport.getCourseCode(), basicExamStudentImport.getTeachingRoomName())) {
                String key = basicExamStudentImport.getCourseCode() + SystemConstant.HYPHEN + basicExamStudentImport.getTeachingRoomName();
                if (!basicCourseIdMap.containsKey(key)) {
                    List<SysOrg> sysOrgList = sysOrgService.getSecondOrg(schoolId, basicExamStudentImport.getTeachingRoomName());
                    if (CollectionUtils.isEmpty(sysOrgList)) {
                        stringJoiner.add("开课学院[" + basicExamStudentImport.getTeachingRoomName() + "]在学院层级不存在");
                    } else {
                        SysOrg sysOrg = sysOrgList.get(0);
                        // 校验课程代码和开课学院
                        BasicCourse basicCourse = basicCourseService.getByTeachRoomIdAndCode(sysOrg.getId(), basicExamStudentImport.getCourseCode());
                        if (Objects.isNull(basicCourse)) {
                            stringJoiner.add("课程代码[" + basicExamStudentImport.getCourseCode() + "]在开课学院[" + basicExamStudentImport.getTeachingRoomName() + "]不存在");
                        } else {
                            basicExamStudent.setCourseId(basicCourse.getId());
                            basicCourseIdMap.put(key, basicCourse.getId());
                        }
                    }
                } else {
                    basicExamStudent.setCourseId(basicCourseIdMap.get(key));
                }
            }

            //校验任课老师
            if (StringUtils.isNotBlank(basicExamStudentImport.getTeacherCode())) {
                SysUser sysUser = sysUserService.getByLoginName(schoolId, basicExamStudentImport.getTeacherCode());
                if (sysUser == null) {
                    stringJoiner.add("任课老师工号[" + basicExamStudentImport.getTeacherCode() + "]用户不存在");
                } else {
                    if (!sysUser.getEnable()) {
                        stringJoiner.add("任课老师工号[" + basicExamStudentImport.getTeacherCode() + "]用户已禁用");
                    } else if (!sysUser.getRealName().equals(basicExamStudentImport.getTeacherName())) {
                        stringJoiner.add("任课老师[" + basicExamStudentImport.getTeacherName() + "]与用户管理中姓名[" + sysUser.getRealName() + "]不一致");
                    } else {
                        basicExamStudent.setTeacherId(sysUser.getId());
                    }
                }
            }

            if (basicExamStudent.getCourseId() != null) {
                // 校验同课程下，同一教学班只能有一个任课老师
                if (StringUtils.isNotBlank(basicExamStudent.getTeachClassName())) {
                    String courseTeachClassTeacherKey = basicExamStudent.getCourseId() + basicExamStudent.getTeachClassName();
                    if (courseTeachClassTeacherMap.containsKey(courseTeachClassTeacherKey)) {
                        Long teacherId = courseTeachClassTeacherMap.get(courseTeachClassTeacherKey);
                        if (teacherId != null && basicExamStudent.getTeacherId() != null && !teacherId.equals(basicExamStudent.getTeacherId())) {
                            stringJoiner.add("同课程同教学班，只允许设置一个任课老师");
                        }
                    } else {
                        courseTeachClassTeacherMap.put(courseTeachClassTeacherKey, basicExamStudent.getTeacherId());
                    }
                }

                // 校验课程+学号是否存在
                if (StringUtils.isNotBlank(basicExamStudentImport.getStudentCode())) {
                    String studentKey = basicExamStudent.getCourseId() + SystemConstant.HYPHEN + basicExamStudentImport.getStudentCode();
                    if (courseIdStudentCodeMap.containsKey(studentKey)) {
                        BasicExamStudent student = courseIdStudentCodeMap.get(studentKey);
                        basicExamStudent.setId(student.getId());
                        basicExamStudent.setPaperNumber(student.getPaperNumber());
                        basicExamStudent.setPaperType(student.getPaperType());
                        List<CodeNameEnableValue> requiredFieldList = StringUtils.isNotBlank(student.getRequiredFields()) ? JSON.parseArray(student.getRequiredFields(), CodeNameEnableValue.class) : Collections.emptyList();
                        // 更新requiredFields中的任课老师和任课老师工号
                        for (CodeNameEnableValue codeNameEnableValue : requiredFieldList) {
                            if (codeNameEnableValue.getCode().equals(RequiredFieldsEnum.TEACHER_CODE.getCode())) {
                                codeNameEnableValue.setValue(basicExamStudentImport.getTeacherCode());
                            }
                            if (codeNameEnableValue.getCode().equals(RequiredFieldsEnum.TEACHER_NAME.getCode())) {
                                codeNameEnableValue.setValue(basicExamStudentImport.getTeacherName());
                            }
                        }
                        basicExamStudent.setRequiredFields(JSON.toJSONString(requiredFieldList));
                    } else {
                        stringJoiner.add(String.format("课程代码[%s]，开课学院[%s]，学号[%s]考生管理中不存在", basicExamStudentImport.getCourseCode(), basicExamStudentImport.getTeachingRoomName(), basicExamStudentImport.getStudentCode()));
                    }
                }

                // 校验课程学号唯一
                String courseCodeStudentCodeKey = basicExamStudent.getCourseId() + basicExamStudentImport.getStudentCode();
                if (courseCodeStudentCodeList.contains(courseCodeStudentCodeKey)) {
                    stringJoiner.add(String.format("课程代码[%s]，开课学院[%s]，学号[%s]存在重复考生", basicExamStudentImport.getCourseCode(), basicExamStudentImport.getTeachingRoomName(), basicExamStudentImport.getStudentCode()));
                } else {
                    courseCodeStudentCodeList.add(courseCodeStudentCodeKey);
                }
            }

            if (stringJoiner.toString().length() > 0) {
                basicExamStudentImport.setErrorMsg(stringJoiner.toString());
            }

            if (!hasError && stringJoiner.toString().length() > 0) {
                hasError = true;
            }

            if (stringJoiner.toString().length() == 0) {
                basicExamStudents.add(basicExamStudent);
            }
        }

        map.put(SystemConstant.ERROR_DATA_LIST, basicExamStudentImportList);
        map.put(SystemConstant.DATASOURCE, basicExamStudents);
        map.put(SystemConstant.DATA_COUNT, basicExamStudentImportList.size());
        map.put(SystemConstant.SUCCESS_DATA_COUNT, basicExamStudents.size());
        map.put(SystemConstant.ERROR_DATA_COUNT, basicExamStudentImportList.size() - basicExamStudents.size());
        map.put(SystemConstant.HAS_ERROR_DATA, hasError);
        map.put(SystemConstant.COLUMN_NAMES, basicExamStudentParseDto.getColumnNames());

        return map;
    }

    @Transactional
    @Override
    public Map<String, Object> executeImportExamStudentLogic(Map<String, Object> map, TBTask tbTask, StringJoiner stringJoinerSummary) {
        SysUser requestUser = (SysUser) map.get(SystemConstant.USER);
        InputStream inputStream = (InputStream) map.get("inputStream");
        Long schoolId = requestUser.getSchoolId();
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(tbTask.getPrintPlanId());
        if (examPrintPlan == null) {
            throw ExceptionResultEnum.ERROR.exception("印刷计划不存在");
        }

        String printPlanName = examPrintPlan.getName();
        if (!tbTaskPdfService.countByPrintPlanIdAndEntityId(schoolId, tbTask.getPrintPlanId())) {
            throw ExceptionResultEnum.ERROR.exception("印刷计划[" + printPlanName + "]有数据正在生成PDF文件，无法导入考务数据");
        }
        PrintPlanStatusEnum printPlanStatus = examPrintPlan.getStatus();
        if (PrintPlanStatusEnum.NEW != printPlanStatus && PrintPlanStatusEnum.READY != printPlanStatus) {
            throw ExceptionResultEnum.ERROR.exception("印刷计划状态为[" + PrintPlanStatusEnum.NEW.getName() + "、" + PrintPlanStatusEnum.READY.getName() + "]时才可导入，当前状态[" + printPlanStatus.getName() + "]");
        }

        Long examId = examPrintPlan.getExamId();
        BasicExam basicExam = basicExamService.getById(examId);
        if (Objects.isNull(basicExam)) {
            throw ExceptionResultEnum.ERROR.exception("考试不存在");
        }
        Long semesterId = SystemConstant.convertIdToLong(basicExam.getSemesterId());

        // 解析excel文件
        SystemConstant.addSummary(stringJoinerSummary, "开始解析文件内容");
        BasicExamStudentParseDto basicExamStudentParseDto;
        try {
            basicExamStudentParseDto = this.parseImportExamStudentData(schoolId, inputStream);
        } catch (Exception e) {
            SystemConstant.addSummary(stringJoinerSummary, "解析文件内容结束，解析失败。" + e.getMessage());
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        }
        SystemConstant.addSummary(stringJoinerSummary, "解析文件内容结束，开始进行数据校验");

        List<BasicExamStudentImport> basicExamStudentImportList = basicExamStudentParseDto.getBasicExamStudentImportList();
        if (CollectionUtils.isEmpty(basicExamStudentImportList)) {
            throw ExceptionResultEnum.ERROR.exception("未读取到有效数据");
        }

        List<BasicExamStudent> basicExamStudentList = basicExamStudentService.list(new QueryWrapper<BasicExamStudent>().lambda()
                .select(BasicExamStudent::getId, BasicExamStudent::getCourseId, BasicExamStudent::getStudentCode, BasicExamStudent::getPaperNumber, BasicExamStudent::getExamStartTime, BasicExamStudent::getExamEndTime)
                .eq(BasicExamStudent::getSchoolId, schoolId).eq(BasicExamStudent::getExamId, examId));
        Map<String, BasicExamStudent> courseIdStudentCodeMap = basicExamStudentList.stream().collect(Collectors.toMap(k -> k.getCourseId() + SystemConstant.HYPHEN + k.getStudentCode(), Function.identity(), (v1, v2) -> v1));

        // 文件中试卷编号对应多个课程代码
        Map<String, Long> paperNumberCourseIdInMap = new HashMap<>();
        // 试卷编号->考试时间
        Map<String, String> paperNumberExamTimeMap = basicExamStudentList.stream().filter(m -> StringUtils.isNotBlank(m.getPaperNumber())).collect(Collectors.toMap(BasicExamStudent::getPaperNumber, m -> m.getExamStartTime() + "-" + m.getExamEndTime(), (v1, v2) -> v1));
        Map<String, Long> basicCourseIdMap = new HashMap<>();

        // 课程管理中课程代码-课程名
        Map<String, BasicCourse> courseCodeNameInBasicCourseMap = new HashMap<>();
        List<BasicCourse> basicCourseList = basicCourseService.list(new QueryWrapper<BasicCourse>().lambda().eq(BasicCourse::getSchoolId, schoolId));
        basicCourseList.forEach(e -> courseCodeNameInBasicCourseMap.put(e.getTeachingRoomId().toString().concat(e.getCode()), e));

        // 课程学号唯一
        List<String> courseCodeStudentCodeList = new ArrayList<>();
        // 是否有错误提示
        boolean hasError = false;

        // 命题任务列表
        List<ExamTask> examTaskList = examTaskService.listByExamId(examId);
        Map<String, List<ExamTask>> paperNumberMap = examTaskList.stream().collect(Collectors.groupingBy(m -> m.getPaperNumber()));

        List<BasicExamStudent> basicExamStudents = new ArrayList<>();
        for (BasicExamStudentImport basicExamStudentImport : basicExamStudentImportList) {
            BasicExamStudent basicExamStudent = new BasicExamStudent();
            BeanUtils.copyProperties(basicExamStudentImport, basicExamStudent);
            basicExamStudent.setSchoolId(schoolId);
            basicExamStudent.setSemesterId(semesterId);
            basicExamStudent.setExamId(examId);
            basicExamStudent.setRequiredFields(CollectionUtils.isNotEmpty(basicExamStudentImport.getRequiredFieldList()) ? JSON.toJSONString(basicExamStudentImport.getRequiredFieldList()) : null);
            basicExamStudent.setExtendFields(CollectionUtils.isNotEmpty(basicExamStudentImport.getExtendFieldList()) ? JSON.toJSONString(basicExamStudentImport.getExtendFieldList()) : null);

            StringJoiner stringJoiner = new StringJoiner(";");
            // 校验必填项
            basicExamStudentImport.getRequiredFieldList().forEach(m -> {
                if (m.getEnable() && StringUtils.isBlank(m.getValue())) {
                    stringJoiner.add(m.getName() + "必填");
                }
            });

            // 学号校验
            if (StringUtils.isNotBlank(basicExamStudent.getStudentCode()) && !basicExamStudent.getStudentCode().matches(SystemConstant.REGULAR_EXPRESSION_OF_STUDENT_CODE)) {
                stringJoiner.add("学号只能由大写字母、数字组成");
            }

            // 校验任课老师，都填或者都不填
            if (StringUtils.isNotBlank(basicExamStudentImport.getTeacherCode()) && StringUtils.isBlank(basicExamStudentImport.getTeacherName())) {
                stringJoiner.add(RequiredFieldsEnum.TEACHER_NAME.getName() + "必填");
            }
            if (StringUtils.isNotBlank(basicExamStudentImport.getTeacherName()) && StringUtils.isBlank(basicExamStudentImport.getTeacherCode())) {
                stringJoiner.add(RequiredFieldsEnum.TEACHER_CODE.getName() + "必填");
            }

            // 解析并校验考试日期、考试时间
            if (StringUtils.isNoneBlank(basicExamStudentImport.getExamDate(), basicExamStudentImport.getExamTime())) {
                try {
                    Map<String, Object> timeMap = ConvertUtil.analyzeExamTime(basicExamStudentImport.getExamDate(), basicExamStudentImport.getExamTime());
                    basicExamStudent.setExamStartTime(Long.valueOf(String.valueOf(timeMap.get("startTime"))));
                    basicExamStudent.setExamEndTime(Long.valueOf(String.valueOf(timeMap.get("endTime"))));

                    // 反向更新examDate为标准日期格式，
                    basicExamStudentImport.setExamDate(String.valueOf(timeMap.get("dateStr")));
                } catch (Exception e) {
                    stringJoiner.add(e.getMessage());
                }
            }

            // 校验开课学院是否存在该课程代码
            if (StringUtils.isNoneBlank(basicExamStudentImport.getCourseCode(), basicExamStudentImport.getTeachingRoomName())) {
                String key = basicExamStudentImport.getCourseCode() + SystemConstant.HYPHEN + basicExamStudentImport.getTeachingRoomName();
                if (!basicCourseIdMap.containsKey(key)) {
                    List<SysOrg> sysOrgList = sysOrgService.getSecondOrg(schoolId, basicExamStudentImport.getTeachingRoomName());
                    if (CollectionUtils.isEmpty(sysOrgList)) {
                        stringJoiner.add("开课学院[" + basicExamStudentImport.getTeachingRoomName() + "]在学院层级不存在");
                    } else if (sysOrgList.size() == 1) {
                        SysOrg sysOrg = sysOrgList.get(0);
                        // 校验课程代码和开课学院
                        BasicCourse basicCourse = courseCodeNameInBasicCourseMap.get(sysOrg.getId().toString().concat(basicExamStudentImport.getCourseCode()));
                        if (Objects.isNull(basicCourse)) {
                            stringJoiner.add("课程代码[" + basicExamStudentImport.getCourseCode() + "]在开课学院[" + basicExamStudentImport.getTeachingRoomName() + "]不存在");
                        } else {
                            basicExamStudent.setCourseId(basicCourse.getId());
                            basicCourseIdMap.put(key, basicCourse.getId());
                        }
                    } else {
                        stringJoiner.add("开课学院[" + basicExamStudentImport.getTeachingRoomName() + "]存在多个相同值");
                    }
                } else {
                    basicExamStudent.setCourseId(basicCourseIdMap.get(key));
                }
            }

            // 校验试卷编号和课程代码对应关系，试卷编号只能绑定一个课程
            if (basicExamStudent.getCourseId() != null) {
                if (paperNumberCourseIdInMap.containsKey(basicExamStudent.getPaperNumber())) {
                    Long courseId = paperNumberCourseIdInMap.get(basicExamStudent.getPaperNumber());
                    if (!basicExamStudent.getCourseId().equals(courseId)) {
                        stringJoiner.add(String.format("试卷编号[%s]对应多个不同的课程代码", basicExamStudent.getPaperNumber()));
                    }
                } else {
                    paperNumberCourseIdInMap.put(basicExamStudent.getPaperNumber(), basicExamStudent.getCourseId());
                }

                // 校验试卷编号课程代码是否已命题且命题任务中课程代码是否一致
                if (StringUtils.isNotBlank(basicExamStudent.getPaperNumber())) {
                    if (paperNumberMap.containsKey(basicExamStudent.getPaperNumber())) {
                        List<ExamTask> examTasks = paperNumberMap.get(basicExamStudent.getPaperNumber());
                        if (examTasks.size() > 1) {
                            stringJoiner.add(String.format("试卷编号[%s]在考试[%s]下有多条命题任务，请联系管理员处理", basicExamStudent.getPaperNumber(), basicExam.getName()));
                        } else {
                            Long courseId = paperNumberCourseIdInMap.get(basicExamStudent.getPaperNumber());
                            ExamTask examTask = examTasks.get(0);
                            if (!examTask.getCourseId().equals(courseId)) {
                                stringJoiner.add(String.format("试卷编号[%s]对应课程代码与命题任务中对应课程代码不一致", basicExamStudent.getPaperNumber()));
                            } else {
                                basicExamStudent.setCoursePaperId(String.valueOf(examTask.getId()));
                            }
                        }
                    } else {
                        stringJoiner.add(String.format("试卷编号[%s]未命题", basicExamStudent.getPaperNumber()));
                    }
                }
            }

            //校验任课老师是否存在，且工号和姓名是否与系统一致
            if (StringUtils.isNoneBlank(basicExamStudentImport.getTeacherCode(), basicExamStudentImport.getTeacherName())) {
                SysUser sysUser = sysUserService.getByLoginName(schoolId, basicExamStudentImport.getTeacherCode());
                if (sysUser == null) {
                    stringJoiner.add("任课老师工号[" + basicExamStudentImport.getTeacherCode() + "]用户不存在");
                } else {
                    if (!sysUser.getEnable()) {
                        stringJoiner.add("任课老师工号[" + basicExamStudentImport.getTeacherCode() + "]用户已禁用");
                    } else if (!sysUser.getRealName().equals(basicExamStudentImport.getTeacherName())) {
                        stringJoiner.add("任课老师[" + basicExamStudentImport.getTeacherName() + "]与用户管理中姓名[" + sysUser.getRealName() + "]不一致");
                    } else {
                        basicExamStudent.setTeacherId(sysUser.getId());
                    }
                }

                // 校验同课程下，同一教学班只能有一个任课老师
//                String courseTeachClassTeacherKey = basicExamStudent.getCourseId() + basicExamStudent.getTeachClassName();
//                if (courseTeachClassTeacherMap.containsKey(courseTeachClassTeacherKey)) {
//                    Long teacherId = courseTeachClassTeacherMap.get(courseTeachClassTeacherKey);
//                    if ((teacherId == null && basicExamStudent.getTeacherId() != null) || (teacherId != null && basicExamStudent.getTeacherId() == null) || (teacherId != null && !teacherId.equals(basicExamStudent.getTeacherId()))) {
//                        stringJoiner.add("同课程下相同教学班，只能有一位任课老师");
//                    }
//                } else {
//                    courseTeachClassTeacherMap.put(courseTeachClassTeacherKey, basicExamStudent.getTeacherId());
//                }
            }

            // 校验试卷编号学号是否已生成印刷任务
            if (StringUtils.isNoneBlank(basicExamStudent.getPaperNumber(), basicExamStudent.getStudentCode())) {
                int studentCount = examStudentService.countByExamIdAndPaperNumberAndStudentCode(examId, basicExamStudent.getPaperNumber(), basicExamStudent.getStudentCode());
                if (studentCount > 0) {
                    stringJoiner.add("试卷编号[" + basicExamStudent.getPaperNumber() + "]、学号[" + basicExamStudent.getStudentCode() + "]已有印刷任务，不允许重复导入");
                } else {
                    if (basicExamStudent.getCourseId() != null) {
                        // 校验课程学号唯一
                        String courseCodeStudentCodeKey = basicExamStudent.getCourseId() + basicExamStudent.getStudentCode();
                        if (courseCodeStudentCodeList.contains(courseCodeStudentCodeKey)) {
                            stringJoiner.add("课程代码[" + basicExamStudentImport.getCourseCode() + "]已存在学号[" + basicExamStudentImport.getStudentCode() + "]的考生数据");
                        } else {
                            courseCodeStudentCodeList.add(courseCodeStudentCodeKey);
                        }

                        if (courseIdStudentCodeMap.containsKey(basicExamStudent.getCourseId() + SystemConstant.HYPHEN + basicExamStudent.getStudentCode())) {
                            BasicExamStudent basicStudent = courseIdStudentCodeMap.get(basicExamStudent.getCourseId() + SystemConstant.HYPHEN + basicExamStudent.getStudentCode());
                            basicExamStudent.setId(basicStudent.getId());
                        }
                    }
                }
            }

            // 校验试卷编号、考试日期、考试时间、考点、考场是否已命题
            if (basicExamStudent.getCourseId() != null && StringUtils.isNotBlank(basicExamStudent.getPaperNumber())) {
                List<ExamDetail> examDetailList = examDetailService.listExamDetailByExamIdAndCourseIdAndPaperNumber(examId, basicExamStudent.getCourseId(), basicExamStudent.getPaperNumber());
                if (CollectionUtils.isNotEmpty(examDetailList)) {
                    long count = examDetailList.stream().filter(m -> basicExamStudent.getExamStartTime().equals(m.getExamStartTime()) && basicExamStudent.getExamEndTime().equals(m.getExamEndTime()) && basicExamStudent.getExamPlace().equals(m.getExamPlace()) && basicExamStudent.getExamRoom().equals(m.getExamRoom())).count();
                    if (count != 0) {
                        stringJoiner.add("试卷编号[" + basicExamStudentImport.getPaperNumber() + "]、考试日期[" + basicExamStudentImport.getExamDate() + "]、考试时间[" + basicExamStudentImport.getExamTime() + "]、考场[" + basicExamStudentImport.getExamPlace() + "]、考点[" + basicExamStudentImport.getExamRoom() + "]已有印刷任务");
                    }
                }

                // 同一试卷编号只能对应同一考试时间
                if (basicExamStudent.getExamStartTime() != null && basicExamStudent.getExamEndTime() != null) {
                    String examTimeValue = basicExamStudent.getExamStartTime() + "-" + basicExamStudent.getExamEndTime();
                    if (paperNumberExamTimeMap.containsKey(basicExamStudent.getPaperNumber())) {
                        if (!paperNumberExamTimeMap.get(basicExamStudent.getPaperNumber()).equals(examTimeValue)) {
                            stringJoiner.add("试卷编号[" + basicExamStudent.getPaperNumber() + "]只能存在相同考试时间");
                        }
                    } else {
                        paperNumberExamTimeMap.put(basicExamStudent.getPaperNumber(), examTimeValue);
                    }
                }
            }

            if (stringJoiner.toString().length() > 0) {
                basicExamStudentImport.setErrorMsg(stringJoiner.toString());
            }

            if (!hasError && stringJoiner.toString().length() > 0) {
                hasError = true;
            }

            if (stringJoiner.toString().length() == 0) {
                basicExamStudents.add(basicExamStudent);
            }
        }

        //校验命题任务是否存在并审核通过

        map.put(SystemConstant.ERROR_DATA_LIST, basicExamStudentImportList);
        map.put(SystemConstant.DATASOURCE, basicExamStudents);
        map.put(SystemConstant.DATA_COUNT, basicExamStudentImportList.size());
        map.put(SystemConstant.SUCCESS_DATA_COUNT, basicExamStudents.size());
        map.put(SystemConstant.ERROR_DATA_COUNT, basicExamStudentImportList.size() - basicExamStudents.size());
        map.put(SystemConstant.HAS_ERROR_DATA, hasError);
        map.put(SystemConstant.COLUMN_NAMES, basicExamStudentParseDto.getColumnNames());
        map.put("paperNumberCourseIdInMap", paperNumberCourseIdInMap);

        return map;
    }

    private BasicExamStudentParseDto parseImportExamStudentData(Long schoolId, InputStream inputStream) {
        ExcelReader excelReader = ExcelReader.create(ExcelType.XLSX, inputStream, 1);
        List<DataMap> dataMapList;
        try {
            dataMapList = excelReader.getDataMapList();
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("读取excel内容失败");
        }
        BasicExamStudentParseDto basicExamStudentParseDto = new BasicExamStudentParseDto();
        List<BasicExamStudentImport> list = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(dataMapList)) {
            String[] columnNames = excelReader.getColumnNames();

            BasicExamRule basicExamRule = basicExamRuleService.getBySchoolId(schoolId);
            if (basicExamRule == null) {
                throw ExceptionResultEnum.ERROR.exception("通用规则未设置");
            }

            // 必填字段
            Map<String, EnumResult> requiredMap = basicExamRule.getRequiredFieldList().stream().filter(m -> m.getEnable()).collect(Collectors.toMap(EnumResult::getName, e -> e));
            // 扩展字段
            Map<String, EnumResult> extendMap = basicExamRule.getExtendFieldList().stream().filter(m -> m.getEnable()).collect(Collectors.toMap(EnumResult::getName, e -> e));

            // 通用规则表头
            List<String> actualTitleList = Stream.concat(requiredMap.keySet().stream(), extendMap.keySet().stream()).collect(Collectors.toList());
            if (!CollectionUtils.isEqualCollection(Arrays.asList(columnNames), actualTitleList)) {
                throw ExceptionResultEnum.ERROR.exception("表头名称错误，正确表头为【" + String.join(",", actualTitleList) + "】");
            }

            basicExamStudentParseDto.setColumnNames(columnNames);

            AtomicInteger i = new AtomicInteger(2);
            dataMapList.forEach(m -> {
                if (m.size() > 0) {
                    BasicExamStudentImport basicExamStudentImport = new BasicExamStudentImport();
                    basicExamStudentImport.setRowIndex(i.getAndIncrement());
                    Class<BasicExamStudentImport> aClass = (Class<BasicExamStudentImport>) basicExamStudentImport.getClass();
                    for (Map.Entry<String, String> entry : m.entrySet()) {
                        // 必填字段
                        if (requiredMap.containsKey(entry.getKey())) {
                            if (entry.getValue() == null) {
                                continue;
                            }
                            EnumResult enumResult = requiredMap.get(entry.getKey());
                            try {
                                Field declaredField = aClass.getDeclaredField(enumResult.getCode());
                                declaredField.setAccessible(true);
                                // 学生姓名、任课老师姓名存在英文名，只去掉前后空格
                                declaredField.set(basicExamStudentImport, RequiredFieldsEnum.TEACHER_NAME.getCode().equals(enumResult.getCode()) || RequiredFieldsEnum.STUDENT_NAME.getCode().equals(enumResult.getCode()) ? entry.getValue().trim() : StringUtils.deleteWhitespace(entry.getValue()));
                            } catch (NoSuchFieldException e) {
                                throw ExceptionResultEnum.ERROR.exception("未获取到表头为[" + entry.getKey() + "]的属性值");
                            } catch (IllegalAccessException e) {
                                throw ExceptionResultEnum.ERROR.exception("表头为[" + entry.getKey() + "]的值赋值失败");
                            }
                            basicExamStudentImport.getRequiredFieldList().add(new CodeNameEnableDisabledValue(enumResult.getCode(), enumResult.getName(), enumResult.getEnable(), enumResult.getDisabled(), StringUtils.isNotBlank(entry.getValue()) ? entry.getValue() : null));
                        }
                        // 扩展字段
                        if (extendMap.containsKey(entry.getKey())) {
                            EnumResult enumResult = extendMap.get(entry.getKey());
                            basicExamStudentImport.getExtendFieldList().add(new CodeNameEnableDisabledValue(enumResult.getCode(), enumResult.getName(), enumResult.getEnable(), enumResult.getDisabled(), StringUtils.isNotBlank(entry.getValue()) ? entry.getValue() : null));
                        }
                    }

                    List<CodeNameEnableDisabledValue> requiredFieldList = basicExamStudentImport.getRequiredFieldList();
                    // 必填字段
                    if (requiredMap.keySet().size() > requiredFieldList.size()) {
                        for (String s : requiredMap.keySet()) {
                            Optional<CodeNameEnableDisabledValue> optional = basicExamStudentImport.getRequiredFieldList().stream().filter(r -> s.equals(r.getName())).findFirst();
                            if (!optional.isPresent()) {
                                EnumResult enumResult = requiredMap.get(s);
                                basicExamStudentImport.getRequiredFieldList().add(new CodeNameEnableDisabledValue(enumResult.getCode(), enumResult.getName(), enumResult.getEnable(), enumResult.getDisabled(), null));
                            }
                        }
                    }
                    List<CodeNameEnableDisabledValue> extendFieldList = basicExamStudentImport.getExtendFieldList();
                    // 扩展字段
                    if (extendMap.keySet().size() > extendFieldList.size()) {
                        for (String s : extendMap.keySet()) {
                            Optional<CodeNameEnableDisabledValue> optional = basicExamStudentImport.getExtendFieldList().stream().filter(r -> s.equals(r.getName())).findFirst();
                            if (!optional.isPresent()) {
                                EnumResult enumResult = extendMap.get(s);
                                basicExamStudentImport.getExtendFieldList().add(new CodeNameEnableDisabledValue(enumResult.getCode(), enumResult.getName(), enumResult.getEnable(), enumResult.getDisabled(), null));
                            }
                        }
                    }
                    list.add(basicExamStudentImport);
                }
            });
        }
        basicExamStudentParseDto.setBasicExamStudentImportList(list);
        return basicExamStudentParseDto;
    }

    @Transactional
    @Override
    public Map<String, Object> executeImportObjectiveStructLogic(Map<String, Object> map) throws Exception {
        SysUser requestUser = (SysUser) map.get(SystemConstant.USER);
        InputStream inputStream = (InputStream) map.get("inputStream");
        Long examId = SystemConstant.convertIdToLong(String.valueOf(map.get("examId")));
        Long schoolId = requestUser.getSchoolId();
        BasicExam basicExam = basicExamService.getById(examId);
        if (Objects.isNull(basicExam)) {
            throw ExceptionResultEnum.ERROR.exception("考试不存在");
        }
        if (!ExamModelEnum.MODEL4.equals(basicExam.getExamModel())) {
            throw ExceptionResultEnum.ERROR.exception("非模式4的考试，无法使用此功能");
        }

        List<MarkPaper> markPaperList = markPaperService.findMarkPaperListByExamId(examId);
        Map<String, MarkPaper> markPaperCourseMap = markPaperList.stream().collect(Collectors.toMap(MarkPaper::getCourseCode, Function.identity()));
        Map<String, MarkPaper> markPaperPaperNumberMap = markPaperList.stream().collect(Collectors.toMap(MarkPaper::getPaperNumber, Function.identity()));

        List<MarkQuestion> markQuestionList = markQuestionService.listByExamIdAndObjective(examId, true);
        Set<String> objectivePaperNumberSet = markQuestionList.stream().map(MarkQuestion::getPaperNumber).collect(Collectors.toSet());

        List<MarkQuestion> markQuestionList1 = markQuestionService.listByExamIdAndObjective(examId, false);
        Set<String> subjectivePaperNumberSet = markQuestionList1.stream().map(m -> SystemConstant.mergeString(SystemConstant.HYPHEN, m.getPaperNumber(), m.getMainNumber(), m.getSubNumber())).collect(Collectors.toSet());

        List<SysUserResult> sysUserList = sysUserService.listBySchoolId(schoolId);
        Map<String, SysUserResult> sysUserMap = sysUserList.stream().collect(Collectors.toMap(SysUserResult::getLoginName, Function.identity()));

        ExcelReader excelReader = ExcelReader.create(ExcelType.XLSX, inputStream, 0);
        List<ObjectiveStructImportDto> objectiveStructImportDtoList = excelReader.getObjectList(ObjectiveStructImportDto.class);

        // 课程下试卷编号map
        Map<String, String> coursePaperNumberMap = new HashMap<>();
        // 课程、试卷编号下大题号、小题号
        Map<String, String> coursePaperQuestionMap = new HashMap<>();

        List<MarkQuestion> markQuestions = new ArrayList<>();
        List<MarkQuestionAnswer> markQuestionAnswers = new ArrayList<>();
        boolean hasError = false;
        List<ObjectiveStructDto> objectiveStructDtos = new ArrayList<>();
        // 按课程+试卷编号分组
        for (ObjectiveStructImportDto objectiveStructImportDto : objectiveStructImportDtoList) {
            ObjectiveStructDto objectiveStructDto = new ObjectiveStructDto();
            BeanUtils.copyProperties(objectiveStructImportDto, objectiveStructDto);
            try {
                List<String> errorList = new ArrayList<>();

                if (StringUtils.isBlank(objectiveStructDto.getCourseCode())) {
                    errorList.add("课程代码不能为空");
                }
                if (StringUtils.isBlank(objectiveStructDto.getCourseName())) {
                    errorList.add("课程名称不能为空");
                }
                if (StringUtils.isBlank(objectiveStructDto.getPaperNumber())) {
                    errorList.add("试卷编号不能为空");
                }
                String mainTitle = objectiveStructDto.getMainTitle();
                if (StringUtils.isBlank(mainTitle)) {
                    errorList.add("大题名称不能为空");
                }
                Integer mainNumber = objectiveStructDto.getMainNumber();
                if (mainNumber == null) {
                    errorList.add("大题号不能为空");
                }
                Integer subNumber = objectiveStructDto.getSubNumber();
                if (subNumber == null) {
                    errorList.add("小题号不能为空");
                }
                String answer = objectiveStructDto.getAnswer();
                if (StringUtils.isBlank(answer)) {
                    errorList.add("标答不能为空");
                }
                Integer optionCount = objectiveStructDto.getOptionCount();
                if (optionCount == null) {
                    errorList.add("选项个数不能为空");
                }
                Double totalScore = objectiveStructDto.getTotalScore();
                if (totalScore == null) {
                    errorList.add("小题满分不能为空");
                }
                Integer questionType = objectiveStructDto.getQuestionType();
                if (questionType == null) {
                    errorList.add("题型不能为空");
                }
                if (CollectionUtils.isNotEmpty(errorList)) {
                    throw ExceptionResultEnum.ERROR.exception(String.join(";", errorList));
                }

                // 检查选项个数和标答范围是否一对待
                validObjectiveAnswer(answer, optionCount, questionType);

                String courseCode = objectiveStructDto.getCourseCode().trim();
                String courseName = objectiveStructDto.getCourseName().trim();
                String paperNumber = objectiveStructDto.getPaperNumber().trim();

                // 课程是否存在
                if (!markPaperCourseMap.containsKey(courseCode)) {
                    throw ExceptionResultEnum.ERROR.exception("未查询到此课程");
                } else {
                    MarkPaper markPaper = markPaperCourseMap.get(courseCode);
                    if (!courseName.equals(markPaper.getCourseName())) {
                        throw ExceptionResultEnum.ERROR.exception("评卷设置中，课程名称为[" + markPaper.getCourseName() + "]");
                    }
                    objectiveStructDto.setCourseId(markPaper.getCourseId());
                }

                // 试卷编号是否存在
                if (!markPaperPaperNumberMap.containsKey(paperNumber)) {
                    throw ExceptionResultEnum.ERROR.exception("未查询到此试卷编号");
                }

                // 试卷编号是否设置了主观题
                if (objectivePaperNumberSet.contains(paperNumber)) {
                    throw ExceptionResultEnum.ERROR.exception("该试卷编号下已经存在客观题试卷结构，不允许导入");
                }

                // 校验课程下试卷编号唯一
                if (coursePaperNumberMap.containsKey(courseCode)) {
                    if (!paperNumber.equals(coursePaperNumberMap.get(courseCode))) {
                        errorList.add("课程存在不同试卷编号");
                        throw ExceptionResultEnum.ERROR.exception(String.join(";", errorList));
                    }
                } else {
                    coursePaperNumberMap.put(courseCode, paperNumber);
                }

                // 校验试卷编号唯一
                if (coursePaperNumberMap.containsValue(paperNumber)) {
                    for (Map.Entry<String, String> entry : coursePaperNumberMap.entrySet()) {
                        if (entry.getValue().equals(paperNumber) && !courseCode.equals(entry.getKey())) {
                            errorList.add("试卷编号已被其它课程占用");
                            throw ExceptionResultEnum.ERROR.exception(String.join(";", errorList));
                        }
                    }
                }

                // 题号重复校验
                String coursePaper = SystemConstant.mergeString(SystemConstant.HYPHEN, courseCode, paperNumber);
                String questionNumber = SystemConstant.mergeString(SystemConstant.HYPHEN, mainNumber, subNumber);
                if (coursePaperQuestionMap.containsKey(coursePaper)) {
                    if (questionNumber.equals(coursePaperQuestionMap.get(coursePaper))) {
                        errorList.add("存在相同大题号、小题号");
                        throw ExceptionResultEnum.ERROR.exception(String.join(";", errorList));
                    }
                } else {
                    coursePaperQuestionMap.put(coursePaper, questionNumber);
                }

                // 题号是否被主观题使用
                if (subjectivePaperNumberSet.contains(SystemConstant.mergeString(SystemConstant.HYPHEN, paperNumber, mainNumber, subNumber))) {
                    errorList.add("大题号、小题号已被主观题使用");
                    throw ExceptionResultEnum.ERROR.exception(String.join(";", errorList));
                }

                // 组装数据
                assembleObjectiveImportData(examId, objectiveStructDto, markQuestions, markQuestionAnswers, sysUserMap);
            } catch (Exception e) {
                hasError = true;
                objectiveStructDto.setErrorMsg(e.getMessage());
            } finally {
                objectiveStructDtos.add(objectiveStructDto);
            }
        }

        map.put(SystemConstant.ERROR_DATA_LIST, objectiveStructDtos);
        map.put(SystemConstant.DATASOURCE, markQuestions);
        map.put(SystemConstant.DATASOURCE1, markQuestionAnswers);
        map.put(SystemConstant.DATA_COUNT, objectiveStructDtos.size());
        map.put(SystemConstant.HAS_ERROR_DATA, hasError);

        return map;
    }

    private void validObjectiveAnswer(String answer, Integer optionCount, Integer questionType) {
        if (optionCount <= 0 || optionCount > 26) {
            throw ExceptionResultEnum.ERROR.exception("选项个数只能填1~26之间的整数");
        }
        // 选项可选范围
        List<String> optionScope = OptionsEnum.scopeByOptionCount(optionCount);
        String optionScopeString = String.join("", optionScope);
        String[] answers = answer.split("");
        // 单选题
        if (questionType == 1) {
            if (answers.length > 1) {
                throw ExceptionResultEnum.ERROR.exception("单选题有且仅有一个答案");
            }
            for (String s : answers) {
                if (!optionScope.contains(s)) {
                    throw ExceptionResultEnum.ERROR.exception("答案只能从[" + optionScopeString + "]中选择一个");
                }
            }
        }
        // 多选题
        else if (questionType == 2) {
            for (String s : answers) {
                if (!optionScope.contains(s)) {
                    throw ExceptionResultEnum.ERROR.exception("答案只能从[" + optionScopeString + "]中选择一个或多个");
                }
            }
        }
        // 判断题
        else if (questionType == 3) {
            if (optionCount != 2) {
                throw ExceptionResultEnum.ERROR.exception("判断题选项个数只能为2");
            }
            if (answers.length > 1) {
                throw ExceptionResultEnum.ERROR.exception("判断题有且仅有一个答案A或B，A代表正确，B代表错误");
            }
            for (String s : answers) {
                if (!optionScope.contains(s)) {
                    throw ExceptionResultEnum.ERROR.exception("答案只能从[" + optionScopeString + "]中选择一个");
                }
            }
        } else {
            throw ExceptionResultEnum.ERROR.exception("题型只能从1、2、3中选择一个填写，1代表单选题，2代表多选题，3代表判断判断题");
        }
    }

    @Transactional
    @Override
    public Map<String, Object> executeImportSubjectiveStructLogic(Map<String, Object> map) throws Exception {
        SysUser requestUser = (SysUser) map.get(SystemConstant.USER);
        InputStream inputStream = (InputStream) map.get("inputStream");
        Long examId = SystemConstant.convertIdToLong(String.valueOf(map.get("examId")));
        Long schoolId = requestUser.getSchoolId();
        BasicExam basicExam = basicExamService.getById(examId);
        if (Objects.isNull(basicExam)) {
            throw ExceptionResultEnum.ERROR.exception("考试不存在");
        }
        if (!ExamModelEnum.MODEL4.equals(basicExam.getExamModel())) {
            throw ExceptionResultEnum.ERROR.exception("非模式4的考试，无法使用此功能");
        }
        SysRole markerRole = sysRoleService.getOne(new QueryWrapper<SysRole>().lambda().eq(SysRole::getType, RoleTypeEnum.MARKER));
        if (Objects.isNull(markerRole)) {
            throw ExceptionResultEnum.ERROR.exception("系统中未查询到评卷员角色，请联系管理员");
        }

        List<MarkPaper> markPaperList = markPaperService.findMarkPaperListByExamId(examId);
        Map<String, MarkPaper> markPaperCourseMap = markPaperList.stream().collect(Collectors.toMap(MarkPaper::getCourseCode, Function.identity()));
        Map<String, MarkPaper> markPaperPaperNumberMap = markPaperList.stream().collect(Collectors.toMap(MarkPaper::getPaperNumber, Function.identity()));

        List<MarkQuestion> markQuestionList = markQuestionService.listByExamIdAndObjective(examId, false);
        Set<String> subjectivePaperNumberSet = markQuestionList.stream().map(MarkQuestion::getPaperNumber).collect(Collectors.toSet());

        List<MarkQuestion> markQuestionList1 = markQuestionService.listByExamIdAndObjective(examId, true);
        Set<String> objectivePaperNumberSet = markQuestionList1.stream().map(m -> SystemConstant.mergeString(SystemConstant.HYPHEN, m.getPaperNumber(), m.getMainNumber(), m.getSubNumber())).collect(Collectors.toSet());

        List<SysUserResult> sysUserList = sysUserService.listBySchoolId(schoolId);
        Map<String, SysUserResult> sysUserMap = sysUserList.stream().collect(Collectors.toMap(SysUserResult::getLoginName, Function.identity()));

        ExcelReader excelReader = ExcelReader.create(ExcelType.XLSX, inputStream, 0);
        List<SubjectiveStructImportDto> subjectiveStructDtoList = excelReader.getObjectList(SubjectiveStructImportDto.class);

        // 课程下试卷编号map
        Map<String, String> coursePaperNumberMap = new HashMap<>();
        // 课程、试卷编号下大题号、小题号
        Map<String, String> coursePaperQuestionMap = new HashMap<>();

        List<MarkQuestion> markQuestions = new ArrayList<>();
        List<MarkUserQuestion> markUserQuestions = new ArrayList<>();
        boolean hasError = false;
        List<SubjectiveStructDto> subjectiveStructDtos = new ArrayList<>();
        // 按课程+试卷编号分组
        for (SubjectiveStructImportDto subjectiveStructImportDto : subjectiveStructDtoList) {
            SubjectiveStructDto subjectiveStructDto = new SubjectiveStructDto();
            BeanUtils.copyProperties(subjectiveStructImportDto, subjectiveStructDto);
            try {
                List<String> errorList = new ArrayList<>();

                if (StringUtils.isBlank(subjectiveStructDto.getCourseCode())) {
                    errorList.add("课程代码不能为空");
                }
                if (StringUtils.isBlank(subjectiveStructDto.getCourseName())) {
                    errorList.add("课程名称不能为空");
                }
                if (StringUtils.isBlank(subjectiveStructDto.getPaperNumber())) {
                    errorList.add("试卷编号不能为空");
                }
                if (StringUtils.isBlank(subjectiveStructDto.getMainTitle())) {
                    errorList.add("大题名称不能为空");
                }
                Integer mainNumber = subjectiveStructDto.getMainNumber();
                if (mainNumber == null) {
                    errorList.add("大题号不能为空");
                }
                Integer subNumber = subjectiveStructDto.getSubNumber();
                if (subNumber == null) {
                    errorList.add("小题号不能为空");
                }
                Double totalScore = subjectiveStructDto.getTotalScore();
                if (totalScore == null) {
                    errorList.add("小题满分不能为空");
                }
                Double intervalScore = subjectiveStructDto.getIntervalScore();
                if (intervalScore == null) {
                    errorList.add("小题号不能为空");
                }
                if (StringUtils.isBlank(subjectiveStructDto.getMarkerCodes())) {
                    errorList.add("绑定工号不能为空");
                }

                if (CollectionUtils.isNotEmpty(errorList)) {
                    throw ExceptionResultEnum.ERROR.exception(String.join(";", errorList));
                }

                if (totalScore < intervalScore) {
                    errorList.add("最小分不能大于小题满分");
                    throw ExceptionResultEnum.ERROR.exception(String.join(";", errorList));
                }

                String courseCode = subjectiveStructDto.getCourseCode().trim();
                String courseName = subjectiveStructDto.getCourseName().trim();
                String paperNumber = subjectiveStructDto.getPaperNumber().trim();

                // 课程是否存在
                if (!markPaperCourseMap.containsKey(courseCode)) {
                    errorList.add("未查询到此课程");
                    throw ExceptionResultEnum.ERROR.exception(String.join(";", errorList));
                } else {
                    MarkPaper markPaper = markPaperCourseMap.get(courseCode);
                    if (!courseName.equals(markPaper.getCourseName())) {
                        errorList.add("评卷设置中，课程名称为[" + markPaper.getCourseName() + "]");
                        throw ExceptionResultEnum.ERROR.exception(String.join(";", errorList));
                    }
                    subjectiveStructDto.setCourseId(markPaper.getCourseId());
                }

                // 试卷编号是否存在
                if (!markPaperPaperNumberMap.containsKey(paperNumber)) {
                    errorList.add("未查询到此试卷编号");
                    throw ExceptionResultEnum.ERROR.exception(String.join(";", errorList));
                }

                // 试卷编号是否设置了主观题
                if (subjectivePaperNumberSet.contains(paperNumber)) {
                    errorList.add("该试卷编号下已经存在主观题试卷结构，不允许导入");
                    throw ExceptionResultEnum.ERROR.exception(String.join(";", errorList));
                }

                // 校验课程下试卷编号唯一
                if (coursePaperNumberMap.containsKey(courseCode)) {
                    if (!paperNumber.equals(coursePaperNumberMap.get(courseCode))) {
                        errorList.add("课程存在不同试卷编号");
                        throw ExceptionResultEnum.ERROR.exception(String.join(";", errorList));
                    }
                } else {
                    coursePaperNumberMap.put(courseCode, paperNumber);
                }

                // 校验试卷编号唯一
                if (coursePaperNumberMap.containsValue(paperNumber)) {
                    for (Map.Entry<String, String> entry : coursePaperNumberMap.entrySet()) {
                        if (entry.getValue().equals(paperNumber) && !courseCode.equals(entry.getKey())) {
                            errorList.add("试卷编号已被其它课程占用");
                            throw ExceptionResultEnum.ERROR.exception(String.join(";", errorList));
                        }
                    }
                }

                // 题号重复校验
                String coursePaper = SystemConstant.mergeString(SystemConstant.HYPHEN, courseCode, paperNumber);
                String questionNumber = SystemConstant.mergeString(SystemConstant.HYPHEN, mainNumber, subNumber);
                if (coursePaperQuestionMap.containsKey(coursePaper)) {
                    if (questionNumber.equals(coursePaperQuestionMap.get(coursePaper))) {
                        errorList.add("存在相同大题号、小题号");
                        throw ExceptionResultEnum.ERROR.exception(String.join(";", errorList));
                    }
                } else {
                    coursePaperQuestionMap.put(coursePaper, questionNumber);
                }

                // 题号是否被客观题使用
                if (objectivePaperNumberSet.contains(SystemConstant.mergeString(SystemConstant.HYPHEN, paperNumber, mainNumber, subNumber))) {
                    errorList.add("大题号、小题号已被客观题使用");
                    throw ExceptionResultEnum.ERROR.exception(String.join(";", errorList));
                }

                // 组装数据
                assembleSubjectiveImportData(examId, subjectiveStructDto, markQuestions, markUserQuestions, sysUserMap);
            } catch (Exception e) {
                hasError = true;
                subjectiveStructDto.setErrorMsg(e.getMessage());
            } finally {
                subjectiveStructDtos.add(subjectiveStructDto);
            }
        }

        map.put(SystemConstant.ERROR_DATA_LIST, subjectiveStructDtos);
        map.put(SystemConstant.DATASOURCE, markQuestions);
        map.put(SystemConstant.DATASOURCE1, markUserQuestions);
        map.put(SystemConstant.DATA_COUNT, subjectiveStructDtos.size());
        map.put(SystemConstant.HAS_ERROR_DATA, hasError);

        return map;
    }

    private void assembleObjectiveImportData(Long examId, ObjectiveStructDto objectiveStructDto, List<MarkQuestion> markQuestions, List<MarkQuestionAnswer> markQuestionAnswers, Map<String, SysUserResult> sysUserMap) {
        MarkQuestion markQuestion = new MarkQuestion();
        markQuestion.setId(SystemConstant.getDbUuid());
        markQuestion.setExamId(examId);
        markQuestion.setCourseId(objectiveStructDto.getCourseId());
        markQuestion.setPaperNumber(objectiveStructDto.getPaperNumber());
        markQuestion.setObjective(true);
        markQuestion.setMainNumber(objectiveStructDto.getMainNumber());
        markQuestion.setSubNumber(objectiveStructDto.getSubNumber());
        markQuestion.setMainTitle(objectiveStructDto.getMainTitle());
        markQuestion.setOptionCount(objectiveStructDto.getOptionCount());
        markQuestion.setQuestionType(objectiveStructDto.getQuestionType());
        markQuestion.setTotalScore(objectiveStructDto.getTotalScore());
        markQuestions.add(markQuestion);

        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, objectiveStructDto.getPaperNumber());
        String paperType = markPaper.getPaperType();
        if (StringUtils.isNotBlank(paperType)) {
            for (String type : paperType.split(SystemConstant.COMMA)) {
                MarkQuestionAnswer markQuestionAnswer = new MarkQuestionAnswer();
                markQuestionAnswer.setId(SystemConstant.getDbUuid());
                markQuestionAnswer.setExamId(examId);
                markQuestionAnswer.setPaperNumber(objectiveStructDto.getPaperNumber());
                markQuestionAnswer.setPaperType(type);
                markQuestionAnswer.setMainNumber(objectiveStructDto.getMainNumber());
                markQuestionAnswer.setSubNumber(objectiveStructDto.getSubNumber());
                markQuestionAnswer.setAnswer(objectiveStructDto.getAnswer());
                markQuestionAnswer.setObjectivePolicy(ObjectivePolicy.NONE);
                markQuestionAnswers.add(markQuestionAnswer);
            }
        }
    }

    private void assembleSubjectiveImportData(Long examId, SubjectiveStructDto subjectiveStructDto, List<MarkQuestion> markQuestions, List<MarkUserQuestion> markUserQuestions, Map<String, SysUserResult> sysUserMap) {
        MarkQuestion markQuestion = new MarkQuestion();
        markQuestion.setId(SystemConstant.getDbUuid());
        markQuestion.setExamId(examId);
        markQuestion.setCourseId(subjectiveStructDto.getCourseId());
        markQuestion.setPaperNumber(subjectiveStructDto.getPaperNumber());
        markQuestion.setObjective(false);
        markQuestion.setMainNumber(subjectiveStructDto.getMainNumber());
        markQuestion.setSubNumber(subjectiveStructDto.getSubNumber());
        markQuestion.setMainTitle(subjectiveStructDto.getMainTitle());
        markQuestion.setOptionCount(0);
        markQuestion.setQuestionType(QuestionType.QUESTION_ANSWER.getValue());
        markQuestion.setDoubleRate(0D);// 单评为0，双评为1
        markQuestion.setTotalScore(subjectiveStructDto.getTotalScore());
        // 间隔分（整数默认1，小数默认0.5）
        markQuestion.setIntervalScore(subjectiveStructDto.getIntervalScore());
        markQuestions.add(markQuestion);

        // 评卷员
        String markerCodes = subjectiveStructDto.getMarkerCodes();
        if (StringUtils.isNotBlank(markerCodes)) {
            for (String code : markerCodes.split(SystemConstant.COMMA)) {
                if (sysUserMap.containsKey(code.trim())) {
                    MarkUserQuestion markUserQuestion = new MarkUserQuestion();
                    markUserQuestion.setId(SystemConstant.getDbUuid());
                    markUserQuestion.setExamId(examId);
                    markUserQuestion.setPaperNumber(subjectiveStructDto.getPaperNumber());
                    markUserQuestion.setQuestionId(markQuestion.getId());
                    markUserQuestion.setUserId(sysUserMap.get(code.trim()).getId());
                    markUserQuestion.setEnable(true);
                    markUserQuestions.add(markUserQuestion);
                } else {
                    throw ExceptionResultEnum.ERROR.exception("工号" + code + "用户不存在");
                }
            }
        }

    }
}
