package com.qmth.distributed.print.business.templete.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.dto.ExamDetailPdfDownloadDto;
import com.qmth.distributed.print.business.bean.dto.ExaminationExportDto;
import com.qmth.distributed.print.business.bean.result.TSyncExamStudentScoreResult;
import com.qmth.distributed.print.business.entity.TCStatisticsTemp;
import com.qmth.distributed.print.business.entity.TSyncExamStudentScore;
import com.qmth.distributed.print.business.entity.TeachClazz;
import com.qmth.distributed.print.business.enums.ImageTrajectoryEnum;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.service.TaskLogicService;
import com.qmth.distributed.print.business.util.CreatePdfUtil;
import com.qmth.teachcloud.common.base.BaseEntity;
import com.qmth.teachcloud.common.bean.dto.excel.DescribeImportDto;
import com.qmth.teachcloud.common.bean.dto.excel.StatisticsImportDto;
import com.qmth.teachcloud.common.bean.dto.excel.SysUserImportDto;
import com.qmth.teachcloud.common.bean.dto.excel.export.SysUserErrorExportDto;
import com.qmth.teachcloud.common.bean.params.ArraysParams;
import com.qmth.teachcloud.common.bean.result.DictionaryResult;
import com.qmth.teachcloud.common.bean.vo.FilePathVo;
import com.qmth.teachcloud.common.bean.vo.PrintPathVo;
import com.qmth.teachcloud.common.config.DictionaryConfig;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.*;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.LocalCatalogEnum;
import com.qmth.teachcloud.common.enums.PdfTypeEnum;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.util.*;
import com.qmth.teachcloud.common.util.excel.ExcelError;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.InputStream;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * @Description: 任务处理逻辑impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
@Service
public class TaskLogicServiceImpl implements TaskLogicService {
    private final static Logger log = LoggerFactory.getLogger(TaskLogicServiceImpl.class);

    @Resource
    ExamPrintPlanService examPrintPlanService;
    @Resource
    ExamTaskService examTaskService;
    @Resource
    ExamTaskDetailService examTaskDetailService;
    @Resource
    ExamCardService examCardService;
    @Resource
    BasicAttachmentService basicAttachmentService;
    @Resource
    ExamDetailService examDetailService;
    @Resource
    BasicCourseService basicCourseService;
    @Resource
    CreatePdfUtil createPdfUtil;
    @Resource
    TBTaskPdfService tbTaskPdfService;
    @Resource
    BasicRoleDataPermissionService basicRoleDataPermissionService;
    @Resource
    SysUserService sysUserService;
    @Resource
    FileStoreUtil fileStoreUtil;
    @Resource
    SysOrgService sysOrgService;
    @Resource
    TCStatisticsService tcStatisticsService;
    @Resource
    TCStatisticsTempService tcStatisticsTempService;
    @Resource
    TSyncExamStudentScoreService tSyncExamStudentScoreService;
    @Resource
    @Lazy
    PrintCommonService printCommonService;
    @Resource
    DownloadService downloadService;
    @Resource
    TeachClazzService teachClazzService;
    @Resource
    DictionaryConfig dictionaryConfig;
    @Resource
    FileUploadService fileUploadService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Map<String, Object> executeExaminationLogic(Map<String, Object> map) throws Exception {
        List<ExaminationExportDto> examinationExportDtoList = examDetailService.findExaminationExportDtoDatasource(map);

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        ExcelUtil.excelMake(ExaminationExportDto.class, examinationExportDtoList, outputStream);
        InputStream in = new ByteArrayInputStream(outputStream.toByteArray());

        StringJoiner stringJoiner = SystemConstant.getDirName(UploadFileEnum.FILE, true);
        stringJoiner.add(SystemConstant.getNanoId()).add(".").add(SystemConstant.XLSX);

        JSONObject jsonObject = new JSONObject();
        String dirName = FileUtil.replaceSplit(stringJoiner.toString());
        boolean oss = dictionaryConfig.sysDomain().isOss();
        if (oss || (!oss && dictionaryConfig.fssPrivateDomain().getConfig().startsWith(SystemConstant.START_PARENT))) {//上传至oss
            fileStoreUtil.ossUpload(dirName, in, DigestUtils.md5Hex(new ByteArrayInputStream(outputStream.toByteArray())), fileStoreUtil.getUploadEnumByPath(dirName).getFssType());
            jsonObject.put(SystemConstant.TYPE, oss ? SystemConstant.OSS : SystemConstant.LOCAL);
            jsonObject.put(SystemConstant.PATH, dirName);
        } else {
            dirName = FileUtil.replaceSplit((dictionaryConfig.fssPublicDomain().getConfig() + File.separator + dirName));
            fileStoreUtil.copyInputStreamToFile(in, new File(stringJoiner.toString()), DigestUtils.md5Hex(new ByteArrayInputStream(outputStream.toByteArray())), LocalCatalogEnum.LOCAL_FILE);
            jsonObject.put(SystemConstant.TYPE, SystemConstant.LOCAL);
            jsonObject.put(SystemConstant.PATH, dirName);
        }
        jsonObject.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.FILE);
        map.put(SystemConstant.PATH, jsonObject.toString());
        map.put(SystemConstant.DATA_COUNT, examinationExportDtoList.size());
        return map;
    }

    /**
     * 下载pdf
     *
     * @param map
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> executeDownloadPdfLogic(Map<String, Object> map) {
        List<File> sourceFileList = new ArrayList<>();
        File zipFile = null;
        String zipLocalRootPath = null;
        try {
            TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
            JSONArray jsonArray = JSONArray.parseArray(tbTask.getRemark());
            ArraysParams arraysParams = new ArraysParams(jsonArray.toArray(new Long[jsonArray.size()]));
            if (Objects.isNull(arraysParams) || arraysParams.getIds().length == 0) {
                throw ExceptionResultEnum.ERROR.exception("请选择要下载的数据");
            }

            List<ExamDetailPdfDownloadDto> examDetailList = examDetailService.findPdfDownload(Arrays.asList(arraysParams.getIds()));
            if (CollectionUtils.isNotEmpty(examDetailList)) {
                Long time = System.currentTimeMillis();

                zipFile = SystemConstant.getFileTempDirVar(SystemConstant.ZIP_PREFIX);
                zipLocalRootPath = zipFile.getParent() + File.separator + time;

                for (ExamDetailPdfDownloadDto e : examDetailList) {
                    StringJoiner dirPath = new StringJoiner("")
                            .add(zipLocalRootPath).add(File.separator)
                            .add(e.getSemesterName()).add(File.separator)
                            .add(e.getExamName()).add(File.separator)
                            .add(e.getCourseNameCode()).add(File.separator)
                            .add(e.getPackageCode()).add(File.separator)
                            .add(e.getCourseNameCode()).add(SystemConstant.HYPHEN)
                            .add(e.getPaperNumber()).add(SystemConstant.HYPHEN);
                    //试卷合并文件
                    if (Objects.nonNull(e.getAttachmentId())) {
                        BasicAttachment attachment = basicAttachmentService.getById(e.getAttachmentId());
                        if (attachment != null) {
                            String fileName = dirPath + PdfTypeEnum.PAPER.getTitle() + SystemConstant.PDF_PREFIX;
                            File paperFile = fileUploadService.downloadFile(attachment, fileName);
                            if (Objects.nonNull(paperFile)) {
                                sourceFileList.add(paperFile);
                            }
                        }
                    }

                    //题卡合并文件
                    if (Objects.nonNull(e.getCardAttachmentId())) {
                        BasicAttachment cardAttachment = basicAttachmentService.getById(e.getCardAttachmentId());
                        if (cardAttachment != null) {
                            String fileName = dirPath + PdfTypeEnum.CARD_A3.getTitle() + SystemConstant.PDF_PREFIX;
                            File cardFile = fileUploadService.downloadFile(cardAttachment, fileName);
                            if (Objects.nonNull(cardFile)) {
                                sourceFileList.add(cardFile);
                            }
                        }
                    }

                    //卷袋贴/签到表/登记表文件
                    if (Objects.nonNull(e.getAttachmentPath())) {
                        JSONObject js = JSONObject.parseObject(e.getAttachmentPath());
                        List<PrintPathVo> printPathVoList = JSON.parseArray(js.getString(SystemConstant.PATH), PrintPathVo.class);
                        for (PrintPathVo pathVo : printPathVoList) {
                            String fileName = dirPath + pathVo.getPrintType().getName() + SystemConstant.PDF_PREFIX;
                            File cardFile = fileUploadService.downloadFile(pathVo.getType(), pathVo.getUploadType(), pathVo.getPdfPath(), fileName);
                            if (Objects.nonNull(cardFile)) {
                                sourceFileList.add(cardFile);
                            }
                        }
                    }

                    //考场所有文件合并的大pdf文件
                    if (Objects.nonNull(e.getMergePdfPath())) {
                        FilePathVo pathVo = JSON.parseObject(e.getMergePdfPath(), FilePathVo.class);
                        String fileName = dirPath + PdfTypeEnum.MERGE_PDF.getTitle() + SystemConstant.PDF_PREFIX;
                        File mergeFile = fileUploadService.downloadFile(pathVo.getType(), pathVo.getUploadType(), pathVo.getPath(), fileName);
                        if (Objects.nonNull(mergeFile)) {
                            sourceFileList.add(mergeFile);
                        }
                    }
                }
                if (!CollectionUtils.isEmpty(sourceFileList)) {
                    boolean oss = dictionaryConfig.sysDomain().isOss();
                    StringJoiner stringJoiner = new StringJoiner("");
                    if (!oss && Objects.nonNull(dictionaryConfig.fssPublicDomain()) && !StringUtils.isBlank(dictionaryConfig.fssPublicDomain().getConfig()) && !dictionaryConfig.fssPublicDomain().getConfig().startsWith(SystemConstant.START_PARENT)) {
                        stringJoiner.add(dictionaryConfig.fssPublicDomain().getConfig()).add(File.separator);
                    }
                    stringJoiner = SystemConstant.getDirName(stringJoiner, UploadFileEnum.FILE, true);
                    stringJoiner.add(SystemConstant.getNanoId()).add(File.separator);
                    stringJoiner.add("印刷任务管理批量下载印品数据包").add(SystemConstant.ZIP_PREFIX);
                    String zipDirName = FileUtil.replaceSplit(stringJoiner.toString());
                    SysUser sysUser = sysUserService.getById(tbTask.getCreateId());
                    JSONObject jsonObject = SystemConstant.createZip(zipFile, zipLocalRootPath, zipDirName, new String(Base64Util.decode(sysUser.getPassword())));
                    tbTask.setResultFilePath(jsonObject.toJSONString());
                }
            }
            map.put(SystemConstant.SIZE, Objects.nonNull(sourceFileList) ? sourceFileList.size() : 0);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            if (Objects.nonNull(zipFile)) {
                zipFile.delete();
            }
            if (Objects.nonNull(zipLocalRootPath)) {
                ConvertUtil.delFolder(zipLocalRootPath);
            }
            if (!CollectionUtils.isEmpty(sourceFileList)) {
                for (File file : sourceFileList) {
                    file.delete();
                }
            }
        }
        return map;
    }

    @Transactional
    @Override
    public Map<String, Object> executeImportSysUserLogic(Map<String, Object> map) throws Exception {
        InputStream inputStream = (InputStream) map.get("inputStream");

        List<SysUserErrorExportDto> errorDataList = new ArrayList<>();
        AtomicInteger totalInteger = new AtomicInteger(0);
        AtomicInteger successInteger = new AtomicInteger(0);
        List<LinkedMultiValueMap<Integer, Object>> finalList = ExcelUtil.excelReader(inputStream, Lists.newArrayList(SysUserImportDto.class, DescribeImportDto.class), (finalExcelList, finalColumnNameList, finalExcelErrorList) -> {
            Map<String, SysUserImportDto> checkDtoMap = new HashMap<>();

            for (int i = 0; i < finalExcelList.size(); i++) {
                LinkedMultiValueMap<Integer, Object> excelMap = finalExcelList.get(i);
                List<Object> sysUserImportDtoList = excelMap.get(i);
                if (CollectionUtils.isEmpty(sysUserImportDtoList)) {
                    continue;
                }
                map.put(SystemConstant.DATA_COUNT, sysUserImportDtoList.size());
                List<SysUserImportDto> removeDataList = new ArrayList<>();
                Map<String, Integer> phoneNumberMap = new HashMap<>();
                for (int y = 0; y < Objects.requireNonNull(sysUserImportDtoList).size(); y++) {
                    SysUserImportDto sysUserImportDto = (SysUserImportDto) sysUserImportDtoList.get(y);
                    totalInteger.getAndIncrement();
                    SysUserErrorExportDto sysUserErrorExportDto = new SysUserErrorExportDto();
                    BeanUtils.copyProperties(sysUserImportDto, sysUserErrorExportDto);

                    String name = sysUserImportDto.getName();
                    String loginName = sysUserImportDto.getLoginName();
                    String phoneNumber = sysUserImportDto.getPhoneNumber();
                    String orgName = sysUserImportDto.getOrgName();
                    String roleName = sysUserImportDto.getRoleName();

                    StringJoiner errorStringJoiner = new StringJoiner(";");
                    String errorStringEmpty = ExcelUtil.checkExcelField(sysUserImportDto);
                    if (errorStringEmpty.length() > 0) {
                        errorStringJoiner.add(errorStringEmpty);
                    }
                    if (checkDtoMap.containsKey(loginName)) {
                        SysUserImportDto primaryCell = checkDtoMap.get(loginName);
                        String priName = primaryCell.getName();
                        String priPhoneNumber = primaryCell.getPhoneNumber();
                        String priOrgName = primaryCell.getOrgName();
                        if (!Objects.equals(priName, name)) {
                            errorStringJoiner.add("用户名/工号[" + loginName + "]的用户存在不同的姓名");
                        }
                        if (!Objects.equals(priPhoneNumber, phoneNumber)) {
                            errorStringJoiner.add("用户名/工号[" + loginName + "]的用户存在不同的手机号");
                        }
                        if (!Objects.equals(priOrgName, orgName)) {
                            errorStringJoiner.add("用户名/工号[" + loginName + "]的用户存在不同的组织架构");
                        }
                        String priRoleName = primaryCell.getRoleName();
                        if (SystemConstant.strNotNull(roleName)) {
                            priRoleName = priRoleName + SystemConstant.COMMA + roleName;
                            primaryCell.setRoleName(priRoleName);
                        }
                    } else {
                        checkDtoMap.put(loginName, sysUserImportDto);
                    }

                    if (StringUtils.isNotBlank(loginName) && !loginName.matches(SystemConstant.REGULAR_EXPRESSION_OF_CODE)) {
                        errorStringJoiner.add("用户名/工号[" + loginName + "]不符合规范");
                    }
                    if (StringUtils.isNotBlank(phoneNumber) && !phoneNumber.matches(SystemConstant.REGULAR_EXPRESSION_OF_PHONE)) {
                        errorStringJoiner.add("手机号[" + phoneNumber + "]不符合输入规范");
                    }
                    if (StringUtils.isNotBlank(phoneNumber)) {
                        if (phoneNumberMap.containsKey(phoneNumber)) {
                            errorStringJoiner.add("手机号[" + phoneNumber + "]在文件中重复");
                        } else {
                            phoneNumberMap.put(phoneNumber, y);
                        }
                    }
                    String errorString = errorStringJoiner.toString();
                    if (errorString.length() > 0) {
                        sysUserErrorExportDto.setErrorMsg(errorString);
                        errorDataList.add(sysUserErrorExportDto);
                        removeDataList.add(sysUserImportDto);
                    } else {
                        // 校验通过的数据
                        successInteger.getAndIncrement();
                    }
                }
                sysUserImportDtoList.removeAll(removeDataList);
            }
            if (!errorDataList.isEmpty()) {
                map.put(SystemConstant.ERROR_DATA_LIST, errorDataList);
            }
            map.put(SystemConstant.DATA_COUNT, totalInteger.get());
            map.put(SystemConstant.SUCCESS_DATA_COUNT, successInteger.get());
            map.put(SystemConstant.ERROR_DATA_COUNT, errorDataList.size());
            return finalExcelList;
        }, 2);
        return sysUserService.executeSysUserImportLogic(finalList, map);
    }

    @Override
    @Transactional
    public Map<String, Object> executeImportStatisticsLogic(Map<String, Object> map) throws Exception {
        InputStream inputStream = (InputStream) map.get("inputStream");
        Long examId = SystemConstant.convertIdToLong(String.valueOf(map.get("examId")));
        SysUser sysUser = (SysUser) map.get(SystemConstant.USER);

        List<LinkedMultiValueMap<Integer, Object>> finalList = ExcelUtil.excelReader(inputStream, Lists.newArrayList(StatisticsImportDto.class), (finalExcelList, finalColumnNameList, finalExcelErrorList) -> {
            // 只允许导入一个sheet
            if (finalExcelList.size() > 1) {
                throw ExceptionResultEnum.ERROR.exception("excel中只允许有一个sheet");
            }
            if (finalExcelErrorList.size() > 0) {
                throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(finalExcelErrorList));
            }
            List<ExcelError> excelErrorTemp = new ArrayList<>();
            // 校验系统中是否存在
            Map<String, SysOrg> collegeOrgMap = new HashMap<>();
            Map<String, SysOrg> teachingRoomMap = new HashMap<>();
            Map<String, BasicCourse> courseMap = new HashMap<>();
            Map<String, DictionaryResult> clazzMap = new HashMap<>();
            String batchNo = SystemConstant.getNanoId();
            List<TCStatisticsTemp> tcStatisticsImportTempList = new ArrayList<>();

            for (int i = 0; i < finalExcelList.size(); i++) {
                LinkedMultiValueMap<Integer, Object> excelMap = finalExcelList.get(i);
                List<Object> statisticsTempList = excelMap.get(i);
                for (int y = 0; y < Objects.requireNonNull(statisticsTempList).size(); y++) {
                    StatisticsImportDto statisticsImportDto = (StatisticsImportDto) statisticsTempList.get(y);
                    SysOrg collegeOrg = this.validSysOrgExists(collegeOrgMap, statisticsImportDto.getCollegeName(), sysUser.getSchoolId(), excelErrorTemp, (y + 1), (i + 1), "开课学院");
                    SysOrg teachingRoomOrg = this.validSysOrgExists(teachingRoomMap, statisticsImportDto.getTeachingRoomName(), sysUser.getSchoolId(), excelErrorTemp, (y + 1), (i + 1), "开课学院");
                    BasicCourse basicCourse = this.validBasicCourseExists(courseMap, statisticsImportDto.getCourseName(), sysUser.getSchoolId(), statisticsImportDto.getCourseCode(), excelErrorTemp, (y + 1), (i + 1));
                    String teacherName = statisticsImportDto.getTeacherName();
                    String clazzNames = statisticsImportDto.getClazzName();
                    List<DictionaryResult> dictionaryResultList = new ArrayList<>();
                    for (String clazzName : clazzNames.split(",")) {
                        DictionaryResult clazz = this.validBasicClazzExists(clazzMap, clazzName, sysUser.getSchoolId(), excelErrorTemp, (y + 1), (i + 1));
                        dictionaryResultList.add(clazz);
                    }
                    String clazzIds = dictionaryResultList.stream().map(e -> String.valueOf(e.getId())).collect(Collectors.joining(","));


                    if (excelErrorTemp.size() == 0) {

                        TCStatisticsTemp tcStatistics = new TCStatisticsTemp(
                                examId,
                                collegeOrg.getId(),
                                collegeOrg.getName(),
                                teachingRoomOrg.getId(),
                                teachingRoomOrg.getName(),
                                basicCourse.getName(),
                                basicCourse.getCode(),
                                teacherName,
                                clazzIds,
                                clazzNames,
                                batchNo,
                                sysUser.getId());
                        tcStatisticsImportTempList.add(tcStatistics);
                    }
                }
            }
            if (excelErrorTemp.size() > 0) {
                List<String> errors = excelErrorTemp.stream().map(ExcelError::getExcelErrorType).collect(Collectors.toList());
                throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(errors));
            }
            //加入删除(根据考试id和当前登录人)
            tcStatisticsService.removeImportData(examId, sysUser.getId());
            tcStatisticsTempService.saveBatch(tcStatisticsImportTempList);
            map.put("dataCount", tcStatisticsImportTempList.size());
            tcStatisticsService.importJoinData(tcStatisticsImportTempList);
            tcStatisticsTempService.removeByIds(tcStatisticsImportTempList.stream().map(BaseEntity::getId).collect(Collectors.toSet()));
            return finalExcelList;
        }, 2);
        return map;
    }

    /**
     * 成绩导出
     *
     * @param map
     * @return
     * @throws Exception
     */
    @Override
    public Map<String, Object> executeExportScoreLogic(Map<String, Object> map) throws Exception {
        ByteArrayOutputStream fos = null;
        try {
            SysUser sysUser = (SysUser) map.get(SystemConstant.USER);
            Long semesterId = null, examId = null, clazzId = null;
            String courseCode = null;
            semesterId = Objects.nonNull(map.get("semesterId")) ? Long.valueOf(map.get("semesterId").toString()) : null;
            examId = Objects.nonNull(map.get("examId")) ? Long.valueOf(map.get("examId").toString()) : null;
            clazzId = Objects.nonNull(map.get("clazzId")) ? Long.valueOf(map.get("clazzId").toString()) : null;
            courseCode = Objects.nonNull(map.get("courseCode")) ? (String) map.get("courseCode") : null;
            String servletPath = Objects.nonNull(map.get("servletPath")) ? (String) map.get("servletPath") : null;

            fos = new ByteArrayOutputStream();
            List<TSyncExamStudentScoreResult> tSyncExamStudentScoreResultList = tSyncExamStudentScoreService.export(sysUser, semesterId, examId, clazzId, courseCode, servletPath);
            ExcelUtil.excelMake(TSyncExamStudentScoreResult.class, tSyncExamStudentScoreResultList, fos);
            String excelExportFilePath = printCommonService.saveTaskAttachment(fos, (boolean) map.get(SystemConstant.OSS));
            map.computeIfAbsent("count", v -> Objects.isNull(tSyncExamStudentScoreResultList) ? 0 : tSyncExamStudentScoreResultList.size());
            TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
            tbTask.setResultFilePath(excelExportFilePath);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
        } finally {
            if (Objects.nonNull(fos)) {
                fos.flush();
                fos.close();
            }
        }
        return map;
    }

    /**
     * 成绩轨迹下载
     *
     * @param map
     * @return
     * @throws Exception
     */
    @Override
    @Transactional
    public Map<String, Object> executeDownloadScoreLogic(Map<String, Object> map) throws Exception {
        File zipFile = null;
        List<File> sourceFiles = null;
        List<TSyncExamStudentScore> errorTSyncExamStudentScoreList = null;
        boolean oss = dictionaryConfig.sysDomain().isOss();
        try {
            SysUser sysUser = (SysUser) map.get(SystemConstant.USER);
            Long semesterId = null, examId = null, clazzId = null;
            String courseCode = null;
            semesterId = Objects.nonNull(map.get("semesterId")) ? (Long) map.get("semesterId") : null;
            examId = Objects.nonNull(map.get("examId")) ? (Long) map.get("examId") : null;
            clazzId = Objects.nonNull(map.get("clazzId")) ? (Long) map.get("clazzId") : null;
            courseCode = Objects.nonNull(map.get("courseCode")) ? (String) map.get("courseCode") : null;

            String servletPath = Objects.nonNull(map.get("servletPath")) ? (String) map.get("servletPath") : null;

            List<TSyncExamStudentScoreResult> tSyncExamStudentScoreResultList = tSyncExamStudentScoreService.export(sysUser, semesterId, examId, clazzId, courseCode, servletPath);
            if (Objects.nonNull(tSyncExamStudentScoreResultList) && tSyncExamStudentScoreResultList.size() > 0) {
                List<TSyncExamStudentScore> tSyncExamStudentScoreList = new Gson().fromJson(JacksonUtil.parseJson(tSyncExamStudentScoreResultList), new TypeToken<List<TSyncExamStudentScore>>() {
                }.getType());
                Long time = System.currentTimeMillis();
                StringJoiner stringJoiner = new StringJoiner("");
                if (!oss && Objects.nonNull(dictionaryConfig.fssPublicDomain()) && !StringUtils.isBlank(dictionaryConfig.fssPublicDomain().getConfig()) && !dictionaryConfig.fssPublicDomain().getConfig().startsWith(SystemConstant.START_PARENT)) {
                    stringJoiner.add(dictionaryConfig.fssPublicDomain().getConfig()).add(File.separator);
                }
                stringJoiner = SystemConstant.getDirName(stringJoiner, UploadFileEnum.FILE, true);
                stringJoiner.add("成绩轨迹下载_" + time).add(SystemConstant.ZIP_PREFIX);

                String zipDirName = FileUtil.replaceSplit(stringJoiner.toString());
                zipFile = SystemConstant.getFileTempDirVar(SystemConstant.ZIP_PREFIX);

                sourceFiles = new LinkedList<>();
                List<TSyncExamStudentScore> updateTSyncExamStudentScoreList = new ArrayList<>();
                errorTSyncExamStudentScoreList = new ArrayList<>();
                for (TSyncExamStudentScore t : tSyncExamStudentScoreList) {
                    try {
                        boolean update = Objects.isNull(t.getTrajectoryUrls()) ? true : false;
                        t = tSyncExamStudentScoreService.createImageTrajectory(t, ImageTrajectoryEnum.DOWNLOAD, false, sysUser.getId());
                        if (Objects.nonNull(t.getTrajectoryFileList())) {
                            sourceFiles.addAll(t.getTrajectoryFileList());
                        } else {
                            errorTSyncExamStudentScoreList.add(t);
                        }
                        if (update || Objects.isNull(t.getTrajectoryUrls())) {
                            updateTSyncExamStudentScoreList.add(t);
                        }
                    } catch (Exception e) {
                        log.error(SystemConstant.LOG_ERROR, e);
                        errorTSyncExamStudentScoreList.add(t);
                    }
                }
                tSyncExamStudentScoreService.saveOrUpdateBatch(updateTSyncExamStudentScoreList);
                if (!sourceFiles.isEmpty()) {
                    JSONObject jsonObject = SystemConstant.createZip(zipFile, sourceFiles, zipDirName);
                    TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
                    tbTask.setResultFilePath(jsonObject.toJSONString());
                } else {
                    throw ExceptionResultEnum.ERROR.exception("阅卷方式未使用轨迹阅卷，无轨迹图");
                }
            } else {
                throw ExceptionResultEnum.ERROR.exception("没有可下载数据");
            }
            map.computeIfAbsent("count", v -> tSyncExamStudentScoreResultList.size());
            if (Objects.nonNull(errorTSyncExamStudentScoreList) && errorTSyncExamStudentScoreList.size() > 0) {
                StringJoiner stringJoiner = new StringJoiner("").add("\r\n");
                for (TSyncExamStudentScore t : errorTSyncExamStudentScoreList) {
                    stringJoiner.add("[").add(t.getName()).add(",").add(t.getExamNumber()).add("]\r\n");
                }
                List<TSyncExamStudentScore> finalErrorTSyncExamStudentScoreList = errorTSyncExamStudentScoreList;
                map.computeIfAbsent("error", v -> "其中未下载成功数据" + finalErrorTSyncExamStudentScoreList.size() + "条:" + stringJoiner.toString());
            }
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            if (Objects.nonNull(zipFile)) {
                zipFile.delete();
            }
            if (oss && !CollectionUtils.isEmpty(sourceFiles)) {
                for (File f : sourceFiles) {
                    f.delete();
                }
            }
        }
        return map;
    }

    /**
     * 验证机构是否存在
     *
     * @param orgMap         机构map
     * @param key            机构名称
     * @param schoolId       学校id
     * @param excelErrorTemp 错误信息
     * @param row            行
     * @param sheet          区
     * @param cloumnName     cloumnName
     * @return 机构
     */
    private SysOrg validSysOrgExists(Map<String, SysOrg> orgMap,
                                     String key,
                                     Long schoolId,
                                     List<ExcelError> excelErrorTemp,
                                     int row,
                                     int sheet,
                                     String cloumnName) {
        SysOrg sysOrg = null;
        if (!orgMap.containsKey(key)) {//不存在查询
            QueryWrapper<SysOrg> sysOrgQueryWrapper = new QueryWrapper<>();
            sysOrgQueryWrapper.lambda().eq(SysOrg::getSchoolId, schoolId)
                    .eq(SysOrg::getName, key)
                    .eq(SysOrg::getEnable, true);
            sysOrg = sysOrgService.getOne(sysOrgQueryWrapper);
        } else {
            sysOrg = orgMap.get(key);
        }
        if (Objects.isNull(sysOrg)) {
            excelErrorTemp.add(new ExcelError(row, "excel第" + sheet + "个sheet第" + row + "行[" + cloumnName + "]不存在"));
        } else {
            SysOrg finalSysOrg = sysOrg;
            orgMap.computeIfAbsent(key, v -> finalSysOrg);
        }
        return sysOrg;
    }

    /**
     * 验证课程是否存在
     *
     * @param courseMap      courseMap
     * @param key            课程名称
     * @param schoolId       学校id
     * @param value          课程编号
     * @param excelErrorTemp 错误信息
     * @param row            行
     * @param sheet          区
     * @return 基础课程
     */
    private BasicCourse validBasicCourseExists(Map<String, BasicCourse> courseMap,
                                               String key,
                                               Long schoolId,
                                               String value,
                                               List<ExcelError> excelErrorTemp,
                                               int row,
                                               int sheet) {
        BasicCourse basicCourse = null;
        if (!courseMap.containsKey(key)) {//不存在查询
            QueryWrapper<BasicCourse> basicCourseQueryWrapper = new QueryWrapper<>();
            basicCourseQueryWrapper.lambda().eq(BasicCourse::getSchoolId, schoolId)
                    .eq(BasicCourse::getName, key)
                    .eq(BasicCourse::getCode, value)
                    .eq(BasicCourse::getEnable, true);
            basicCourse = basicCourseService.getOne(basicCourseQueryWrapper);
        } else {
            basicCourse = courseMap.get(key);
        }
        if (Objects.isNull(basicCourse)) {
            excelErrorTemp.add(new ExcelError(row, "excel第" + sheet + "个sheet第" + row + "行[" + "课程代码" + "]不存在"));
        } else {
            BasicCourse finalBasicCourse = basicCourse;
            courseMap.computeIfAbsent(key, v -> finalBasicCourse);
        }
        return basicCourse;
    }


    /**
     * 验证班级是否存在
     *
     * @param clazzMap       班级map
     * @param key            班级名称
     * @param schoolId       学校id
     * @param excelErrorTemp 错误信息
     * @param row            行
     * @param sheet          区
     * @return 班级信息
     */
    private DictionaryResult validBasicClazzExists(Map<String, DictionaryResult> clazzMap,
                                                   String key,
                                                   Long schoolId,
                                                   List<ExcelError> excelErrorTemp,
                                                   int row,
                                                   int sheet) {
        DictionaryResult clazz = null;
        if (!clazzMap.containsKey(key)) {//不存在查询
            TeachClazz teachClazz = teachClazzService.getOne(new QueryWrapper<TeachClazz>().lambda().eq(TeachClazz::getSchoolId, schoolId).eq(TeachClazz::getClazzName, key));
            if (Objects.nonNull(teachClazz)) {
                clazz = new DictionaryResult();
                clazz.setId(teachClazz.getId());
                clazz.setName(teachClazz.getClazzName());
            } else {
//                QueryWrapper<BasicClazz> basicClazzQueryWrapper = new QueryWrapper<>();
//                basicClazzQueryWrapper.lambda().eq(BasicClazz::getSchoolId, schoolId)
//                        .eq(BasicClazz::getClazzName, key)
//                        .eq(BasicClazz::getEnable, true);
//                BasicClazz basicClazz = basicClazzService.getOne(basicClazzQueryWrapper);
//                if (Objects.nonNull(basicClazz)) {
//                    clazz = new DictionaryResult();
//                    clazz.setId(basicClazz.getId());
//                    clazz.setName(basicClazz.getClazzName());
//                }
            }
        } else {
            clazz = clazzMap.get(key);
        }
        if (Objects.isNull(clazz)) {
            excelErrorTemp.add(new ExcelError(row, "excel第" + sheet + "个sheet第" + row + "行[" + "班级名称" + "]不存在"));
        } else {
            DictionaryResult finalClazz = clazz;
            clazzMap.computeIfAbsent(key, v -> finalClazz);
        }
        return clazz;
    }
}
