package com.qmth.distributed.print.api;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.result.EditResult;
import com.qmth.distributed.print.business.entity.TBVersion;
import com.qmth.distributed.print.business.service.TBVersionService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FieldUniqueEnum;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.util.Objects;

/**
 * <p>
 * 版本管理表 前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2025-02-27
 */
@Api(tags = "版本管理Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_VERSION)
public class TBVersionController {
    private final static Logger log = LoggerFactory.getLogger(TBVersionController.class);

    @Resource
    TBVersionService tbVersionService;

    @ApiOperation(value = "版本管理列表")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "分页查询", response = TBVersion.class)})
    public Result versionList(@ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                              @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(tbVersionService.versionList(new Page<>(pageNumber, pageSize)));
    }

    @ApiOperation(value = "版本管理新增/修改")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "新增", response = EditResult.class)})
    public Result save(@ApiParam(value = "版本信息", required = true) @Valid @RequestBody TBVersion tbVersion, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        try {
            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
            if (Objects.isNull(tbVersion.getId())) {
                tbVersion.insertInfo(sysUser.getId());
            } else {
                TBVersion tbVersionDb = tbVersionService.getById(tbVersion.getId());
                Objects.requireNonNull(tbVersionDb, "未找到版本信息");
                tbVersion.updateInfo(sysUser.getId());
            }
            tbVersionService.saveOrUpdate(tbVersion);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "版本管理编辑")
    @RequestMapping(value = "/edit", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "编辑", response = TBVersion.class)})
    public Result edit(@ApiParam(value = "主键", required = true) @RequestParam Long id) {
        TBVersion tbVersion = tbVersionService.getById(id);
        Objects.requireNonNull(tbVersion, "未找到版本信息");
        return ResultUtil.ok(tbVersionService.getById(id));
    }

    @ApiOperation(value = "版本管理删除")
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "删除", response = EditResult.class)})
    public Result delete(@ApiParam(value = "主键", required = true) @RequestParam Long id) {
        TBVersion tbVersion = tbVersionService.getById(id);
        Objects.requireNonNull(tbVersion, "未找到版本信息");
        return ResultUtil.ok(tbVersionService.removeById(id));
    }
}
