package com.qmth.distributed.print.api.mark;


import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.result.EditResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FieldUniqueEnum;
import com.qmth.teachcloud.common.util.RedisUtil;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.entity.MarkAiQuestionLevel;
import com.qmth.teachcloud.mark.entity.MarkAiQuestionParam;
import com.qmth.teachcloud.mark.entity.MarkAiQuestionPoint;
import com.qmth.teachcloud.mark.service.MarkAiQuestionLevelService;
import com.qmth.teachcloud.mark.service.MarkAiQuestionParamService;
import com.qmth.teachcloud.mark.service.MarkAiQuestionPointService;
import io.swagger.annotations.*;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * AI评卷参数表 前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2025-04-10
 */
@Api(tags = "AI评卷参数Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_MARK + "/ai_question_param")
public class MarkAiQuestionParamController {
    private final static Logger log = LoggerFactory.getLogger(MarkAiQuestionParamController.class);

    @Resource
    MarkAiQuestionParamService markAiQuestionParamService;

    @Resource
    MarkAiQuestionPointService markAiQuestionPointService;

    @Resource
    MarkAiQuestionLevelService markAiQuestionLevelService;

    @Resource
    RedisUtil redisUtil;

    @ApiOperation(value = "AI评卷参数新增/修改")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "成功/失败", response = EditResult.class)})
    @Transactional
    public Result save(@ApiParam(value = "AI评卷参数信息", required = true) @Valid @RequestBody MarkAiQuestionParam markAiQuestionParam, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        String lockKey = SystemConstant.REDIS_AI_QUESTION_PARAM_SAVE_PREFIX + markAiQuestionParam.getQuestionId();
        boolean lock = redisUtil.lock(lockKey, SystemConstant.REDIS_LOCK_DEFAULT_TIME_OUT);
        if (!lock) {
            throw ExceptionResultEnum.ERROR.exception("正在设置AI评卷参数，请稍候再试！");
        }
        try {
            switch (markAiQuestionParam.getMode()) {
                case POINT://按得分点
                    if (CollectionUtils.isEmpty(markAiQuestionParam.getPointList())) {
                        throw ExceptionResultEnum.ERROR.exception("得分点信息为空");
                    }
                    markAiQuestionParam.getPointList().stream().peek(s -> {
                        s.setAiQuestionId(markAiQuestionParam.getId());
                        s.setId(SystemConstant.getDbUuid());
                    }).collect(Collectors.toList());
                    if (Objects.isNull(markAiQuestionParam.getId())) {//新增
                        markAiQuestionParam.insertInfo(sysUser.getId());
                        markAiQuestionPointService.saveBatch(markAiQuestionParam.getPointList());
                    } else {//修改
                        List<MarkAiQuestionPoint> markAiQuestionPointDbList = markAiQuestionPointService.listByAiQuestionId(markAiQuestionParam.getId());
                        List<MarkAiQuestionLevel> markAiQuestionLevelDbList = markAiQuestionLevelService.listByAiQuestionId(markAiQuestionParam.getId());
                        //删除档次的数据
                        if (CollectionUtils.isNotEmpty(markAiQuestionLevelDbList)) {
                            markAiQuestionLevelService.removeByIds(markAiQuestionLevelDbList.stream().map(MarkAiQuestionLevel::getId).collect(Collectors.toList()));
                        }
                        //全删point数据全增
                        if (CollectionUtils.isNotEmpty(markAiQuestionPointDbList)) {
                            markAiQuestionPointService.removeByIds(markAiQuestionPointDbList.stream().map(MarkAiQuestionPoint::getId).collect(Collectors.toList()));
                        }
                        markAiQuestionPointService.saveBatch(markAiQuestionParam.getPointList());
                        markAiQuestionParam.updateInfo(sysUser.getId());
                    }
                    break;
                case LEVEL://按档次
                    if (CollectionUtils.isEmpty(markAiQuestionParam.getLevelList())) {
                        throw ExceptionResultEnum.ERROR.exception("档次信息为空");
                    }
                    markAiQuestionParam.getLevelList().stream().peek(s -> {
                        s.setAiQuestionId(markAiQuestionParam.getId());
                        s.setId(SystemConstant.getDbUuid());
                    }).collect(Collectors.toList());
                    if (Objects.isNull(markAiQuestionParam.getId())) {//新增
                        markAiQuestionParam.insertInfo(sysUser.getId());
                        markAiQuestionLevelService.saveBatch(markAiQuestionParam.getLevelList());
                    } else {
                        List<MarkAiQuestionPoint> markAiQuestionPointDbList = markAiQuestionPointService.listByAiQuestionId(markAiQuestionParam.getId());
                        List<MarkAiQuestionLevel> markAiQuestionLevelDbList = markAiQuestionLevelService.listByAiQuestionId(markAiQuestionParam.getId());
                        //删除得分点的数据
                        if (CollectionUtils.isNotEmpty(markAiQuestionPointDbList)) {
                            markAiQuestionPointService.removeByIds(markAiQuestionPointDbList.stream().map(MarkAiQuestionPoint::getId).collect(Collectors.toList()));
                        }
                        //全删level数据全增
                        if (CollectionUtils.isNotEmpty(markAiQuestionLevelDbList)) {
                            markAiQuestionLevelService.removeByIds(markAiQuestionLevelDbList.stream().map(MarkAiQuestionLevel::getId).collect(Collectors.toList()));
                        }
                        markAiQuestionLevelService.saveBatch(markAiQuestionParam.getLevelList());
                        markAiQuestionParam.updateInfo(sysUser.getId());
                    }
                    break;
                default:
                    break;
            }
            markAiQuestionParamService.saveOrUpdate(markAiQuestionParam);
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        } finally {
            redisUtil.releaseLock(lockKey);
        }
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "AI评卷参数查询")
    @RequestMapping(value = "/info", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "AI评卷参数信息", response = MarkAiQuestionParam.class)})
    public Result info(@ApiParam(value = "题目id", required = true) @RequestParam Long questionId) {
        MarkAiQuestionParam markAiQuestionParam = markAiQuestionParamService.getByExamIdAndPaperNumberAndQuestionId(null, null, questionId);
        Objects.requireNonNull(markAiQuestionParam, "AI评卷参数信息为空");
        List<MarkAiQuestionPoint> markAiQuestionPointDbList = markAiQuestionPointService.listByAiQuestionId(markAiQuestionParam.getId());
        List<MarkAiQuestionLevel> markAiQuestionLevelDbList = markAiQuestionLevelService.listByAiQuestionId(markAiQuestionParam.getId());
        markAiQuestionParam.setListInfo(markAiQuestionPointDbList, markAiQuestionLevelDbList);
        return ResultUtil.ok(markAiQuestionParam);
    }
}
