package com.qmth.distributed.print.api.mark;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.teachcloud.common.annotation.OperationLogDetail;
import com.qmth.teachcloud.common.bean.params.mark.group.QuestionPictureConfigParams;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.log.OperationTypeEnum;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.mark.bean.mark.DoubleMarkParam;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkClassDetailDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkClassProgressDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkGroupQuestionsDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkGroupSummaryProgressDto;
import com.qmth.teachcloud.mark.service.MarkQuestionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.util.List;

/**
 * <p>
 * 评卷分组表 前端控制器
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Api(tags = "评卷-主观题管理")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_MARK + "/question/subjective")
public class MarkQuestionSubjectiveController extends BaseController {

    @Resource
    private MarkQuestionService markQuestionService;

    /**
     * 主观题评卷设置-各步骤状态
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/step/status", method = RequestMethod.POST)
    public Result stepStatus(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                             @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) {
        return ResultUtil.ok(markQuestionService.stepStatus(examId, paperNumber));
    }

    /**
     * 主观题评卷设置-查询
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Result setList(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                          @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) {
        return ResultUtil.ok(markQuestionService.listGroupTaskByExamIdAndPaperNumber(examId, paperNumber));
    }

    /**
     * 主观题评卷设置-评卷区设置
     */
    @ApiOperation(value = "评卷区设置")
    @RequestMapping(value = "/picture/config/update", method = RequestMethod.POST)
    @OperationLogDetail(operationType = OperationTypeEnum.UPDATE, detail = "设置评卷区操作，考试ID：{{examId}}、试卷编号：{{paperNumber}}、分组号：{{groupNumber}}")
    public Result updatePictureConfig(@RequestBody QuestionPictureConfigParams questionPictureConfigParams) {
        validMarkPaperForMark(questionPictureConfigParams.getExamId(), questionPictureConfigParams.getPaperNumber());
        markQuestionService.updatePicListByQuestionId(questionPictureConfigParams);
        return ResultUtil.ok(true);
    }

    /**
     * 主观题评卷设置-评卷方式设置
     */
    @ApiOperation(value = "评卷方式设置（单双评）")
    @RequestMapping(value = "/double/mark/update", method = RequestMethod.POST)
    @OperationLogDetail(operationType = OperationTypeEnum.UPDATE, detail = "设置评卷区操作，考试ID：{{examId}}、试卷编号：{{paperNumber}}、分组号：{{groupNumber}}")
    public Result saveDoubleMark(@RequestBody DoubleMarkParam doubleMarkParam) {
        validMarkPaperForMark(doubleMarkParam.getExamId(), doubleMarkParam.getPaperNumber());
        markQuestionService.updateDoubleMarkByQuestionId(doubleMarkParam);
        return ResultUtil.ok(true);
    }


    /**
     * 进度列表查询
     */
    @ApiOperation(value = "评卷进度列表")
    @RequestMapping(value = "/summary", method = RequestMethod.POST)
    public Result summary(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                          @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) {
        MarkGroupSummaryProgressDto markGroupSummaryProgressDto = markQuestionService.summaryGroupProgress(examId, paperNumber);
        return ResultUtil.ok(markGroupSummaryProgressDto);
    }

    /**
     * 班级阅卷进度列表查询
     */
    @ApiOperation(value = "班级阅卷进度列表")
    @RequestMapping(value = "/class/summary", method = RequestMethod.POST)
    public Result classSummary(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                               @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber,
                               @ApiParam(value = "班级") @RequestParam(required = false) String className,
                               @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                               @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        IPage<MarkClassProgressDto> markGroupClassProgressDtoIPage = markQuestionService.summaryGroupClassProgress(examId, paperNumber, className, pageNumber, pageSize);
        return ResultUtil.ok(markGroupClassProgressDtoIPage);
    }

    /**
     * 班级阅卷进度列表查询
     */
    @ApiOperation(value = "班级阅卷进度列表")
    @RequestMapping(value = "/class/detail", method = RequestMethod.POST)
    public Result classSummary(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                               @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber,
                               @ApiParam(value = "班级") @RequestParam(required = false) String className) {
        List<MarkClassDetailDto> markGroupClassProgressDtoIPage = markQuestionService.detailGroupClassProgress(examId, paperNumber, className);
        return ResultUtil.ok(markGroupClassProgressDtoIPage);
    }

    /**
     * 评阅题目列表
     */
    @ApiOperation(value = "评阅题目列表")
    @RequestMapping(value = "/number/list", method = RequestMethod.POST)
    public Result listGroupQuestions(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                                     @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber) {
        List<MarkGroupQuestionsDto> markGroupQuestionsDtoList = markQuestionService.listGroupQuestions(examId, paperNumber);
        return ResultUtil.ok(markGroupQuestionsDtoList);
    }

}
