package com.qmth.distributed.print.api.mark;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkRejectHistoryDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkTaskDto;
import com.qmth.teachcloud.mark.entity.MarkTask;
import com.qmth.teachcloud.mark.enums.MarkTaskStatus;
import com.qmth.teachcloud.mark.service.MarkRejectHistoryService;
import com.qmth.teachcloud.mark.service.MarkTaskService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;

/**
 * <p>
 * 任务管理 前端控制器
 * </p>
 *
 * @author xf
 * @since 2024-08-22
 */
@Api(tags = "评卷-任务管理")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_MARK + "/task")
public class MarkTaskController extends BaseController {

    @Resource
    private MarkTaskService markTaskService;

    /**
     * 任务列表查询
     */
    @ApiOperation(value = "任务列表")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Result list(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                       @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber,
                       @ApiParam(value = "题目ID") @RequestParam(required = false) Long questionId,
                       @ApiParam(value = "评卷员") @RequestParam(required = false) String loginName,
                       @ApiParam(value = "状态") @RequestParam(required = false) MarkTaskStatus status,
                       @ApiParam(value = "学号") @RequestParam(required = false) String studentCode,
                       @ApiParam(value = "密号") @RequestParam(required = false) String secretNumber,
                       @ApiParam(value = "教学班") @RequestParam(required = false) String teachClassName,
                       @ApiParam(value = "小题得分") @RequestParam(required = false) Double subScore,
                       @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                       @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        IPage<MarkTaskDto> markTaskDtoIPage = markTaskService.pageMarkTask(examId, paperNumber, questionId, loginName, status, studentCode, secretNumber, teachClassName, subScore, pageNumber, pageSize);
        return ResultUtil.ok(markTaskDtoIPage);
    }

    /**
     * 打回
     */
    @ApiOperation(value = "评卷轨迹-打回")
    @RequestMapping(value = "/get_mark_track", method = RequestMethod.POST)
    public Result getMarkTrack(@ApiParam(value = "任务ID", required = true) @RequestParam Long id) {
        return ResultUtil.ok(markTaskService.getMarkTrack(id));
    }

    /**
     * 打回
     */
    @ApiOperation(value = "打回")
    @RequestMapping(value = "/reject", method = RequestMethod.POST)
    public Result reject(@ApiParam(value = "任务ID", required = true) @RequestParam Long id,
                         @ApiParam(value = "打回原因", required = true) @RequestParam String rejectReason) {
        MarkTask markTask = markTaskService.getById(id);
        validMarkPaperForMark(markTask.getExamId(), markTask.getPaperNumber());
        markTaskService.rejectMarkTask(id, rejectReason);
        return ResultUtil.ok(true);
    }

}
