package com.qmth.distributed.print.api.mark;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.teachcloud.common.annotation.OperationLogDetail;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.log.OperationTypeEnum;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.mark.bean.marker.MarkerAddParam;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkUserGroupProgressDto;
import com.qmth.teachcloud.mark.entity.MarkUserQuestion;
import com.qmth.teachcloud.mark.service.MarkUserQuestionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.util.List;

/**
 * <p>
 * 评卷员 前端控制器
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Api(tags = "评卷-评卷员")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_MARK + "/marker")
public class MarkUserQuestionController extends BaseController {

    @Resource
    private MarkUserQuestionService markUserQuestionService;

    /**
     * 评卷员管理列表查询
     */
    @ApiOperation(value = "评卷员管理列表")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Result classSummary(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                               @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber,
                               @ApiParam(value = "题目") @RequestParam(required = false) Long questionId,
                               @ApiParam(value = "评卷员") @RequestParam(required = false) String loginName,
                               @ApiParam(value = "班级") @RequestParam(required = false) String className,
                               @ApiParam(value = "是否已评卷") @RequestParam(defaultValue = "false") Boolean marked,
                               @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                               @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        IPage<MarkUserGroupProgressDto> markGroupClassProgressDtoIPage = markUserQuestionService.pageGroupUserByExamIdAndPaperNumberAndQuestionId(examId, paperNumber, questionId, loginName, className, marked, pageNumber, pageSize);
        return ResultUtil.ok(markGroupClassProgressDtoIPage);
    }

    /**
     * 评卷员管理列表查询
     */
    @ApiOperation(value = "评卷员管理列表查询")
    @RequestMapping(value = "/list_bind_marker", method = RequestMethod.POST)
    public Result listBindMarker(@ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                                 @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber,
                                 @ApiParam(value = "题目ID", required = true) @RequestParam Long questionId) {
        return ResultUtil.ok(markUserQuestionService.listBindMarkers(examId, paperNumber, questionId));
    }

    /**
     * 绑定评卷员
     */
    @ApiOperation(value = "绑定评卷员")
    @RequestMapping(value = "/add", method = RequestMethod.POST)
    @OperationLogDetail(operationType = OperationTypeEnum.UPDATE, detail = "绑定评卷员操作，考试ID：{{examId}}、试卷编号：{{paperNumber}}、分组号：{{groupNumber}}、评卷员ID集合：{{userIds}}")
    public Result addMarker(@RequestBody MarkerAddParam markerAddParam) {
        validMarkPaperForMark(markerAddParam.getExamId(), markerAddParam.getPaperNumber());
        markUserQuestionService.addMarkUserQuestion(markerAddParam);
        return ResultUtil.ok(true);
    }

    /**
     * 重置评卷员
     */
    @ApiOperation(value = "重置评卷员")
    @RequestMapping(value = "/reset", method = RequestMethod.POST)
    @OperationLogDetail(operationType = OperationTypeEnum.UPDATE, detail = "重置评卷员操作，用户分组ID：{{markUserQuestionId}}")
    public Result resetMarker(@ApiParam(value = "用户分组ID", required = true) @RequestParam Long markUserQuestionId) {
        MarkUserQuestion markUserQuestion = markUserQuestionService.getById(markUserQuestionId);
        if (markUserQuestion == null) {
            throw ExceptionResultEnum.ERROR.exception("评卷员不存在");
        }
        validMarkPaperForMark(markUserQuestion.getExamId(), markUserQuestion.getPaperNumber());
        return ResultUtil.ok(markUserQuestionService.resetMarker(markUserQuestion));
    }

    /**
     * 解绑评卷员
     */
    @ApiOperation(value = "解绑评卷员")
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    @OperationLogDetail(operationType = OperationTypeEnum.UPDATE, detail = "解绑评卷员操作，用户分组ID：{{markUserQuestionId}}")
    public Result deleteMarker(@ApiParam(value = "用户分组ID", required = true) @RequestParam Long markUserQuestionId) {
        MarkUserQuestion markUserQuestion = markUserQuestionService.getById(markUserQuestionId);
        if (markUserQuestion == null) {
            throw ExceptionResultEnum.ERROR.exception("评卷员不存在");
        }
        validMarkPaperForMark(markUserQuestion.getExamId(), markUserQuestion.getPaperNumber());
        markUserQuestionService.deleteMarker(markUserQuestion);
        return ResultUtil.ok(true);
    }

    /**
     * 回收评卷员正在评卷的试卷
     */
    @ApiOperation(value = "回收正在评卷的试卷")
    @RequestMapping(value = "/release", method = RequestMethod.POST)
    @OperationLogDetail(operationType = OperationTypeEnum.UPDATE, detail = "回收正在评卷的试卷操作，用户分组ID集合：{{markUserQuestionIds}}")
    public Result releaseMarker(@ApiParam(value = "用户分组ID集合", required = true) @RequestParam List<Long> markUserQuestionIds) {
        MarkUserQuestion markUserGroup = markUserQuestionService.getById(markUserQuestionIds.get(0));
        if (markUserGroup == null) {
            throw ExceptionResultEnum.ERROR.exception("评卷员不存在");
        }
        validMarkPaperForMark(markUserGroup.getExamId(), markUserGroup.getPaperNumber());
        markUserQuestionService.releaseMarker(markUserQuestionIds);
        return ResultUtil.ok(true);
    }

    /**
     * 设置评卷任务
     */
    @ApiOperation(value = "设置评卷任务")
    @RequestMapping(value = "/setTaskCount", method = RequestMethod.POST)
    @OperationLogDetail(operationType = OperationTypeEnum.UPDATE, detail = "设置评卷任务操作，用户分组ID集合：{{markUserQuestionIds}}、评卷任务数：{{topCount}}")
    public Result setTaskCount(@ApiParam(value = "用户分组ID集合", required = true) @RequestParam List<Long> markUserQuestionIds,
                               @ApiParam(value = "评卷任务数", required = true) @RequestParam Integer topCount) {
        MarkUserQuestion markUserQuestion = markUserQuestionService.getById(markUserQuestionIds.get(0));
        if (markUserQuestion == null) {
            throw ExceptionResultEnum.ERROR.exception("评卷员不存在");
        }
        validMarkPaperForMark(markUserQuestion.getExamId(), markUserQuestion.getPaperNumber());
        markUserQuestionService.setTopCount(markUserQuestionIds, topCount);
        return ResultUtil.ok(true);
    }

}
