package com.qmth.teachcloud.common.entity;

import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.teachcloud.common.base.BaseEntity;
import com.qmth.teachcloud.common.bean.dto.mark.MarkUser;
import com.qmth.teachcloud.common.bean.dto.mark.PictureConfig;
import com.qmth.teachcloud.common.bean.marking.MarkConfigItem;
import com.qmth.teachcloud.common.enums.ObjectivePolicy;
import com.qmth.teachcloud.common.enums.ScorePolicy;
import com.qmth.teachcloud.common.enums.mark.MarkPaperAiMark;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import java.io.Serializable;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 小题信息表
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@TableName("mark_question")
@ApiModel(value="MarkQuestion对象", description="小题信息表")
public class MarkQuestion extends BaseEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "考试ID")
    private Long examId;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "课程ID")
    private Long courseId;

    @ApiModelProperty(value = "试卷类型")
    private String paperNumber;
    @ApiModelProperty(value = "试卷卷型")
    private String paperType;

    @ApiModelProperty(value = "是否客观题")
    private Boolean objective;

    @ApiModelProperty(value = "分组号")
    private Integer groupNumber;

    @ApiModelProperty(value = "大题号")
    private Integer mainNumber;

    @ApiModelProperty(value = "小题号")
    private Integer subNumber;

    @ApiModelProperty(value = "大题名称")
    private String mainTitle;

    @ApiModelProperty(value = "客观题答案数量")
    private Integer optionCount;

    @ApiModelProperty(value = "满分")
    private Double totalScore;

    @ApiModelProperty(value = "评卷间隔分")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Double intervalScore;


    @ApiModelProperty(value = "正确答案")
    @TableField(exist = false)
    private String answer;

    @ApiModelProperty(value = "客观题判分策略")
    @TableField(exist = false)
    private ObjectivePolicy objectivePolicy;

    @ApiModelProperty(value = "客观题判分策略计分")
    @TableField(exist = false)
    private Double objectivePolicyScore;

    @ApiModelProperty(value = "题型")
    private Integer questionType;
    
    private Integer paperIndex;
    
    private Integer pageIndex;

    private String name;

    @ApiModelProperty(value = "小图配置")
    private String picList;

    @ApiModelProperty(value = "双评比例")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Double doubleRate;

    @ApiModelProperty(value = "仲裁阈值")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Double arbitrateThreshold;

    @ApiModelProperty(value = "合分策略")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private ScorePolicy scorePolicy;

    @ApiModelProperty(value = "任务总量")
    private Integer taskCount;

    @ApiModelProperty(value = "已评数量")
    private Integer markedCount;

    @ApiModelProperty(value = "剩余数量")
    private Integer leftCount;

    @ApiModelProperty(value = "AI评卷")
    private MarkPaperAiMark aiMark;

    @ApiModelProperty(value = "是否有人评任务")
    private Boolean personTask;

    @TableField(exist = false)
    private double markScore;

    @TableField(exist = false)
    private List<MarkUser> markers;

    @TableField(exist = false)
    private List<PictureConfig> pictureConfigs;

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public Long getCourseId() {
        return courseId;
    }

    public void setCourseId(Long courseId) {
        this.courseId = courseId;
    }

    public String getPaperNumber() {
        return paperNumber;
    }

    public void setPaperNumber(String paperNumber) {
        this.paperNumber = paperNumber;
    }

    public String getPaperType() {
        return paperType;
    }

    public void setPaperType(String paperType) {
        this.paperType = paperType;
    }

    public Boolean getObjective() {
        return objective;
    }

    public void setObjective(Boolean objective) {
        this.objective = objective;
    }

    public Integer getGroupNumber() {
        return groupNumber;
    }

    public void setGroupNumber(Integer groupNumber) {
        this.groupNumber = groupNumber;
    }

    public Integer getMainNumber() {
        return mainNumber;
    }

    public void setMainNumber(Integer mainNumber) {
        this.mainNumber = mainNumber;
    }
    public Integer getSubNumber() {
        return subNumber;
    }

    public void setSubNumber(Integer subNumber) {
        this.subNumber = subNumber;
    }
    public String getMainTitle() {
        return mainTitle;
    }

    public void setMainTitle(String mainTitle) {
        this.mainTitle = mainTitle;
    }
    public String getAnswer() {
        return answer;
    }

    public void setAnswer(String answer) {
        this.answer = answer;
    }

    public Integer getOptionCount() {
        return optionCount;
    }

    public void setOptionCount(Integer optionCount) {
        this.optionCount = optionCount;
    }

    public Double getTotalScore() {
        return totalScore;
    }

    public void setTotalScore(Double totalScore) {
        this.totalScore = totalScore;
    }
    public Double getIntervalScore() {
        return intervalScore;
    }

    public void setIntervalScore(Double intervalScore) {
        this.intervalScore = intervalScore;
    }
    public ObjectivePolicy getObjectivePolicy() {
        return objectivePolicy;
    }

    public void setObjectivePolicy(ObjectivePolicy objectivePolicy) {
        this.objectivePolicy = objectivePolicy;
    }
    public Integer getQuestionType() {
        return questionType;
    }

    public void setQuestionType(Integer questionType) {
        this.questionType = questionType;
    }
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getQuestionNumber() {
        return getMainNumber() + "-" + getSubNumber();
    }
    public Integer getPaperIndex() {
		return paperIndex;
	}

	public void setPaperIndex(Integer paperIndex) {
		this.paperIndex = paperIndex;
	}

	public Integer getPageIndex() {
		return pageIndex;
	}

	public void setPageIndex(Integer pageIndex) {
		this.pageIndex = pageIndex;
	}

    public String getPicList() {
        return picList;
    }

    public void setPicList(String picList) {
        this.picList = picList;
    }

    public Double getDoubleRate() {
        return doubleRate;
    }

    public void setDoubleRate(Double doubleRate) {
        this.doubleRate = doubleRate;
    }

    public Double getArbitrateThreshold() {
        return arbitrateThreshold;
    }

    public void setArbitrateThreshold(Double arbitrateThreshold) {
        this.arbitrateThreshold = arbitrateThreshold;
    }

    public ScorePolicy getScorePolicy() {
        return scorePolicy;
    }

    public void setScorePolicy(ScorePolicy scorePolicy) {
        this.scorePolicy = scorePolicy;
    }

    public Integer getTaskCount() {
        return taskCount;
    }

    public void setTaskCount(Integer taskCount) {
        this.taskCount = taskCount;
    }

    public Integer getMarkedCount() {
        return markedCount;
    }

    public void setMarkedCount(Integer markedCount) {
        this.markedCount = markedCount;
    }

    public Integer getLeftCount() {
        return leftCount;
    }

    public void setLeftCount(Integer leftCount) {
        this.leftCount = leftCount;
    }

    public MarkPaperAiMark getAiMark() {
        return aiMark;
    }

    public void setAiMark(MarkPaperAiMark aiMark) {
        this.aiMark = aiMark;
    }

    public Boolean getPersonTask() {
        return personTask;
    }

    public void setPersonTask(Boolean personTask) {
        this.personTask = personTask;
    }

    public double getMarkScore() {
        return markScore;
    }

    public void setMarkScore(double markScore) {
        this.markScore = markScore;
    }


    public List<MarkUser> getMarkers() {
        return markers;
    }

    public void setMarkers(List<MarkUser> markers) {
        this.markers = markers;
    }

    public List<PictureConfig> getPictureConfigs() {
        return pictureConfigs;
    }

    public void setPictureConfigs(List<PictureConfig> pictureConfigs) {
        this.pictureConfigs = pictureConfigs;
    }

    public List<MarkConfigItem> getPictureConfigList() {
        return MarkConfigItem.parse(this.picList);
    }
    @Override
    public String toString() {
        return "MarkQuestion{" +
                "examId=" + examId +
                ", paperNumber='" + paperNumber + '\'' +
                ", paperType='" + paperType + '\'' +
                ", objective=" + objective +
                ", mainNumber=" + mainNumber +
                ", subNumber=" + subNumber +
                ", mainTitle='" + mainTitle + '\'' +
                ", optionCount=" + optionCount +
                ", totalScore=" + totalScore +
                ", intervalScore=" + intervalScore +
                ", answer='" + answer + '\'' +
                ", objectivePolicy=" + objectivePolicy +
                ", objectivePolicyScore=" + objectivePolicyScore +
                ", questionType=" + questionType +
                ", paperIndex=" + paperIndex +
                ", pageIndex=" + pageIndex +
                ", name='" + name + '\'' +
                ", picList='" + picList + '\'' +
                ", doubleRate=" + doubleRate +
                ", arbitrateThreshold=" + arbitrateThreshold +
                ", scorePolicy=" + scorePolicy +
                ", taskCount=" + taskCount +
                ", markedCount=" + markedCount +
                ", leftCount=" + leftCount +
                ", markScore=" + markScore +
                ", markers=" + markers +
                '}';
    }

    public Double getObjectivePolicyScore() {
        return objectivePolicyScore;
    }

    public void setObjectivePolicyScore(Double objectivePolicyScore) {
        this.objectivePolicyScore = objectivePolicyScore;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        MarkQuestion that = (MarkQuestion) o;
        return mainNumber.equals(that.mainNumber) && subNumber.equals(that.subNumber) && questionType.equals(that.questionType);
    }

    @Override
    public int hashCode() {
        return Objects.hash(mainNumber, subNumber, questionType);
    }
}
