package com.qmth.teachcloud.common.enums;

import com.qmth.teachcloud.common.contant.SystemConstant;

/**
 * @Description: pdf格式 enum
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/4/7
 */
public enum PageSizeEnum {
    A3(842.0F, 1191.0F, 297, 420),
    A4(595.0F, 842.0F, 210, 297),
    A5(420.0F, 595.0F, 148, 210),
    B3(1000.0F, 1417.0F, 364, 514),
    B4(708.0F, 1000.0F, 257, 364),
    B5(498.0F, 708.0F, 182, 257),
    K8(737.0F, 1043.0F, 270, 390);

    private float width;

    private float height;

    // 单位mm
    private float pageWidth;
    // 单位mm
    private float pageHeight;

    private PageSizeEnum(float width, float height, float pageWidth, float pageHeight) {
        this.width = width;
        this.height = height;
        this.pageWidth = pageWidth;
        this.pageHeight = pageHeight;
    }

    public float getWidth() {
        return width;
    }

    public float getHeight() {
        return height;
    }

    public float getPageWidth() {
        return pageWidth;
    }

    public float getPageHeight() {
        return pageHeight;
    }

    /**
     * 状态转换 toName
     *
     * @param width
     * @param height
     * @return
     */
    public static String convertToName(float width, float height) {
        float deviation = SystemConstant.PAPER_DEVIATION;
        for (PageSizeEnum e : PageSizeEnum.values()) {
            float standardWidth = e.getWidth();
            float standardHeight = e.getHeight();
            if (standardWidth - deviation <= width && width <= standardWidth + deviation && standardHeight - deviation <= height && height <= standardHeight + deviation) {
                return e.name();
            }
        }
        return null;
    }

    /**
     * 状态转换 toEnum
     *
     * @param width
     * @param height
     * @return
     */
    public static PageSizeEnum convertToEnum(float width, float height) {
        for (PageSizeEnum e : PageSizeEnum.values()) {
            float deviation = SystemConstant.PAPER_DEVIATION;
            if (PageSizeEnum.A3.equals(e)) {
                deviation = 2 * SystemConstant.PAPER_DEVIATION;
            }
            float standardWidth = e.getWidth();
            float standardHeight = e.getHeight();
            if (standardWidth - deviation <= width && width <= standardWidth + deviation && standardHeight - deviation <= height && height <= standardHeight + deviation) {
                return e;
            }
            if (standardWidth - deviation <= height && height <= standardWidth + deviation && standardHeight - deviation <= width && width <= standardHeight + deviation) {
//                throw ExceptionResultEnum.ERROR.exception("请检查上传的【"+ e.name() + "】纸张字体方向是否反了");
                return e;
            }
        }
        throw ExceptionResultEnum.ERROR.exception("pdf格式不符合标准规范，请提供标准的pdf（可使用wps、office的word转换）");
    }
}
