package com.qmth.teachcloud.data;

import com.qmth.teachcloud.common.bean.result.SysConfigResult;
import com.qmth.teachcloud.common.bean.sync.*;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysConfig;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.CommonCacheService;
import com.qmth.teachcloud.common.bean.sync.TSyncDataStudent;
import com.qmth.teachcloud.data.enums.DataType;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.datasource.DriverManagerDataSource;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.sql.DataSource;
import java.util.*;

@Component
public class DataUtil {
    private static final Logger log = LoggerFactory.getLogger(DataUtil.class);

    // 一页1000条
    private static int PAGE_SIZE = 1000;

    @Resource
    private CommonCacheService commonCacheService;

    public List<OrgDataVo> listOrg(JdbcTemplate jdbcTemplate, DataType dataType, String datasourceType) {

        String sql;
        if ("ORACLE".equals(datasourceType.trim())) {
            sql = getOracleSql(dataType, false);
        } else if ("MYSQL".equals(datasourceType.trim())) {
            sql = getMysqlSql(dataType, false);
        } else {
            throw ExceptionResultEnum.ERROR.exception("只支持Oracle和Mysql数据库");
        }
        return query(jdbcTemplate, sql, OrgDataVo.class);
    }

    public List<UserDataVo> listUser(JdbcTemplate jdbcTemplate, DataType dataType, String datasourceType) {
        String sql;
        if ("ORACLE".equals(datasourceType.trim())) {
            sql = getOracleSql(dataType, true, "code");
            return pageOracle(jdbcTemplate, sql, UserDataVo.class);
        } else if ("MYSQL".equals(datasourceType.trim())) {
            sql = getMysqlSql(dataType, true);
            return pageMysql(jdbcTemplate, sql, UserDataVo.class);
        } else {
            throw ExceptionResultEnum.ERROR.exception("只支持Oracle和Mysql数据库");
        }
    }

    public List<CourseDataVo> listCourse(JdbcTemplate jdbcTemplate, DataType dataType, String datasourceType) {
        String sql;
        if ("ORACLE".equals(datasourceType.trim())) {
            sql = getOracleSql(dataType, false);
        } else if ("MYSQL".equals(datasourceType.trim())) {
            sql = getMysqlSql(dataType, false);
        } else {
            throw ExceptionResultEnum.ERROR.exception("只支持Oracle和Mysql数据库");
        }
        return query(jdbcTemplate, sql, CourseDataVo.class);
    }

    public List<ExamTaskDataVo> listExamTask(JdbcTemplate jdbcTemplate, DataType dataType, String datasourceType) {
        String sql;
        if ("ORACLE".equals(datasourceType.trim())) {
            sql = getOracleSql(dataType, false);
        } else if ("MYSQL".equals(datasourceType.trim())) {
            sql = getMysqlSql(dataType, false);
        } else {
            throw ExceptionResultEnum.ERROR.exception("只支持Oracle和Mysql数据库");
        }
        return query(jdbcTemplate, sql, ExamTaskDataVo.class);
    }

    public List<TSyncDataStudent> listExamStudent(JdbcTemplate jdbcTemplate, DataType dataType, String datasourceType) {
        String sql;
        if ("ORACLE".equals(datasourceType.trim())) {
            sql = getOracleSql(dataType, true, "coursecode", "studentcode");
            return pageOracle(jdbcTemplate, sql, TSyncDataStudent.class);
        } else if ("MYSQL".equals(datasourceType.trim())) {
            sql = getMysqlSql(dataType, true);
            return pageMysql(jdbcTemplate, sql, TSyncDataStudent.class);
        } else {
            throw ExceptionResultEnum.ERROR.exception("只支持Oracle和Mysql数据库");
        }
    }

    /**
     * 不分页
     *
     * @param sql
     */
    private <T> List<T> query(JdbcTemplate jdbcTemplate, String sql, Class<T> tClass) {
        List<T> list;
        try {
            list = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(tClass));
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("查询失败：" + e.getMessage());
        }
        return list;
    }

    /**
     * 分页
     *
     * @param sql
     */
    private <T> List<T> pageMysql(JdbcTemplate jdbcTemplate, String sql, Class<T> tClass) {
        //将查询的语句封装到List集合中，集合中存储每一个员工实体
        List<T> listAll = new ArrayList<>();
        try {
            List<T> list;
            int pageNumber = 0;
            do {
                int offset = pageNumber * PAGE_SIZE + PAGE_SIZE;
                list = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(tClass), pageNumber, offset);
                if (CollectionUtils.isNotEmpty(list)) {
                    listAll.addAll(list);
                    pageNumber++;
                }
            } while (CollectionUtils.isNotEmpty(list));
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("查询失败：" + e.getMessage());
        }
        return listAll;
    }

    /**
     * 分页
     *
     * @param sql
     */
    private <T> List<T> pageOracle(JdbcTemplate jdbcTemplate, String sql, Class<T> tClass) {
        //将查询的语句封装到List集合中，集合中存储每一个员工实体
        List<T> listAll = new ArrayList<>();
        try {
            List<T> list;
            int pageNumber = 1;
            do {
                int startIndex = PAGE_SIZE * (pageNumber - 1) + 1;
                int endIndex = pageNumber * PAGE_SIZE;
                list = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(tClass), startIndex, endIndex);
                if (CollectionUtils.isNotEmpty(list)) {
                    listAll.addAll(list);
                    pageNumber++;
                }
            } while (CollectionUtils.isNotEmpty(list));
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("查询失败：" + e.getMessage());
        }
        return listAll;
    }

    private String getMysqlSql(DataType dataType, boolean isPage) {
        //sql语句
        String sql;
        if (isPage) {
            sql = "select * from " + dataType.getTable() + " limit ? ?";
        } else {
            sql = "select * from " + dataType.getTable();
        }
        return sql;
    }

    private String getOracleSql(DataType dataType, boolean isPage, String... sortColumn) {
        //sql语句
        String sql;
        if (isPage) {
            StringJoiner stringJoiner = new StringJoiner("");
            stringJoiner.add("SELECT *  FROM (SELECT t.*, ROW_NUMBER() OVER (ORDER BY ");
            StringJoiner sortStr = new StringJoiner(",");
            for (String s : sortColumn) {
                sortStr.add(" " + s + " ");
            }
            stringJoiner.add(sortStr.toString());
            stringJoiner.add(") AS rn FROM ");
            stringJoiner.add("\"" + dataType.getTable() + "\"");
            stringJoiner.add(" t)  WHERE rn BETWEEN ? AND ?");
            sql = stringJoiner.toString();
        } else {
            sql = "select * from \"" + dataType.getTable() + "\"";
        }
        log.info(dataType.getName() + "数据同步sql:" + sql);
        return sql;
    }

    public DataSource createDataSourceMap(Long schoolId) {
        SysConfig datasourceType = commonCacheService.addSysConfigCache(schoolId, SystemConstant.DATA_DATASOURCE_TYPE);
        if (datasourceType == null || StringUtils.isBlank(datasourceType.getConfigValue())) {
            throw ExceptionResultEnum.ERROR.exception("未设置数据库类型");
        }
        // 数据库HOST
        SysConfig datasourceHost = commonCacheService.addSysConfigCache(schoolId, SystemConstant.DATA_DATASOURCE_HOST);
        if (datasourceHost == null || StringUtils.isBlank(datasourceHost.getConfigValue())) {
            throw ExceptionResultEnum.ERROR.exception("未设置HOST");
        }
        // 数据库PORT
        SysConfig datasourcePort = commonCacheService.addSysConfigCache(schoolId, SystemConstant.DATA_DATASOURCE_PORT);
        if (datasourcePort == null || StringUtils.isBlank(datasourcePort.getConfigValue())) {
            throw ExceptionResultEnum.ERROR.exception("未设置端口");
        }
        // 数据库DataName
        SysConfig datasourceDataName = commonCacheService.addSysConfigCache(schoolId, SystemConstant.DATA_DATASOURCE_DATANAME);
        if (datasourceDataName == null || StringUtils.isBlank(datasourceDataName.getConfigValue())) {
            throw ExceptionResultEnum.ERROR.exception("未设置数据库名");
        }
        // 数据库用户名
        SysConfig datasourceUserName = commonCacheService.addSysConfigCache(schoolId, SystemConstant.DATA_DATASOURCE_USERNAME);
        if (datasourceUserName == null || StringUtils.isBlank(datasourceUserName.getConfigValue())) {
            throw ExceptionResultEnum.ERROR.exception("未设置用户名");
        }
        String userName = datasourceUserName.getConfigValue().trim();
        // 数据库密码
        SysConfig datasourcePassword = commonCacheService.addSysConfigCache(schoolId, SystemConstant.DATA_DATASOURCE_PASSWORD);
        if (datasourcePassword == null || StringUtils.isBlank(datasourcePassword.getConfigValue())) {
            throw ExceptionResultEnum.ERROR.exception("未设置密码");
        }
        String password = datasourcePassword.getConfigValue().trim();
        Map<String, String> map = new HashMap<>();

        String driverClassName;
        String url;
        if ("ORACLE".equals(datasourceType.getConfigValue())) {
            driverClassName = "com.mysql.jdbc.Driver";
            url = "jdbc:oracle:thin:@" + datasourceHost.getConfigValue().trim() + ":" + datasourcePort.getConfigValue().trim() + ":" + datasourceDataName.getConfigValue().trim();
        } else if ("MYSQL".equals(datasourceType.getConfigValue())) {
            driverClassName = "com.mysql.jdbc.Driver";
            url = "jdbc:mysql://" + datasourceHost.getConfigValue().trim() + ":" + datasourcePort.getConfigValue().trim() + "/" + datasourceDataName.getConfigValue().trim() + "?useUnicode=true&characterEncoding=utf8&zeroDateTimeBehavior=convertToNull&useSSL=true&serverTimezone=GMT%2B8";
        } else {
            throw ExceptionResultEnum.ERROR.exception("只支持Oracle和Mysql数据库");
        }
        try {
            Class.forName(driverClassName);
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
        return new DriverManagerDataSource(url, userName, password);
    }


    public DataSource createDataSourceMap(List<SysConfigResult> params) {
        SysConfigResult datasourceType = params.stream().filter(m -> m.getCode().equals(SystemConstant.DATA_DATASOURCE_TYPE)).findFirst().orElse(null);
        if (datasourceType == null || StringUtils.isBlank(datasourceType.getValue().toString())) {
            throw ExceptionResultEnum.ERROR.exception("未设置数据库类型");
        }
        String type = datasourceType.getValue().toString().trim();
        // 数据库HOST
        SysConfigResult datasourceHost = params.stream().filter(m -> m.getCode().equals(SystemConstant.DATA_DATASOURCE_HOST)).findFirst().orElse(null);
        if (datasourceHost == null || StringUtils.isBlank(datasourceHost.getValue().toString())) {
            throw ExceptionResultEnum.ERROR.exception("未设置HOST");
        }
        String host = datasourceHost.getValue().toString().trim();
        // 数据库PORT
        SysConfigResult datasourcePort = params.stream().filter(m -> m.getCode().equals(SystemConstant.DATA_DATASOURCE_PORT)).findFirst().orElse(null);
        if (datasourcePort == null || StringUtils.isBlank(datasourcePort.getValue().toString())) {
            throw ExceptionResultEnum.ERROR.exception("未设置端口");
        }
        String port = datasourcePort.getValue().toString().trim();
        // 数据库DataName
        SysConfigResult datasourceDataName = params.stream().filter(m -> m.getCode().equals(SystemConstant.DATA_DATASOURCE_DATANAME)).findFirst().orElse(null);
        if (datasourceDataName == null || StringUtils.isBlank(datasourceDataName.getValue().toString())) {
            throw ExceptionResultEnum.ERROR.exception("未设置数据库名");
        }
        String dataName = datasourceDataName.getValue().toString().trim();
        // 数据库用户名
        SysConfigResult datasourceUserName = params.stream().filter(m -> m.getCode().equals(SystemConstant.DATA_DATASOURCE_USERNAME)).findFirst().orElse(null);
        if (datasourceUserName == null || StringUtils.isBlank(datasourceUserName.getValue().toString())) {
            throw ExceptionResultEnum.ERROR.exception("未设置用户名");
        }
        String userName = datasourceUserName.getValue().toString().trim();
        // 数据库密码
        SysConfigResult datasourcePassword = params.stream().filter(m -> m.getCode().equals(SystemConstant.DATA_DATASOURCE_PASSWORD)).findFirst().orElse(null);
        if (datasourcePassword == null || StringUtils.isBlank(datasourcePassword.getValue().toString())) {
            throw ExceptionResultEnum.ERROR.exception("未设置密码");
        }
        String password = datasourcePassword.getValue().toString().trim();
        Map<String, String> map = new HashMap<>();

        String driverClassName;
        String url;
        if ("ORACLE".equals(type)) {
            driverClassName = "oracle.jdbc.driver.OracleDriver";
            url = "jdbc:oracle:thin:@" + host + ":" + port + ":" + dataName;
        } else if ("MYSQL".equals(type)) {
            driverClassName = "com.mysql.jdbc.Driver";
            url = "jdbc:mysql://" + host + ":" + port + "/" + dataName + "?serverTimezone=Asia/Shanghai&useUnicode=true&characterEncoding=UTF-8&nullCatalogMeansCurrent=true";
        } else {
            throw ExceptionResultEnum.ERROR.exception("只支持Oracle和Mysql数据库");
        }
        try {
            Class.forName(driverClassName);
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
        return new DriverManagerDataSource(url, userName, password);
    }

}
