package com.qmth.teachcloud.data.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import java.io.Serializable;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.github.jeffreyning.mybatisplus.anno.MppMultiId;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

/**
 * <p>
 * 数据同步设置
 * </p>
 *
 * @author xf
 * @since 2024-12-05
 */
@TableName("t_sync_data")
@ApiModel(value="TSyncData对象", description="数据同步设置")
public class TSyncData implements Serializable {

    private static final long serialVersionUID = 1L;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "学校ID")
    @MppMultiId(value = "school_id")
    private Long schoolId;

    @ApiModelProperty(value = "学期ID")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long semesterId;

    @ApiModelProperty(value = "考试ID")
    @MppMultiId(value = "exam_id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long examId;

    @ApiModelProperty(value = "同步数据类型（A：机构同步，B：用户同步，C：课程同步，D：考生同步，E：命题任务同步）")
    private String dataType;

    @ApiModelProperty(value = "同步开始时间")
    private Long startTime;

    @ApiModelProperty(value = "同步结束时间")
    private Long endTime;

    @ApiModelProperty(value = "最后一次同步时间")
    private Long lastSyncTime;

    @ApiModelProperty(value = "状态（0：未同步/已完成，1：同步中）")
    private Boolean status;

    @ApiModelProperty(value = "cron表达式")
    private String cron;

    @ApiModelProperty(value = "手机号（多个用逗号分隔）")
    private String phoneNumber;
    @ApiModelProperty(value = "同步明细")
    private String detail;

    @ApiModelProperty(value = "创建时间")
    private Long createTime;

    @ApiModelProperty(value = "更新时间")
    private Long updateTime;

    @ApiModelProperty(value = "启用/禁用")
    private Boolean enable;

    @ApiModelProperty(value = "学期")
    @TableField(exist = false)
    private String semesterName;

    @ApiModelProperty(value = "考试")
    @TableField(exist = false)
    private String examName;

    public TSyncData() {
    }

    public TSyncData(Long schoolId, Long examId) {
        this.schoolId = schoolId;
        this.examId = examId;
    }

    public Long getSchoolId() {
        return schoolId;
    }

    public void setSchoolId(Long schoolId) {
        this.schoolId = schoolId;
    }
    public Long getSemesterId() {
        return semesterId;
    }

    public void setSemesterId(Long semesterId) {
        this.semesterId = semesterId;
    }
    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public String getDataType() {
        return dataType;
    }

    public void setDataType(String dataType) {
        this.dataType = dataType;
    }

    public Long getStartTime() {
        return startTime;
    }

    public void setStartTime(Long startTime) {
        this.startTime = startTime;
    }
    public Long getEndTime() {
        return endTime;
    }

    public void setEndTime(Long endTime) {
        this.endTime = endTime;
    }
    public Long getLastSyncTime() {
        return lastSyncTime;
    }

    public void setLastSyncTime(Long lastSyncTime) {
        this.lastSyncTime = lastSyncTime;
    }

    public Boolean getStatus() {
        return status;
    }

    public void setStatus(Boolean status) {
        this.status = status;
    }

    public String getCron() {
        return cron;
    }

    public void setCron(String cron) {
        this.cron = cron;
    }

    public String getPhoneNumber() {
        return phoneNumber;
    }

    public void setPhoneNumber(String phoneNumber) {
        this.phoneNumber = phoneNumber;
    }

    public String getDetail() {
        return detail;
    }

    public void setDetail(String detail) {
        this.detail = detail;
    }
    public Long getCreateTime() {
        return createTime;
    }

    public void setCreateTime(Long createTime) {
        this.createTime = createTime;
    }
    public Long getUpdateTime() {
        return updateTime;
    }

    public void setUpdateTime(Long updateTime) {
        this.updateTime = updateTime;
    }

    public Boolean getEnable() {
        return enable;
    }

    public void setEnable(Boolean enable) {
        this.enable = enable;
    }

    public String getSemesterName() {
        return semesterName;
    }

    public void setSemesterName(String semesterName) {
        this.semesterName = semesterName;
    }

    public String getExamName() {
        return examName;
    }

    public void setExamName(String examName) {
        this.examName = examName;
    }

    @Override
    public String toString() {
        return "TSyncData{" +
            "schoolId=" + schoolId +
            ", semesterId=" + semesterId +
            ", examId=" + examId +
            ", dataType=" + dataType +
            ", startTime=" + startTime +
            ", endTime=" + endTime +
            ", lastSyncTime=" + lastSyncTime +
            ", detail=" + detail +
            ", createTime=" + createTime +
            ", updateTime=" + updateTime +
        "}";
    }
}
