package com.qmth.teachcloud.data.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.jeffreyning.mybatisplus.service.MppServiceImpl;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.data.entity.TSyncData;
import com.qmth.teachcloud.data.entity.TSyncDataLog;
import com.qmth.teachcloud.data.mapper.TSyncDataMapper;
import com.qmth.teachcloud.data.service.DataService;
import com.qmth.teachcloud.data.service.TSyncDataLogService;
import com.qmth.teachcloud.data.service.TSyncDataService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

/**
 * <p>
 * 数据同步设置 服务实现类
 * </p>
 *
 * @author xf
 * @since 2024-12-05
 */
@Service
public class TSyncDataServiceImpl extends MppServiceImpl<TSyncDataMapper, TSyncData> implements TSyncDataService {

    @Resource
    private DataService dataService;
    @Resource
    private TSyncDataLogService tSyncDataLogService;
    @Resource
    private ConcurrentService concurrentService;

    @Override
    public TSyncData saveData(TSyncData tSyncData) {
        TSyncData syncData = this.selectByMultiId(new TSyncData(tSyncData.getSchoolId(), tSyncData.getExamId()));
        if (syncData == null) {
            tSyncData.setEnable(true);
            tSyncData.setCreateTime(System.currentTimeMillis());
            this.saveOrUpdateByMultiId(tSyncData);

            return tSyncData;
        } else {
            if (syncData.getStatus()) {
                throw ExceptionResultEnum.ERROR.exception("数据同步中，不能修改");
            }
            syncData.setDataType(tSyncData.getDataType());
            syncData.setStartTime(tSyncData.getStartTime());
            syncData.setEndTime(tSyncData.getEndTime());
            syncData.setCron(tSyncData.getCron());
            syncData.setPhoneNumber(tSyncData.getPhoneNumber());
            syncData.setUpdateTime(System.currentTimeMillis());
            this.updateByMultiId(syncData);

            return syncData;
        }
    }

    @Override
    public boolean syncData(Long schoolId, Long examId) {
        concurrentService.getReadWriteLock("TEACHCLOUD_SYNC_DATA" + "-" + schoolId + "-" + examId).writeLock().lock();
        TSyncData syncData = this.selectByMultiId(new TSyncData(schoolId, examId));
        try {
            if (syncData == null) {
                throw ExceptionResultEnum.ERROR.exception("数据不存在");
            } else if (syncData.getStatus()) {
                throw ExceptionResultEnum.ERROR.exception("数据正在同步中");
            }
            // 更新状态
            syncData.setStatus(true);
            this.updateByMultiId(syncData);
            dataService.syncData(schoolId, examId, syncData);
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            syncData.setStatus(false);
            syncData.setLastSyncTime(System.currentTimeMillis());
            this.updateByMultiId(syncData);
            concurrentService.getReadWriteLock("TEACHCLOUD_SYNC_DATA" + "-" + schoolId + "-" + examId).writeLock().unlock();
        }
        return true;
    }

    @Override
    public IPage<TSyncDataLog> getDataLog(Long schoolId, Long examId, Integer pageNumber, Integer pageSize) {
        Page<TSyncDataLog> page = new Page<>(pageNumber, pageSize);
        QueryWrapper<TSyncDataLog> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(TSyncDataLog::getSchoolId, schoolId)
                .eq(TSyncDataLog::getExamId, examId)
                .orderByDesc(TSyncDataLog::getStartTime);
        return tSyncDataLogService.page(page, queryWrapper);
    }

    @Override
    public IPage<TSyncData> listParam(Long schoolId, Integer pageNumber, Integer pageSize) {
        Page<TSyncData> page = new Page<>(pageNumber, pageSize);
        return this.baseMapper.listParam(page, schoolId);
    }

    @Override
    public boolean enable(TSyncData tSyncData) {
        UpdateWrapper<TSyncData> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(TSyncData::getEnable, tSyncData.getEnable())
                .eq(TSyncData::getSchoolId, tSyncData.getSchoolId())
                .eq(TSyncData::getExamId, tSyncData.getExamId());
        this.update(updateWrapper);
        return true;
    }


}
