package com.qmth.teachcloud.data.util;

import com.alibaba.druid.pool.DruidDataSourceFactory;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Map;

public class DruidJdbcUtils {
    //声明
    private static DataSource ds;

    //构造方法私有化
    private DruidJdbcUtils() {
    }

    //获取DataSource数据源信息（连接池中所有的参数）
    public static DataSource getDataSource(Map map) {
        try {
            //读取德鲁伊的配置文件，让德鲁伊自己封装配置文件参数
            //获取字符输入流
            if (map.isEmpty()) {
                throw ExceptionResultEnum.ERROR.exception("未配置数据库连接参数");
            }
            //创建DruidDataSource（连接池）DruidDataSourceFactory
            ds = DruidDataSourceFactory.createDataSource(map);
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        }
        return ds;
    }

    //获取DataSource数据源信息（连接池的所有参数）
    public static Connection getConnection() {
        //1)从当前线程中获取存储的内容
        Connection conn = null;

        try {
            //2)判断是否获取到连接对象
            if (conn == null) {
                //3）从数据源（连接池中）获取Connection
                conn = ds.getConnection();
            }

        } catch (SQLException e) {
//            e.printStackTrace();
            throw new RuntimeException(e.getMessage());
        }
        return conn;
    }

    //释放资源---在配合dbutils使用时不需要释放资源，dbutils里面会自己释放资源
    public static void close(ResultSet rs, Statement stmt, Connection conn) {
        if (rs != null) {
            try {
                rs.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
        if (stmt != null) {
            try {
                stmt.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
        if (conn != null) {
            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    //主要针对DDL/DML语句操作释放资源---在配合dbutils使用时不需要释放资源，dbutils里面会自己释放资源
    public static void close(Statement stmt, Connection conn) {
        close(null, stmt, conn);
    }

    //开启事务
    public static void setAutoCommit() {
        //获取连接对象
        Connection conn = getConnection();
        //开启手动提交
        try {
            conn.setAutoCommit(false);
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }


    //事务回滚
    public static void rollbackAndClose() throws SQLException {
        //获取连接对象
        Connection conn = getConnection();
        //回滚
        conn.rollback();
        //释放连接对象，从当前线程中解绑
        conn.close();
    }

    //提交事务
    public static void commitAndClose() throws SQLException {
        //获取连接对象
        Connection conn = getConnection();
        //提交事务
        conn.commit();
        //释放资源，从当前线程中解绑
        conn.close();

    }


    public static void main(String[] args) {
//        System.out.println(DruidJdbcUtils.getDataSource());
        System.out.println(DruidJdbcUtils.getConnection());
    }

}
