package com.qmth.teachcloud.mark.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.teachcloud.common.base.BaseEntity;
import com.qmth.teachcloud.mark.enums.AiQuestionParamModeStatus;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.List;

/**
 * <p>
 * AI评卷参数表
 * </p>
 *
 * @author wangliang
 * @since 2025-04-10
 */
@ApiModel(value = "MarkAiQuestionParam对象", description = "AI评卷参数表")
public class MarkAiQuestionParam extends BaseEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "题目id")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long questionId;

    @ApiModelProperty(value = "考试id")
    @JsonSerialize(using = ToStringSerializer.class)
    @NotNull(message = "考试id不能为空")
    private Long examId;

    @ApiModelProperty(value = "科目id")
    @JsonSerialize(using = ToStringSerializer.class)
    @NotNull(message = "科目id不能为空")
    private Long courseId;

    @ApiModelProperty(value = "试卷类型")
    @NotNull(message = "试卷类型不能为空")
    private String paperNumber;

    @ApiModelProperty(value = "评分模式，POINT：按得分点，LEVEL：按档次")
    @NotNull(message = "评分模式不能为空")
    private AiQuestionParamModeStatus mode;

    @ApiModelProperty(value = "试题题干")
    private String mainTitle;

    @ApiModelProperty(value = "最小分")
    private Double minScore;

    @ApiModelProperty(value = "得分点列表")
    @TableField(exist = false)
    List<MarkAiQuestionPoint> pointList;

    @ApiModelProperty(value = "档次列表")
    @TableField(exist = false)
    List<MarkAiQuestionLevel> levelList;

    /**
     * set列表数据
     *
     * @param pointList
     * @param levelList
     */
    public void setListInfo(List<MarkAiQuestionPoint> pointList, List<MarkAiQuestionLevel> levelList) {
        this.pointList = pointList;
        this.levelList = levelList;
    }

    public List<MarkAiQuestionPoint> getPointList() {
        return pointList;
    }

    public void setPointList(List<MarkAiQuestionPoint> pointList) {
        this.pointList = pointList;
    }

    public List<MarkAiQuestionLevel> getLevelList() {
        return levelList;
    }

    public void setLevelList(List<MarkAiQuestionLevel> levelList) {
        this.levelList = levelList;
    }

    public Long getQuestionId() {
        return questionId;
    }

    public void setQuestionId(Long questionId) {
        this.questionId = questionId;
    }

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public Long getCourseId() {
        return courseId;
    }

    public void setCourseId(Long courseId) {
        this.courseId = courseId;
    }

    public String getPaperNumber() {
        return paperNumber;
    }

    public void setPaperNumber(String paperNumber) {
        this.paperNumber = paperNumber;
    }

    public AiQuestionParamModeStatus getMode() {
        return mode;
    }

    public void setMode(AiQuestionParamModeStatus mode) {
        this.mode = mode;
    }

    public String getMainTitle() {
        return mainTitle;
    }

    public void setMainTitle(String mainTitle) {
        this.mainTitle = mainTitle;
    }

    public Double getMinScore() {
        return minScore;
    }

    public void setMinScore(Double minScore) {
        this.minScore = minScore;
    }
}
