package com.qmth.teachcloud.mark.entity;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.teachcloud.mark.dto.mark.SpecialTagDTO;
import com.qmth.teachcloud.mark.dto.mark.manage.TrackDTO;
import com.qmth.teachcloud.mark.enums.MarkTaskStatus;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import org.apache.commons.lang3.StringUtils;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 评卷任务表
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@TableName("mark_task")
@ApiModel(value = "MarkTask对象", description = "评卷任务表")
public class MarkTask implements Serializable {

    private static final long serialVersionUID = 1L;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "主键")
    @TableId(value = "id")
    private Long id;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "考试ID")
    private Long examId;

    @ApiModelProperty(value = "课程ID")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long courseId;

    @ApiModelProperty(value = "课程代码")
    private String courseCode;
    @ApiModelProperty(value = "课程名称")
    private String courseName;
    @ApiModelProperty(value = "试卷编号")
    private String paperNumber;

    @ApiModelProperty(value = "题目ID")
    private Long questionId;

    @ApiModelProperty(value = "大题号")
    private Integer mainNumber;

    @ApiModelProperty(value = "小题号")
    private Integer subNumber;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "考生ID")
    private Long studentId;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "考生管理ID")
    private Long basicStudentId;

    @ApiModelProperty(value = "学号")
    private String studentCode;

    @ApiModelProperty(value = "考生密号")
    private String secretNumber;

    @ApiModelProperty(value = "多评任务序号")
    private Integer taskNumber;

    @ApiModelProperty(value = "状态")
    private MarkTaskStatus status;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "评卷员ID")
    private Long userId;

    @ApiModelProperty(value = "评卷时间")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Long markerTime;

    @ApiModelProperty(value = "评卷总分")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Double markerScore;


    @ApiModelProperty(value = "评卷分数轨迹")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String markerTrackList;

    @ApiModelProperty(value = "评卷特殊标记")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String markerTagList;

    @ApiModelProperty(value = "评卷时长")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Integer markerSpent;

    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty(value = "科组长ID")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Long headerId;

    @ApiModelProperty(value = "科组长评卷时间")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Long headerTime;

    @ApiModelProperty(value = "科组长总分")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Double headerScore;

    @ApiModelProperty(value = "复核轨迹")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String headerTrackList;

    @ApiModelProperty(value = "复核特殊标记")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String headerTagList;

    @ApiModelProperty(value = "打回原因")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String rejectReason;

    @ApiModelProperty(value = "是否AI评卷")
    private Boolean aiMarked;

    @ApiModelProperty(value = "AI评卷错误信息")
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String aiMarkErrorMsg;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public Long getCourseId() {
        return courseId;
    }

    public void setCourseId(Long courseId) {
        this.courseId = courseId;
    }

    public String getCourseCode() {
        return courseCode;
    }

    public void setCourseCode(String courseCode) {
        this.courseCode = courseCode;
    }

    public String getCourseName() {
        return courseName;
    }

    public void setCourseName(String courseName) {
        this.courseName = courseName;
    }

    public String getPaperNumber() {
        return paperNumber;
    }

    public void setPaperNumber(String paperNumber) {
        this.paperNumber = paperNumber;
    }

    public Long getStudentId() {
        return studentId;
    }

    public void setStudentId(Long studentId) {
        this.studentId = studentId;
    }

    public Long getBasicStudentId() {
        return basicStudentId;
    }

    public void setBasicStudentId(Long basicStudentId) {
        this.basicStudentId = basicStudentId;
    }

    public String getStudentCode() {
        return studentCode;
    }

    public void setStudentCode(String studentCode) {
        this.studentCode = studentCode;
    }

    public String getSecretNumber() {
        return secretNumber;
    }

    public void setSecretNumber(String secretNumber) {
        this.secretNumber = secretNumber;
    }

    public Integer getTaskNumber() {
        return taskNumber;
    }

    public void setTaskNumber(Integer taskNumber) {
        this.taskNumber = taskNumber;
    }

    public MarkTaskStatus getStatus() {
        return status;
    }

    public void setStatus(MarkTaskStatus status) {
        this.status = status;
    }

    public Long getUserId() {
        return userId;
    }

    public void setUserId(Long userId) {
        this.userId = userId;
    }

    public Long getMarkerTime() {
        return markerTime;
    }

    public void setMarkerTime(Long markerTime) {
        this.markerTime = markerTime;
    }

    public Double getMarkerScore() {
        return markerScore;
    }

    public void setMarkerScore(Double markerScore) {
        this.markerScore = markerScore;
    }

    public Integer getMarkerSpent() {
        return markerSpent;
    }

    public void setMarkerSpent(Integer markerSpent) {
        this.markerSpent = markerSpent;
    }

    public Long getHeaderId() {
        return headerId;
    }

    public void setHeaderId(Long headerId) {
        this.headerId = headerId;
    }

    public Long getHeaderTime() {
        return headerTime;
    }

    public void setHeaderTime(Long headerTime) {
        this.headerTime = headerTime;
    }

    public Double getHeaderScore() {
        return headerScore;
    }

    public void setHeaderScore(Double headerScore) {
        this.headerScore = headerScore;
    }

    public String getRejectReason() {
        return rejectReason;
    }

    public void setRejectReason(String rejectReason) {
        this.rejectReason = rejectReason;
    }

    public Long getQuestionId() {
        return questionId;
    }

    public void setQuestionId(Long questionId) {
        this.questionId = questionId;
    }

    public Integer getMainNumber() {
        return mainNumber;
    }

    public void setMainNumber(Integer mainNumber) {
        this.mainNumber = mainNumber;
    }

    public Integer getSubNumber() {
        return subNumber;
    }

    public void setSubNumber(Integer subNumber) {
        this.subNumber = subNumber;
    }

    public String getMarkerTrackList() {
        return markerTrackList;
    }

    public void setMarkerTrackList(String markerTrackList) {
        this.markerTrackList = markerTrackList;
    }

    public String getMarkerTagList() {
        return markerTagList;
    }

    public void setMarkerTagList(String markerTagList) {
        this.markerTagList = markerTagList;
    }

    public String getHeaderTrackList() {
        return headerTrackList;
    }

    public void setHeaderTrackList(String headerTrackList) {
        this.headerTrackList = headerTrackList;
    }

    public String getHeaderTagList() {
        return headerTagList;
    }

    public void setHeaderTagList(String headerTagList) {
        this.headerTagList = headerTagList;
    }

    public Boolean getAiMarked() {
        return aiMarked;
    }

    public void setAiMarked(Boolean aiMarked) {
        this.aiMarked = aiMarked;
    }

    public String getAiMarkErrorMsg() {
        return aiMarkErrorMsg;
    }

    public void setAiMarkErrorMsg(String aiMarkErrorMsg) {
        this.aiMarkErrorMsg = aiMarkErrorMsg;
    }

    public List<TrackDTO> listMarkerTrack() {
        List<TrackDTO> markTrackList = new ArrayList<>();
        if (StringUtils.isNotBlank(markerTrackList)) {
            markTrackList = JSON.parseArray(markerTrackList, TrackDTO.class);
        }
        return markTrackList;
    }

    public List<SpecialTagDTO> listMarkerTag() {
        List<SpecialTagDTO> markSpecialTagList = new ArrayList<>();
        if (StringUtils.isNotBlank(markerTagList)) {
            markSpecialTagList = JSON.parseArray(markerTagList, SpecialTagDTO.class);
        }
        return markSpecialTagList;
    }

    public List<TrackDTO> listHeaderTrack() {
        List<TrackDTO> markHeaderTrackList = new ArrayList<>();
        if (StringUtils.isNotBlank(headerTrackList)) {
            markHeaderTrackList = JSON.parseArray(headerTrackList, TrackDTO.class);
        }
        return markHeaderTrackList;
    }

    public List<SpecialTagDTO> listHeaderTag() {
        List<SpecialTagDTO> headerSpecialTagList = new ArrayList<>();
        if (StringUtils.isNotBlank(headerTagList)) {
            headerSpecialTagList = JSON.parseArray(headerTagList, SpecialTagDTO.class);
        }
        return headerSpecialTagList;
    }

    @Override
    public String toString() {
        return "MarkTask{" +
                "id=" + id +
                ", examId=" + examId +
                ", courseCode=" + courseCode +
                ", courseName=" + courseName +
                ", paperNumber=" + paperNumber +
                ", questionId=" + questionId +
                ", studentId=" + studentId +
                ", studentCode=" + studentCode +
                ", secretNumber=" + secretNumber +
                ", taskNumber=" + taskNumber +
                ", status=" + status +
                ", userId=" + userId +
                ", markerTime=" + markerTime +
                ", markerScore=" + markerScore +
                ", markerSpent=" + markerSpent +
                ", headerId=" + headerId +
                ", headerTime=" + headerTime +
                ", headerScore=" + headerScore +
                "}";
    }
}
