package com.qmth.teachcloud.mark.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.bean.dto.mark.MarkSettingDto;
import com.qmth.teachcloud.common.bean.params.mark.setting.MarkPaperSettingParam;
import com.qmth.teachcloud.common.entity.BasicExam;
import com.qmth.teachcloud.common.enums.mark.MarkPaperStatus;
import com.qmth.teachcloud.mark.bean.archivescore.ArchiveScoreQuery;
import com.qmth.teachcloud.mark.bean.archivescore.ArchiveScoreVo;
import com.qmth.teachcloud.mark.bean.document.ArchivePaperQuery;
import com.qmth.teachcloud.mark.bean.document.ArchivePaperVo;
import com.qmth.teachcloud.mark.dto.mark.score.CheckScoreListDto;
import com.qmth.teachcloud.mark.dto.mark.score.MarkPaperPackageDto;
import com.qmth.teachcloud.mark.dto.mark.score.SettingDto;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

/**
 * <p>
 * 考试科目表 服务类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
public interface MarkPaperService extends IService<MarkPaper> {

    IPage<MarkSettingDto> listPaperSetting(Long examId, Long openCollegeId, Long courseId, String paperNumber, Boolean groupStatus, Integer pageNumber, Integer pageSize);

    MarkPaper getByExamIdAndPaperNumber(Long examId, String paperNumber);
    void savePaperSetting(MarkPaper markPaper);

    void saveBatchPaperSetting(MarkPaperSettingParam markPaperSettingParam);

    Boolean finishPaper(BasicExam basicExam, List<String> paperNumbers, MarkPaperStatus status);

    /**
     * 校验试卷
     *
     * @param examId
     * @param paperNumber
     * @param teachClassName
     * @return
     */
    Boolean finishPaper(Long examId, String paperNumber, String teachClassName);

    List<MarkPaper> listQualityMarkPaperByStatus(MarkPaperStatus formal, int uploadCount);

    int getCourseCount(Long id, Long courseId, String coursePaperId, MarkPaperStatus status, DataPermissionRule dpr);

    int getPaperNumberCount(Long id, Long courseId, String coursePaperId, MarkPaperStatus status, DataPermissionRule dpr);

    IPage<CheckScoreListDto> listStudentScoreList(Long examId, Long openCollegeId, Long courseId, String paperNumber,
            Integer pageNumber, Integer pageSize);

    void updateUploadCount(Long examId, String paperNumber, int countUploaded);

    IPage<MarkPaperPackageDto> listPackage(Long examId, String paperNumber, String packageCode, Integer pageNumber,
            Integer pageSize);

    MarkPaper getByExamIdAndCoursePaperId(Long examId, String coursePaperId);

    List<MarkPaper> listByExamId(Long examId, MarkPaperStatus status, DataPermissionRule dpr);

    int countByPropositionTeacherId(boolean status);

    void updateStudentCountByExamIdAndPaperNumber(Long examId, String paperNumber);
    void updateStudentCountByExamIdAndPaperNumberAndSerialNumber(Long examId, String paperNumber, Integer serialNumber);

    SettingDto getSetting(Long examId, String paperNumber);

    IPage<ArchiveScoreVo> scoreList(ArchiveScoreQuery query);

    void deleteByExamIdAndPaperNumber(Long examId, String paperNumber);

    void updateAbsentCount(Long examId, String paperNumber, int absentCount);

    IPage<ArchivePaperVo> documentList(ArchivePaperQuery query);

    /**
     * 根据考试id查询markPaper信息（只查课程信息和试卷编号）
     *
     * @param examId 考试id
     * @return List<MarkPaper>
     */
    List<MarkPaper> findMarkPaperListByExamId(Long examId);

    void importCardJsonData(Long examId, MultipartFile file, String md5);

    List<MarkPaper> listByExamIdAndCourseIdForModel4(Long examId, Long courseId);

    /**
     * 总量分析（成绩查询列表导出）
     *
     * @param query    查询条件
     * @param response response
     */
    void scoreListExport(ArchiveScoreQuery query, HttpServletResponse response) throws IOException;

    void updateQuestionStatus(Long examId, String paperNumber);

    void updateGroupStatus(Long examId, String paperNumber);
}
