package com.qmth.teachcloud.mark.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.teachcloud.common.bean.params.mark.group.QuestionPictureConfigParams;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.mark.bean.mark.DoubleMarkParam;
import com.qmth.teachcloud.mark.dto.mark.MarkPaperFileDto;
import com.qmth.teachcloud.mark.dto.mark.MarkQuestionAnswerVo;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkClassDetailDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkClassProgressDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkGroupQuestionsDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkGroupSummaryProgressDto;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkGroupTaskDto;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkQuestionDto;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkQuestionSubjectiveStepStatusDto;
import com.qmth.teachcloud.mark.params.MarkObjectiveQuestionParams;
import com.qmth.teachcloud.mark.params.MarkQuestionParams;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * <p>
 * 小题信息表 服务类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
public interface MarkQuestionService extends IService<MarkQuestion> {

    /**
     * 查询指定卷型完整试卷结构
     *
     * @param examId
     * @param paperNumber
     * @return
     */
    List<MarkQuestion> listQuestionByExamIdAndPaperNumber(Long examId, String paperNumber);

    void saveQuestions(MarkQuestionParams markQuestionParams);

    /**
     * 客观题标答模板导出
     *
     * @param examId      考试id
     * @param paperNumber 试卷编号
     * @param paperType
     */
    void objectiveAnswerSettingModelExport(Long examId, String paperNumber, String paperType, HttpServletResponse response)
            throws Exception;

    /**
     * 客观题标答导入
     *
     * @param examId      考试id
     * @param paperNumber 试卷编号
     * @param paperType
     * @param file        excel文件
     */
    void objectiveAnswerSettingImport(Long examId, String paperNumber, String paperType, MultipartFile file) throws Exception;

    void saveObjectiveQuestions(MarkObjectiveQuestionParams markObjectiveQuestionParams);

    String uploadSubjectiveFile(Long examId, String paperNumber, String paperType, MultipartFile file, String md5);

    List<MarkPaperFileDto> previewAnswerFileByExamIdAndPaperNumber(Long examId, String paperNumber);

    List<MarkPaperFileDto> previewPaperFileByExamIdAndPaperNumber(Long examId, String paperNumber);

    List<MarkQuestion> listByExamIdAndPaperNumberAndPaperIndexAndPageIndex(Long examId, String paperNumber, Integer paperIndex, Integer pageIndex, boolean isObjective);

    MarkQuestionDto pageQuestionsByExamIdAndPaperNumber(Long examId, String paperNumber);

    List<MarkQuestion> listByExamIdAndPaperNumberAndObjective(Long examId, String paperNumber, Boolean objective);

    void deleteByExamIdAndPaperNumber(Long examId, String paperNumber);

    /**
     * 更新试卷总分
     *
     * @param examId      考试id
     * @param paperNumber 试卷编号
     */
    void updateMarkPaperScore(Long examId, String paperNumber);

    List<MarkQuestionAnswerVo> listQuestionAnswerByExamIdAndPaperNumberAndPaperType(Long examId, String paperNumber, String paperType, Boolean objective);

    List<MarkQuestion> listByExamIdAndPaperNumber(Long examId, String paperNumber);

    void deleteByExamIdAndPaperNumberAndObjective(Long examId, String paperNumber, boolean objective);

    MarkGroupTaskDto listGroupTaskByExamIdAndPaperNumber(Long examId, String paperNumber);

    void updatePicListByQuestionId(QuestionPictureConfigParams groupPictureConfigParams);

    Integer countCurrentCountByExamIdAndPaperNumber(Long examId, String paperNumber);

    MarkGroupSummaryProgressDto summaryGroupProgress(Long examId, String paperNumber);

    IPage<MarkClassProgressDto> summaryGroupClassProgress(Long examId, String paperNumber, String className, Integer pageNumber, Integer pageSize);
    List<MarkClassDetailDto> detailGroupClassProgress(Long examId, String paperNumber, String className);

    List<MarkGroupQuestionsDto> listGroupQuestions(Long examId, String paperNumber);

    void updateTaskCount(Long questionId, int taskCount);

    void updateMarkedCount(Long questionId, int count);

    void updateDoubleMarkByQuestionId(DoubleMarkParam doubleMarkParam);

    MarkQuestionSubjectiveStepStatusDto stepStatus(Long examId, String paperNumber);

    void updatePic(String content, List<MarkQuestion> markQuestions);

    List<MarkQuestion> listByExamIdAndObjective(Long examId, boolean objective);

    int countUnBindMarkerQuestion(Long examId, String paperNumber);
}
