package com.qmth.teachcloud.mark.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.boot.core.ai.client.OcrApiClient;
import com.qmth.boot.core.ai.model.llm.score.AutoScoreRequest;
import com.qmth.boot.core.ai.model.llm.score.AutoScoreResult;
import com.qmth.boot.core.ai.model.llm.score.StandardAnswer;
import com.qmth.boot.core.ai.model.ocr.OcrType;
import com.qmth.boot.core.ai.service.AiService;
import com.qmth.boot.core.retrofit.utils.UploadFile;
import com.qmth.teachcloud.common.bean.dto.mark.PictureConfig;
import com.qmth.teachcloud.common.bean.vo.FilePathVo;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.BasicSchool;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.mapper.BasicCourseMapper;
import com.qmth.teachcloud.common.mapper.BasicSchoolMapper;
import com.qmth.teachcloud.common.service.FileUploadService;
import com.qmth.teachcloud.common.util.FileUtil;
import com.qmth.teachcloud.mark.dto.ai.SheetImageDto;
import com.qmth.teachcloud.mark.entity.*;
import com.qmth.teachcloud.mark.enums.AiQuestionParamModeStatus;
import com.qmth.teachcloud.mark.lock.LockService;
import com.qmth.teachcloud.mark.service.*;
import com.qmth.teachcloud.mark.utils.AiUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

@Service
public class AiMarkServiceImpl implements AiMarkService {

    private final static Logger log = LoggerFactory.getLogger(AiMarkServiceImpl.class);

    private Map<Long, Long> markerLastUpdateTime = new ConcurrentHashMap<>();

    @Resource
    private BasicSchoolMapper basicSchoolMapper;
    @Resource
    private BasicCourseMapper basicCourseMapper;
    @Resource
    private MarkService markService;
    @Resource
    private AiService aiService;
    @Resource
    private FileUploadService fileUploadService;
    @Resource
    private MarkOcrStudentQuestionService markOcrStudentQuestionService;
    @Resource
    private MarkQuestionService markQuestionService;
    @Resource
    private MarkAiQuestionParamService markAiQuestionParamService;
    @Resource
    private MarkAiQuestionPointService markAiQuestionPointService;
    @Resource
    private MarkAiQuestionLevelService markAiQuestionLevelService;
    @Resource
    private MarkUserQuestionService markUserQuestionService;
    @Resource
    private OcrApiClient ocrApiClient;

    @Resource
    private MarkTaskService markTaskService;

    @Resource
    private MarkStudentService markStudentService;

    @Resource
    private MarkProblemHistoryService markProblemHistoryService;

    @Resource
    private MarkArbitrateHistoryService markArbitrateHistoryService;

    @Resource
    private MarkSubjectiveScoreService markSubjectiveScoreService;
    @Resource
    LockService lockService;

    @Override
    public void aiMark(Long examId, String paperNumber, Long questionId) {
        BasicSchool basicSchool = basicSchoolMapper.selectByExamId(examId);
        BasicCourse basicCourse = basicCourseMapper.selectByExamIdAndPaperNumber(examId, paperNumber);
        MarkQuestion markQuestion = markQuestionService.getById(questionId);
        MarkAiQuestionParam markAiQuestionParam = markAiQuestionParamService.getByExamIdAndPaperNumberAndQuestionId(examId, paperNumber, questionId);
        List<MarkTask> markTasks = null;
        int pageNumber = 1;
        do {
            Set<Long> questions = new HashSet<>(Arrays.asList(questionId));
            markTasks = markTaskService.findAiUnMarked(new Page<>(pageNumber, 20), examId, paperNumber, questionId);
            if (markTasks.isEmpty()) {
                break;
            }
            for (MarkTask t : markTasks) {
                if (markService.applyTask(examId, paperNumber, t.getStudentId(), null, questions, Arrays.asList(t.getId()))) {
                    break;
                }
                // 考生作答
                try {
                    String studentQuestionAnswer = this.mergeQuestionOcrResult(this.listStudentQuestionOcrResult(basicSchool, t.getStudentId(), t.getQuestionId()));

                    AutoScoreRequest request = new AutoScoreRequest();
                    request.setSubjectName(basicCourse.getName());
                    request.setTotalScore(markQuestion.getTotalScore());
                    request.setIntervalScore(markAiQuestionParam.getMinScore());
                    request.setQuestionBody(markAiQuestionParam.getMainTitle());
                    request.setStudentAnswer(studentQuestionAnswer);
                    request.setStandardAnswer(buildStandardAnswer(markAiQuestionParam));
                    // AI评卷
                    AutoScoreResult autoScoreResult = aiService.autoScore(request, AiUtil.signature(basicSchool));

                } catch (Exception e) {
                    t.setAiMarkErrorMsg(e.getMessage());
                } finally {
                    t.setAiMarkErrorMsg(null);
                }
            }
            pageNumber++;
            markTasks = markTaskService.findAiUnMarked(new Page<>(pageNumber, 20), examId, paperNumber, questionId);
        } while (CollectionUtils.isNotEmpty(markTasks));
    }

    private List<StandardAnswer> buildStandardAnswer(MarkAiQuestionParam markAiQuestionParam) {
        List<StandardAnswer> standardAnswerList = new ArrayList<>();
        // 得分点
        if (AiQuestionParamModeStatus.POINT.equals(markAiQuestionParam.getMode())) {
            List<MarkAiQuestionPoint> markAiQuestionPoints = markAiQuestionPointService.listByAiQuestionId(markAiQuestionParam.getId());
            if (CollectionUtils.isEmpty(markAiQuestionPoints)) {
                throw ExceptionResultEnum.ERROR.exception("未设置得分点");
            }
            standardAnswerList = markAiQuestionPoints.stream().map(m -> {
                StandardAnswer standardAnswer = new StandardAnswer();
                standardAnswer.setScore(m.getScore());
                standardAnswer.setContent(m.getAnswer());
                return standardAnswer;
            }).collect(Collectors.toList());
        }
        // 档次
        else if (AiQuestionParamModeStatus.LEVEL.equals(markAiQuestionParam.getMode())) {
            List<MarkAiQuestionLevel> markAiQuestionLevels = markAiQuestionLevelService.listByAiQuestionId(markAiQuestionParam.getId());
            if (CollectionUtils.isEmpty(markAiQuestionLevels)) {
                throw ExceptionResultEnum.ERROR.exception("未设置档次");
            }
            standardAnswerList = markAiQuestionLevels.stream().map(m -> {
                StandardAnswer standardAnswer = new StandardAnswer();
                standardAnswer.setLowScore(m.getMinScore());
                standardAnswer.setHighScore(m.getMaxScore());
                standardAnswer.setContent(m.getAnswer());
                return standardAnswer;
            }).collect(Collectors.toList());
        } else {
            throw ExceptionResultEnum.ERROR.exception("评分模式不存在");
        }
        return standardAnswerList;
    }

    private String mergeQuestionOcrResult(List<MarkOcrStudentQuestion> markOcrStudentQuestionList) {
        if (CollectionUtils.isEmpty(markOcrStudentQuestionList)) {
            return null;
        }
        return markOcrStudentQuestionList.stream().filter(m -> StringUtils.isNotBlank(m.getOcrContent())).map(MarkOcrStudentQuestion::getOcrContent).collect(Collectors.joining(SystemConstant.COMMA_OF_CHINESE));
    }

    @Override
    public List<MarkOcrStudentQuestion> listStudentQuestionOcrResult(BasicSchool basicSchool, Long studentId, Long questionId) {
        List<MarkOcrStudentQuestion> markOcrStudentQuestionList = markOcrStudentQuestionService.listByStudentIdAndQuestionId(studentId, questionId);
        if (CollectionUtils.isNotEmpty(markOcrStudentQuestionList)) {
            return markOcrStudentQuestionList;
        }
        MarkQuestion markQuestion = markQuestionService.getById(questionId);
        if (markQuestion == null || StringUtils.isBlank(markQuestion.getPicList())) {
            throw ExceptionResultEnum.ERROR.exception("大题号[" + markQuestion.getMainNumber() + "]、小题号[" + markQuestion.getSubNumber() + "]未设置评卷区");
        }
        // 读取考生图片
        MarkStudent markStudent = markStudentService.getById(studentId);
        if (markStudent == null) {
            throw ExceptionResultEnum.ERROR.exception("未找到考生" + "，考生ID[" + studentId);
        } else if (StringUtils.isBlank(markStudent.getSheetPath())) {
            throw ExceptionResultEnum.ERROR.exception("未查询到原图" + "，考号[" + markStudent.getStudentCode());
        }
        List<File> deleteFileList = new ArrayList<>();
        //考生题卡原图
        try {
            List<FilePathVo> filePathVoList = markStudent.listSheetPath();
            Map<Integer, SheetImageDto> fileMap = new HashMap<>();
            for (int i = 0; i < filePathVoList.size(); i++) {
                FilePathVo filePathVo = filePathVoList.get(i);
                File file = SystemConstant.getFileTempDirVar(SystemConstant.JPG_PREFIX);
                try {
                    fileUploadService.downloadFile(filePathVo.getPath(), filePathVo.getUploadType(), filePathVo.getType(), file.getPath());
                } catch (Exception e) {
                    throw ExceptionResultEnum.ERROR.exception("读取考生原图失败" + "，考号[" + markStudent.getStudentCode() + "。" + e.getMessage());
                }
                fileMap.put(i + 1, new SheetImageDto(i + 1, file, filePathVo.getMd5(), AiUtil.imageDim(file)));
                deleteFileList.add(file);
            }

            // 题目评卷区
            List<PictureConfig> pictureConfigList = JSON.parseArray(markQuestion.getPicList(), PictureConfig.class);

            List<MarkOcrStudentQuestion> markOcrStudentQuestions = new ArrayList<>();
            for (int i = 0; i < pictureConfigList.size(); i++) {
                PictureConfig pictureConfig = pictureConfigList.get(i);
                SheetImageDto sheetImageDto = fileMap.get(pictureConfig.getI());
                File file = SystemConstant.getFileTempDirVar(SystemConstant.JPG_PREFIX);
                AiUtil.subImg(sheetImageDto, pictureConfig, file);

                // OCR识别
                String ocrResult;
                try {
                    ocrResult = ocrApiClient.forImage(AiUtil.signature(basicSchool), OcrType.HANDWRITING, UploadFile.build("image", "", file));
                } catch (Exception e) {
                    throw ExceptionResultEnum.ERROR.exception("OCR识别失败" + "，考号[" + markStudent.getStudentCode() + "。" + e.getMessage());
                }
                MarkOcrStudentQuestion markOcrStudentQuestion = new MarkOcrStudentQuestion();
                markOcrStudentQuestion.setId(SystemConstant.getDbUuid());
                markOcrStudentQuestion.setExamId(markStudent.getExamId());
                markOcrStudentQuestion.setPaperNumber(markStudent.getPaperNumber());
                markOcrStudentQuestion.setStudentId(studentId);
                markOcrStudentQuestion.setQuestionId(questionId);
                markOcrStudentQuestion.setNumber(i + 1);
                markOcrStudentQuestion.setMd5(sheetImageDto.getMd5());
                markOcrStudentQuestion.setOcrContent(ocrResult);
                markOcrStudentQuestion.setCreateTime(System.currentTimeMillis());
                markOcrStudentQuestions.add(markOcrStudentQuestion);
                deleteFileList.add(file);
            }
            markOcrStudentQuestionService.saveBatch(markOcrStudentQuestions);
            return markOcrStudentQuestions;
        } catch (RuntimeException e) {
            throw ExceptionResultEnum.ERROR.exception(e.getMessage());
        } finally {
            if (CollectionUtils.isNotEmpty(deleteFileList)) {
                for (File file : deleteFileList) {
                    FileUtil.deleteFile(file);
                }
            }
        }
    }

}
