package com.qmth.teachcloud.mark.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.mark.entity.MarkAiQuestionLevel;
import com.qmth.teachcloud.mark.entity.MarkAiQuestionParam;
import com.qmth.teachcloud.mark.entity.MarkAiQuestionPoint;
import com.qmth.teachcloud.mark.enums.AiQuestionParamModeStatus;
import com.qmth.teachcloud.mark.mapper.MarkAiQuestionParamMapper;
import com.qmth.teachcloud.mark.service.MarkAiQuestionLevelService;
import com.qmth.teachcloud.mark.service.MarkAiQuestionParamService;
import com.qmth.teachcloud.mark.service.MarkAiQuestionPointService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * AI评卷参数表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2025-04-10
 */
@Service
public class MarkAiQuestionParamServiceImpl extends ServiceImpl<MarkAiQuestionParamMapper, MarkAiQuestionParam> implements MarkAiQuestionParamService {

    @Resource
    private MarkAiQuestionPointService markAiQuestionPointService;
    @Resource
    private MarkAiQuestionLevelService markAiQuestionLevelService;

    @Override
    public boolean existMarkAiQuestionPointOrLevel(Long examId, String paperNumber, Long questionId) {
        MarkAiQuestionParam markAiQuestionParam = this.getByExamIdAndPaperNumberAndQuestionId(examId, paperNumber, questionId);
        if (markAiQuestionParam == null) {
            return false;
        }
        // 按得分点
        if (AiQuestionParamModeStatus.POINT.equals(markAiQuestionParam.getMode())) {
            List<MarkAiQuestionPoint> markAiQuestionPoints = markAiQuestionPointService.listByAiQuestionId(markAiQuestionParam.getId());
            return CollectionUtils.isNotEmpty(markAiQuestionPoints);
        }
        // 按档次
        if (AiQuestionParamModeStatus.LEVEL.equals(markAiQuestionParam.getMode())) {
            List<MarkAiQuestionLevel> markAiQuestionLevels = markAiQuestionLevelService.listByAiQuestionId(markAiQuestionParam.getId());
            return CollectionUtils.isNotEmpty(markAiQuestionLevels);
        }
        return false;
    }

    @Override
    public MarkAiQuestionParam getByExamIdAndPaperNumberAndQuestionId(Long examId, String paperNumber, Long questionId) {
        QueryWrapper<MarkAiQuestionParam> queryWrapper = new QueryWrapper<>();
        if (Objects.nonNull(examId)) {
            queryWrapper.lambda().eq(MarkAiQuestionParam::getExamId, examId);
        }
        if (Objects.nonNull(paperNumber)) {
            queryWrapper.lambda().eq(MarkAiQuestionParam::getPaperNumber, paperNumber);
        }
        queryWrapper.lambda().eq(MarkAiQuestionParam::getQuestionId, questionId);
        return this.getOne(queryWrapper);
    }
}
