package com.qmth.teachcloud.mark.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.enums.ObjectivePolicy;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.entity.MarkQuestionAnswer;
import com.qmth.teachcloud.mark.entity.ScanAnswerCard;
import com.qmth.teachcloud.mark.enums.CardSource;
import com.qmth.teachcloud.mark.mapper.MarkQuestionAnswerMapper;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import com.qmth.teachcloud.mark.service.MarkQuestionAnswerService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.mark.service.MarkQuestionService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 客观题信息 服务实现类
 * </p>
 *
 * @author xf
 * @since 2024-10-16
 */
@Service
public class MarkQuestionAnswerServiceImpl extends ServiceImpl<MarkQuestionAnswerMapper, MarkQuestionAnswer> implements MarkQuestionAnswerService {

    @Resource
    private MarkPaperService markPaperService;

    @Override
    public void deleteByExamIdAndPaperNumberAndMainNumberAndSubNumber(Long examId, String paperNumber, Integer mainNumber, Integer subNumber) {
        UpdateWrapper<MarkQuestionAnswer> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkQuestionAnswer::getExamId, examId)
                .eq(MarkQuestionAnswer::getPaperNumber, paperNumber)
                .eq(MarkQuestionAnswer::getMainNumber, mainNumber)
                .eq(MarkQuestionAnswer::getSubNumber, subNumber);
        this.remove(updateWrapper);
    }

    @Override
    public void resetObjectivePolicyByExamIdAndPaperNumberAndMainNumberAndSubNumber(Long examId, String paperNumber, Integer mainNumber, Integer subNumber) {
        UpdateWrapper<MarkQuestionAnswer> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkQuestionAnswer::getAnswer, null)
                .set(MarkQuestionAnswer::getObjectivePolicy, null)
                .set(MarkQuestionAnswer::getObjectivePolicyScore, null)
                .eq(MarkQuestionAnswer::getExamId, examId)
                .eq(MarkQuestionAnswer::getPaperNumber, paperNumber)
                .eq(MarkQuestionAnswer::getMainNumber, mainNumber)
                .eq(MarkQuestionAnswer::getSubNumber, subNumber);
        this.update(updateWrapper);
    }

    @Override
    public List<MarkQuestionAnswer> listByExamIdAndPaperNumberAndPaperType(Long examId, String paperNumber, String paperType) {
        return this.baseMapper.listByExamIdAndPaperNumberAndPaperType(examId, paperNumber, paperType);
    }

    @Override
    public int countByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkQuestionAnswer> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkQuestionAnswer::getExamId, examId)
                .eq(MarkQuestionAnswer::getPaperNumber, paperNumber);
        return this.count(queryWrapper);
    }

    @Override
    public void deleteByExamIdAndPaperNumber(Long examId, String paperNumber) {
        UpdateWrapper<MarkQuestionAnswer> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkQuestionAnswer::getExamId, examId)
                .eq(MarkQuestionAnswer::getPaperNumber, paperNumber);
        this.remove(updateWrapper);
    }

    @Override
    public void saveByMarkQuestion(Long examId, String paperNumber, List<MarkQuestion> markQuestions, boolean containAnswer) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        List<MarkQuestionAnswer> markQuestionAnswers = new ArrayList<>();
        for (String paperType : markPaper.getPaperTypeList()) {
            List<MarkQuestionAnswer> markQuestionAnswerList = this.listByExamIdAndPaperNumberAndPaperType(examId, paperNumber, paperType);
            for (MarkQuestion markQuestion : markQuestions) {
                if (!markQuestion.getObjective()) {
                    continue;
                }
                MarkQuestionAnswer questionAnswer = markQuestionAnswerList.stream().filter(m -> m.getMainNumber() != null && m.getMainNumber().equals(markQuestion.getMainNumber()) && m.getSubNumber().equals(markQuestion.getSubNumber())).
                        findFirst().orElse(null);
                if (questionAnswer == null) {
                    MarkQuestionAnswer markQuestionAnswer = new MarkQuestionAnswer(examId, paperNumber, paperType, markQuestion.getMainNumber(), markQuestion.getSubNumber());
                    // 添加答案
                    if (containAnswer) {
                        markQuestionAnswer.setAnswer(markQuestion.getAnswer());
                        markQuestionAnswer.setObjectivePolicy(ObjectivePolicy.NONE);
                    }
                    markQuestionAnswers.add(markQuestionAnswer);
                }
            }
        }
        this.saveBatch(markQuestionAnswers);
    }
}
