package com.qmth.teachcloud.mark.service.impl;

import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.entity.MarkUserQuestion;
import com.qmth.teachcloud.mark.enums.LockType;
import com.qmth.teachcloud.mark.lock.LockService;
import com.qmth.teachcloud.mark.service.MarkService;
import com.qmth.teachcloud.mark.service.MarkSyncService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

@Service
public class MarkSyncServiceImpl implements MarkSyncService {

    @Resource
    private MarkService markService;
    @Resource
    private LockService lockService;

    /**
     * 评卷员重置
     *
     * @param markUserQuestion
     */
    @Async
    @Override
    public void markerResetSync(MarkUserQuestion markUserQuestion) {
        try {
            if (markUserQuestion != null) {
                lockService.waitlock(LockType.EXAM_SUBJECT, markUserQuestion.getExamId(), markUserQuestion.getPaperNumber());
                lockService.waitlock(LockType.MARK_USER_QUESTION, markUserQuestion.getId());
                lockService.watch(LockType.QUESTION, markUserQuestion.getQuestionId());
                markService.resetMarker(markUserQuestion);
                markService.resetMarkedQuestionId(markUserQuestion);
            }
        } catch (Exception e) {
            log.error("reset marker error", e);
        } finally {
            lockService.unwatch(LockType.QUESTION, markUserQuestion.getQuestionId());
            lockService.unlock(LockType.MARK_USER_QUESTION, markUserQuestion.getId());
            lockService.unlock(LockType.EXAM_SUBJECT, markUserQuestion.getExamId(), markUserQuestion.getPaperNumber());
            lockService.unlock(LockType.MARKER_RESET, markUserQuestion.getId());
        }
    }

    @Async
    @Override
    public void updateQuality(List<MarkUserQuestion> markUserQuestions, String lockKey) {
        if (CollectionUtils.isNotEmpty(markUserQuestions)) {
            for (MarkUserQuestion markUserQuestion : markUserQuestions) {
                try {
                    markService.updateQuality(markUserQuestion);
                } catch (Exception e) {
                    log.error("marker quality thread error", e);
                }
            }
        }
        lockService.unlock(LockType.BATCH_QUALITY, lockKey);
    }


    @Async
    @Override
    public void calcObjectiveScore(MarkPaper markPaper) {
        try {
            if (markPaper != null) {
                markService.calcObjectiveScore(markPaper);
            }
        } catch (Exception e) {
            log.error("reset marker error", e);
        } finally {
            lockService.unlock(LockType.SCORE_CALCULATE, markPaper.getId());
        }
    }

    @Async
    @Override
    public void deleteMarkedByQuestion(MarkQuestion markQuestion, boolean b) {
        if (markQuestion == null) {
            return;
        }
        try {
            lockService.waitlock(LockType.EXAM_SUBJECT, markQuestion.getExamId(), markQuestion.getPaperNumber());
            lockService.waitlock(LockType.QUESTION, markQuestion.getId());
            markService.deleteMarkTask(markQuestion, b);
        } catch (Exception e) {
            log.error("delete group error", e);
        } finally {
            lockService.unlock(LockType.QUESTION, markQuestion.getId());
            lockService.unlock(LockType.EXAM_SUBJECT, markQuestion.getExamId(), markQuestion.getPaperNumber());
            lockService.unlock(LockType.QUESTION_UPDATE, markQuestion.getId());
        }
    }
}
