package com.qmth.teachcloud.mark.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.bean.dto.mark.MarkUser;
import com.qmth.teachcloud.common.bean.params.mark.group.MarkClassUserParams;
import com.qmth.teachcloud.common.bean.params.mark.group.MarkGroupClassUserParams;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.MarkQuestion;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.mark.bean.student.MarkStudentQuery;
import com.qmth.teachcloud.mark.dto.mark.MarkStudentVo;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkClassProgressDto;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkUserClassDetailDto;
import com.qmth.teachcloud.mark.dto.mark.setting.MarkUserClassDto;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.entity.MarkUserClass;
import com.qmth.teachcloud.mark.entity.MarkUserQuestion;
import com.qmth.teachcloud.mark.mapper.MarkUserClassMapper;
import com.qmth.teachcloud.mark.service.*;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 评卷员班级 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkUserClassServiceImpl extends ServiceImpl<MarkUserClassMapper, MarkUserClass> implements MarkUserClassService {

    @Resource
    private MarkPaperService markPaperService;
    @Resource
    MarkUserQuestionService markUserQuestionService;

    @Resource
    MarkStudentService markStudentService;

    @Resource
    MarkQuestionService markQuestionService;

    @Transactional
    @Override
    public void updateOpenMarkClassByExamIdAndPaperNumber(Long examId, String paperNumber, Boolean classMark) {
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkPaper::getClassMark, classMark)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        markPaperService.update(updateWrapper);

        if (classMark != null) {
            // 开启分班阅，根据任课老师自动绑定班级
            if (classMark) {
                this.autoBindTeacherIdAndTeachClassName(examId, paperNumber);
            } else {
                // 禁用，删除分班数据
                this.deleteByExamIdAndPaperNumber(examId, paperNumber);
            }
        }
    }

    @Override
    public void deleteByExamIdAndPaperNumber(Long examId, String paperNumber) {
        UpdateWrapper<MarkUserClass> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(MarkUserClass::getExamId, examId)
                .eq(MarkUserClass::getPaperNumber, paperNumber);
        this.remove(updateWrapper);
    }

    @Override
    public MarkUserClassDto listMarkUserClassByExamIdAndPaperNumber(Long examId, String paperNumber) {
        // 课程下所有班级数据
        List<String> paperNumberClassList = markStudentService.listClassByExamIdAndPaperNumber(examId, paperNumber);
        List<MarkUser> markUserList = markUserQuestionService.listUserByExamIdAndPaperNumber(examId, paperNumber);

        MarkUserClassDto markUserClassDto = new MarkUserClassDto();
        markUserClassDto.setClassNames(paperNumberClassList);

        List<MarkUserClassDetailDto> markUserClassDetailDtos = new ArrayList<>();
        for (MarkUser marker : markUserList) {
            MarkUserClassDetailDto markUserClassDetailDto = new MarkUserClassDetailDto();
            markUserClassDetailDto.setMarker(marker);
            List<MarkUserClass> markUserClasses = this.baseMapper.listMarkerClassByExamIdAndPaperNumberAndUserId(examId, paperNumber, marker.getUserId());
            // 剔除掉已经更名的班级
            List<String> markUserClassSelectList = markUserClasses.stream().map(MarkUserClass::getClassName).collect(Collectors.toList());
            List<String> intersectionList = (List<String>) CollectionUtils.intersection(paperNumberClassList, markUserClassSelectList);
            markUserClassDetailDto.setMarkerClassList(intersectionList);
            markUserClassDetailDtos.add(markUserClassDetailDto);
        }
        markUserClassDto.setMarkerClass(markUserClassDetailDtos);
        return markUserClassDto;
    }

    @Override
    public List<MarkUser> listClassMarkerByExamIdAndPaperNumberAndQuestionIdAndClassName(Long examId, String paperNumber, Long questionId, String className) {
        return this.baseMapper.listClassMarkerByExamIdAndPaperNumberAndQuestionIdAndClassName(examId, paperNumber, questionId, className);
    }

    @Transactional
    @Override
    public boolean saveMarkClassUser(MarkClassUserParams markClassUserParams) {
        Long examId = markClassUserParams.getExamId();
        String paperNumber = markClassUserParams.getPaperNumber();

        // 校验班级是否绑定完成
        List<String> listUserClass = markStudentService.listClassByExamIdAndPaperNumber(examId, paperNumber);
        if (CollectionUtils.isNotEmpty(listUserClass)) {
            List<MarkQuestion> markQuestionList = markQuestionService.listByExamIdAndPaperNumberAndObjective(examId, paperNumber, false);
            for (MarkQuestion markQuestion : markQuestionList) {
                Set<String> groupBindClassSet = new HashSet<>();
                List<MarkUserQuestion> markUserQuestionList = markUserQuestionService.listByExamIdAndPaperNumberAndQuestionId(examId, paperNumber, markQuestion.getId());
                for (MarkGroupClassUserParams groupUserClassInfo : markClassUserParams.getQuestionMarkerClass()) {
                    if (markUserQuestionList.stream().filter(m -> m.getUserId().equals(groupUserClassInfo.getMarker().getUserId())).count() > 0) {
                        groupBindClassSet.addAll(groupUserClassInfo.getMarkerClassList());
                    }
                }

                Collection<String> subtractList = CollectionUtils.subtract(listUserClass, groupBindClassSet);
                if (CollectionUtils.isNotEmpty(subtractList)) {
                    throw ExceptionResultEnum.ERROR.exception(String.format("班级[%s]在评阅题目[%s]中未被任何评卷员绑定", String.join(",", subtractList), markQuestion.getQuestionNumber()));
                }
            }
        }

        List<MarkUserClass> markUserClasses = new ArrayList<>();
        for (MarkGroupClassUserParams groupUserClassInfo : markClassUserParams.getQuestionMarkerClass()) {
            for (String className : groupUserClassInfo.getMarkerClassList()) {
                MarkUserClass markUserClass = new MarkUserClass();
                markUserClass.setId(SystemConstant.getDbUuid());
                markUserClass.setExamId(examId);
                markUserClass.setPaperNumber(paperNumber);
                markUserClass.setClassName(className);
                markUserClass.setUserId(groupUserClassInfo.getMarker().getUserId());
                markUserClasses.add(markUserClass);
            }
        }
        // 科目下所有数据同时提交
        this.deleteByExamIdAndPaperNumberAndClassName(examId, paperNumber, null);
        this.saveBatch(markUserClasses);
        return true;
    }

    @Override
    public boolean deleteByExamIdAndPaperNumberAndClassName(Long examId, String paperNumber, String className) {
        UpdateWrapper<MarkUserClass> updateWrapper = new UpdateWrapper<>();
        LambdaUpdateWrapper<MarkUserClass> lambdaUpdateWrapper = updateWrapper.lambda();
        lambdaUpdateWrapper.eq(MarkUserClass::getExamId, examId)
                .eq(MarkUserClass::getPaperNumber, paperNumber);
        if (StringUtils.isNotBlank(className)) {
            lambdaUpdateWrapper.eq(MarkUserClass::getClassName, className);
        }
        return this.remove(updateWrapper);
    }

    @Override
    public List<MarkUserClass> listByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkUserClass> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkUserClass::getExamId, examId)
                .eq(MarkUserClass::getPaperNumber, paperNumber);
        return this.list(queryWrapper);
    }

    @Override
    public IPage<MarkClassProgressDto> pageClassByExamIdAndPaperNumber(Page<MarkClassProgressDto> page, Long examId, String paperNumber, String className) {
        return this.baseMapper.pageClassByExamIdAndPaperNumber(page, examId, paperNumber, className);
    }

    @Override
    public List<MarkUserClass> listByExamIdAndPaperNumberAndUserId(Long examId, String paperNumber, Long userId) {
        QueryWrapper<MarkUserClass> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkUserClass::getExamId, examId)
                .eq(MarkUserClass::getPaperNumber, paperNumber)
                .eq(MarkUserClass::getUserId, userId);
        return this.list(queryWrapper);
    }

    @Override
    public void autoBindTeacherIdAndTeachClassName(Long examId, String paperNumber) {
        MarkStudentQuery markStudentQuery = new MarkStudentQuery();
        markStudentQuery.setExamId(examId);
        markStudentQuery.setPaperNumber(paperNumber);
        List<MarkStudentVo> markStudentList = markStudentService.listMarkStudentVo(markStudentQuery);
        Map<String, Long> map = markStudentList.stream().filter(m -> m.getTeacherId() != null).collect(Collectors.toMap(m -> m.getTeachClassName(), n -> n.getTeacherId(), (m, n) -> m));
        if (MapUtils.isNotEmpty(map)) {

            List<MarkUserClass> markUserClassList = new ArrayList<>();
            for (Map.Entry<String, Long> entry : map.entrySet()) {
                if (CollectionUtils.size(markUserQuestionService.listByExamIdAndPaperNumberAndUserIdAndEnableTure(examId, paperNumber, entry.getValue())) > 0) {
                    MarkUserClass markUserClass = new MarkUserClass();
                    markUserClass.setId(SystemConstant.getDbUuid());
                    markUserClass.setExamId(examId);
                    markUserClass.setPaperNumber(paperNumber);
                    markUserClass.setClassName(entry.getKey());
                    markUserClass.setUserId(entry.getValue());
                    markUserClassList.add(markUserClass);
                }
            }
            this.saveBatch(markUserClassList);
        }
    }
}
